! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! Producto matriz banda comprimida por vector
!
! Construye un ejemplo simple: una matriz A de 5 por 5, con una
! sub-diagonal (m1 = 1) y con dos supra-diagonales (m2 = 2), pero
! es guardada en un formato comprimido  A_c de 5 filas y 3 columnas,
! una por cada diagonal.
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! INTEL:
!  ifort -vms -c banda_compr.f90
!  ifort -vms -o banda_compr.exe1 *.o
!
! GNU:   
!  g95 -w -c banda_compr.f90 --free-form
!  g95 -w -o banda_compr.exe2 banda_compr.o
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program  banda_compr
  implicit none
  !
  integer, parameter :: n  = 5 ! nro de filas
  integer, parameter :: m1 = 1 ! nro de sub-diagonales
  integer, parameter :: m2 = 2 ! nro de supra-diagonales
  !
  integer, dimension (:,:), allocatable :: a
  integer, dimension   (:), allocatable :: x, b
  integer :: i, j, k, m, ier
  !
  ! nro de columnas de la matriz "comprimida"
  m = m1 + 1 + m2 ! nro de: subdiagonales, ppal y supradiagonales
  !
  ! aloca
  allocate ( a (n,m), x (n), b (n), stat = ier)
  if (ier .ne. 0)  stop " aloca "
  !
  ! inicializa
  k = 0
  ii : do  i = 1, n
    jj : do  j = 1, m
    a (i,j) = k
    k = k + 1
    end do jj
  end do ii
  a (1,1) = 0
  k = 0
  do  i = n-1, n
   do  j = i-k, m ; a (i,j) = 0 ; end do
   k = k + 2
  end do
  forall (i=1:n)  
    x (i) = n - i + 1
    b (i) = 0
  end forall
  !
  ! eco
  write (*,*)
  write (*,*) "nro de filas matriz comprimida ;  n = ", n  
  write (*,*) "nro de colu. matriz comprimida ;  m = ", m  
  write (*,*) "nro de   sub-diagonales        ; m1 = ", m1 
  write (*,*) "nro de supra-diagonales        ; m2 = ", m2  
  call eco2 (a, "matriz A en formato comprimido")
  call eco1 (x, "vector X a multiplicar")
  !
  ! tarea
  call cbanda1 (a, m1, m2, x, b)
  call cbanda2 (a, m1, m2, x, b)
  !
  contains
  !
  ! ----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
  ! tarea matricial en una unica instruccion
  subroutine cbanda1 (a, m1, m2, x, b)
   implicit none
   integer, dimension (:,:), intent (inout) :: a
   integer                 , intent (in)    :: m1, m2
   integer, dimension (:)  , intent (in)    :: x
   integer, dimension (:)  , intent (out)   :: b
   character (64) :: cad
   integer :: m, n
   !
   ! tamanios y control
   cad = "error (cbanda): m <> m1+m2+1 "
   n = size (a,1)
   m = siza2 ( size (a,2), m1+m2+1, cad)
   !
   ! tarea matricial
   b = sum (a * eoshift ( spread (x, dim = 2, ncopies = m), &
                          dim = 1,                          &
                          shift = prog_arit (-m1,m,1)       &
                        ), dim = 2)
   call  eco1 (b, "vector B con el producto matriz-vector")
   !
  end subroutine
  ! ----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
  ! tarea matricial en varias instrucciones para ver etapas
  subroutine cbanda2 (a, m1, m2, x, b)
   implicit none
   integer, dimension (:,:), intent (inout) :: a
   integer                 , intent (in)    :: m1, m2
   integer, dimension (:)  , intent (in)    :: x
   integer, dimension (:)  , intent (out)   :: b
   integer, dimension (size(a,1),size(a,2)) :: y
   integer, dimension (size(a,2)) :: u
   character (64) :: cad
   integer :: m, n, i
   !
   ! tamanios y control
   cad = "error (cbanda): m <> m1+m2+1 "
   n = size (a,1)
   m = siza2 ( size (a,2), m1+m2+1, cad)
   !
   y = spread (x, dim = 2, ncopies = m)
   call  eco2 (y, "matriz Y con el vector X expandido")
   !
   u = prog_arit (-m1,m,1)
   call  eco1 (u, "vector U con la progresion aritmetica")
   !
   y = eoshift (y, dim = 1, shift = u)
   call  eco2 (y, "matriz Y con el vector X corrido segun U")
   !
   b = sum (a * y, dim = 2)
   call  eco1 (b, "vector B con el producto matriz-vector")
   !
  end subroutine
  ! ----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
  ! genera "ncuantos" elementos de la progresion aritmetica que 
  ! empieza en "ini" y de incremento "delta"
  ! ----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
  function prog_arit (ini, ncuantos, delta)
   integer      , intent (in) :: ini, ncuantos, delta
   integer, dimension (ncuantos) :: prog_arit
   integer :: k
   prog_arit = 0 
   if (ncuantos > 0) prog_arit (1) = ini
   do  k = 2, ncuantos
     prog_arit (k) = prog_arit (k-1) + delta
   end do
  end function
  !-----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
  ! control coherencia de las dimensiones de 2 arreglos matriciales
  integer function siza2 (n1, n2, cad)
   integer      , intent (in) :: n1, n2
   character (*), intent (in) :: cad
   if ( n1 .ne. n2 ) then
    write (*,*)
    write (*,'(a)') cad
    stop
   end if
   siza2 = n1
  end function
  !-----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
  ! impresion
  subroutine eco1 (t, s)
   integer, dimension (:), intent (in) :: t
   character (*)         , intent (in) :: s
   write (*,*)
   write (*,*) s
   write (*,100) t
   read (*,*)
   100 format (20 (1x, i3) )
  end subroutine
  !-----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
  ! impresion
  subroutine eco2 (t, s)
   integer, dimension (:,:), intent (in) :: t
   character (*)           , intent (in) :: s
   integer :: i, m, n
   m = size (t,1)
   n = size (t,2)
   write (*,*)
   write (*,*) s
   do  i = 1, m
   write (*,100) t (i,:)
   end do
   read (*,*)
   100 format (20 (1x, i3) )
  end subroutine
  !-----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
