! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Problema del Circulo
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! INTEL:
!  ifort -vms -c m_ctes.f90
!  ifort -vms -c m_temps.f90
!  ifort -vms -c m_tools.f90
!  ifort -vms -c circulo.f90
!  ifort -vms -o circulo.exe1 *.o
!
! GNU:
!  g95 -c m_ctes.f90
!  g95 -c m_temps.f90
!  g95 -c m_tools.f90
!  g95 -c circulo.f90
!  g95 -o circulo.exe2 *.o
!
! ADAPTOR + MPICH:
!  adaptor -hpf -dm -free -keep -c m_ctes.f90
!  adaptor -hpf -dm -free -keep -c m_temps.f90
!  adaptor -hpf -dm -free -keep -c m_tools.f90
!  adaptor -hpf -dm -free -keep -c circulo.f90
!  adaptor -hpf -dm -free -o circulo.exe3 *.o
!
! Running with MPICH:
!  mpdboot -n 22 -f ~/mpd.hosts
!  mpdtrace
!  mpdringtest
!  mpiexec -machinefile ~/machi.dat -np 21 circulo.exe3
!  mpiexec -machinefile ~/machi.dat -1 -np 4 circulo.exe3
!  mpdallexit
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program circulo
 use m_ctes
 use m_temps
 use m_tools
 implicit none
 integer (iin) :: h, i, j, k = 0, n
 integer (iin), dimension (:), allocatable :: v
 real    (idp), dimension (:), allocatable :: r
 !hpf$ distribute (block) :: v
 !hpf$ align r (i) with v (i)
 integer (iin), dimension (1) :: s
 !
 !carteles
 write (*,*)
 write (*,*) "problema del circulo"
 write (*,*) "exponente (0 <= h <= 20) h : ? " ; read (*,*) h
 !
 n = 2**h ! una potencia de 2
 write (*,*) "n = 2^h = ", n
 !
 !un control para verificar que "n" sea una potencia de 2
 if (iand(n,n-1).ne. 0) call errata ("n no es una potencia de 2")
 !
 !aloca: guarda con el orden "v" es la referencia de "r" en ALIGN
 ier = 0
 allocate (v (n), stat = ier (2) )
 allocate (r (n), stat = ier (1) )
 if ( any (ier .ne. 0) ) call errata (ier(1:2), "circulo: aloca")
 !
 !inicia cronometro;  
 time = cero
 call system_clock (count = h1)   ! t1 = second ()
 !
 !genera una lista random
 call random_number (r)
 v = floor (100 * r)
 ! if (n < 100) write (*,*) "v", v
 !
 !tarea
 repetir: do 
  ! este control es mas lento
  ! j = v (1)
  ! if ( all (v == j) ) exit  ! pues son todos iguales
  ! y este mas rapido 
  s = sum (v)
  if (s (1) == 0) exit        ! pues son todos iguales
  v = abs (v - cshift (v,1) )
  ! if (mod (k,1000) == 0) write (*,*)"k = ",k
  k = k + 1
 end do repetir
 !
 !consulta cronometro
 call system_clock (count = h2)   ! t2 = second ()
 time (2) = h2 - h1
 !
 !salida
 call timer (n,h,"circulo")
 !
 !dloca
 ier = 0
 deallocate (r, stat = ier (1) )
 deallocate (v, stat = ier (2) )
 if ( any (ier .ne. 0) ) call errata (ier(1:2), "circulo: dloca")
 !
end program
! -----------------------------------------------------------------
