! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Compilation with IFORT:
!  ifort -xP -O3 -ipo -vms -c m_ctes.f90
!  ifort -xP -O3 -ipo -vms -c m_temps.f90
!  ifort -xP -O3 -ipo -vms -c m_tools.f90
!  ifort -xP -O3 -ipo -vms -c m_gauss0.f90
!  ifort -xP -O3 -ipo -vms -c cgauss0v.f90
!  ifort -xP -O3 -ipo -vms -o cgauss0v_s.exe *.o
!
! Compilation with G95:
!  g95 -c m_ctes.f90
!  g95 -c m_temps.f90
!  g95 -c m_tools.f90
!  g95 -c m_gauss0.f90
!  g95 -c cgauss0v.f90
!  g95 -o cgauss0v.exe *.o
!
! Compilation with (ADAPTOR + MPICH):
!  adaptor -settings
!  adaptor -hpf -dm -free -v -c m_ctes.f90
!  adaptor -hpf -dm -free -v -c m_temps.f90
!  adaptor -hpf -dm -free -v -c m_tools.f90
!  adaptor -hpf -dm -free -v -c m_gauss0.f90
!  adaptor -hpf -dm -free -v -c cgauss0v.f90
!  adaptor -hpf -dm -free -v -o cgauss0v_p.exe *.o
!
! Running with MPICH:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpdringtest
!  mpiexec -machinefile ~/machi.dat -np 2 cgauss0v_p.exe
!  mpiexec -machinefile ~/machi.dat -1 -np 4 cgauss0v_p.exe
!  mpdallexit
!  pdsh -a hostname
!
! The same running but with MPI (old):
!  mpirun -machinefile ~/machi.dat -np 2 cgauss0v_p.exe
!  mpirun -machinefile ~/machi.dat -nolocal -np 4 cgauss0v_p.exe
!
! Some running details:
!  si n = 8 000 entonces m = n^2, RAM = m * 8 /1e6 =  512 [Mbytes]
!  pero como aqui hay una copia de A se tiene ram = 1,024 [Gbytes]
!
! output example:
!  number of unknowns ; n = ? 
!  max diff =  5.83895598538220E-10
!  time   =  576.0575  seconds
!  mflops =  374.9626  on  8  processors
!  mflops =  46.87032  for one processor
! -----------------------------------------------------------------
program  cgauss0v
  use m_ctes
  use m_temps
  use m_tools
  use m_gauss0
  implicit none
  integer (iin), parameter :: ncut = 1000
  integer (iin) :: j
  complex (isp), dimension (:,:), allocatable ::  a
  complex (isp), dimension (:)  , allocatable ::  b, r
  complex (isp), dimension (:,:), allocatable :: a0
  complex (isp), dimension (:)  , allocatable :: b0
  real    (isp), dimension (:)  , allocatable ::  x, y
  real    (isp) :: s
  !
  !hpf$ distribute (*,block) :: a
  !hpf$ align (i,*) with a (i,*) :: a0
  !hpf$ align (i)   with a (i,*) :: b0, b, r
  !
  integer (iin) :: n, m
  integer (iin) :: i, k, falla = 1
  !
  integer (iin) :: n_unknowns, n_loads
  namelist /gauss_basic_data/ n_unknowns, n_loads
  !
  ! lee del disco
  open  (1, file = "gauss_basic.dat", status = "old", err = 100)
  read  (1,nml = gauss_basic_data)
  write (*,nml = gauss_basic_data)  
  close (1, status = "keep")
  n = n_unknowns
  m = n_loads
  !
  !carteles
  write (*,*)
  write (*,*) "Gauss solution of a SEL with dynamic RAM "
  write (*,*) "number of unknows ; n = ", n
  write (*,*) "number of loads   ; m = ", m
  !
  !aloca sistema
  ier = 0
  allocate ( a (1:n,1:n), stat = ier (1) )
  allocate ( b (1:n)    , stat = ier (2) )
  allocate (a0 (1:n,1:n), stat = ier (3) )
  allocate (b0 (1:n)    , stat = ier (4) )
  allocate ( r (1:n)    , stat = ier (5) )
  allocate ( x (1:n)    , stat = ier (6) )
  allocate ( y (1:n)    , stat = ier (7) )
  if ( any (ier .ne. 0) ) call errata (ier(1:7), "aloca" )
  !
  !define un SEAL A X = B donde A es la matriz del sistema,
  !X, B matrices solucion y carga, respectivamente. 
  do  j = 1, n
    call  random_number (x)
    call  random_number (y)
    a (1:n,j) = cmplx ( x (1:n) , y (1:n) )
  end do
  call  random_number (x)
  call  random_number (y)
  b (1:n) = cmplx ( x (1:n) , y (1:n) )
  !hpf$ independent
  forall (i=1:n)  a (i,i) = a (i,i) + cmplx (real(n),0.0_isp)
  !
  !copia para posterior verificacion de los resultados
  if (n < ncut) then
    a0 = a
    b0 = b
  end if
  !
  !solucion gaussiana
  call  solve_b1 (a, b, imprime = .true.)
  !
  !verifica matriz solucion X con R = B - A X
  if (n < ncut) then
    print *
    print *, "verification; r = A x - b "
    r = matmul (a0,b) - b0 (1:n)
    s = sqrt (dot_product (r,r))
    print *, "|| r ||_2 = ", s
  end if
  !
  !eventual impresion
    print *
    print *, "matriz dato A "
    do  j = 1, n
      do  i = 1, n
        print *,  a0 (i,j)
      end do
    end do ! i
    print *
    print *, "carga B "
    do  i = 1, n
      print *,  b0 (i)
    end do ! i
    print *
    print *, "matriz factorizada A "
    do  j = 1, n
      do  i = 1, n
        print *,  a (i,j)
      end do
    end do ! i
    print *
    print *, "solucion X "
    do  i = 1, n
      print *, b (i)
    end do ! i
    print *
    print *, "residuo  R "
    do  i = 1, n
      print *, r (i)
    end do ! j
  !
  !dloca 
  ier = 0
  deallocate ( y, stat = ier (7) )
  deallocate ( x, stat = ier (6) )
  deallocate ( r, stat = ier (5) )
  deallocate (b0, stat = ier (4) )
  deallocate (a0, stat = ier (3) )
  deallocate ( b, stat = ier (2) )
  deallocate ( a, stat = ier (1) )
  if (any (ier .ne. 0))  call errata (ier(1:7), "dloca" )
  !
  falla = 0
  100 if (falla .ne. 0) stop "error: namelist file was not found ..."
  !
  print *
  print *, "fine ... "
  !
  ! formatos
  110 format (a)
  120 format (1x, i4, 1x, i20, 1x, e24.12)
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
