! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!   Gauss con PIVOTAJE parcial con (A, B), donde
!   A ; matriz de coeficientes
!   B ; matriz de cargas
! -----------------------------------------------------------------
!$Id: 2006/09/25 jdelia Exp $
module  m_gauss1       ! with or without mapping
  use   m_ctes
  use   m_temps
  use   m_tools1
  implicit none
  private              ! por omision todo es privado
  !precision doble     ! without any mapping
  public  dsolve_b1    , zsolve_b1
  public  dsolve_bm    , zsolve_bm
  public  dlu_factor   , zlu_factor
  public  dforward_sol , zforward_sol
  public  dbackward_sol, zbackward_sol
  !precision double    ! with descriptive Mapping
  public  dsolve_b1_adp    , zsolve_b1_adp
  public  dsolve_bm_adp    , zsolve_bm_adp
  public  dlu_factor_adp   , zlu_factor_adp
  public  dforward_sol_adp , zforward_sol_adp
  public  dbackward_sol_adp, zbackward_sol_adp
  !precision cuadruple ! it's not possible due to ADAPTOR restrictions
  !variables internas
  character (11), parameter :: c = "m_gauss1 > "
  !para medir tiempos
  real    (idp), dimension (3) :: time
  integer (iin) :: h1, h2
  contains
  !
  !
  !----------------------------------------------------------------
  subroutine  dsolve_b1 (a, b, imprime)
    integer (iin) :: i, j, k, p, eco
    real    (idp), dimension (:,:), intent (inout) :: a
    real    (idp), dimension (:)  , intent (inout) :: b
    logical      , optional       , intent (in)    :: imprime
    real    (idp), dimension (:)  , allocatable :: y
    real    (idp), dimension (:)  , allocatable :: d
    integer (iin), dimension (:)  , allocatable :: indx
    integer (iin) :: n, m
    !
    !valida tamanios
    n = siza (size (b), size (a,1), size (a,2), "size (a)" )
    !
    !aloca
    ier = 0
    allocate (y    (1:n), stat = ier (1) )
    allocate (d    (1:n), stat = ier (2) )
    allocate (indx (1:n), stat = ier (3) )
    if (any (ier .ne. 0)) call errata (ier(1:3), "solve: aloca")
    !
    !decide si imprimir
    if  ( present (imprime) ) then
      if (imprime) then
        eco = 1
      else
        eco = 0
      end if
    end if
    !
    !tareas
    call     dlu_factor (a, d, indx)
    call   dforward_sol (a, b, y, indx)
    call  dbackward_sol (a, y, b)
    !
    !estadistica
    if  (eco == 1) then
      m = 1
      call  mflops_g (time, n, m, "gauss_pivo")
    end if
    !
    !dloca
    ier = 0
    deallocate (indx, stat = ier (3) )
    deallocate (d   , stat = ier (2) )
    deallocate (y   , stat = ier (1) )
    if (any(ier.ne.0)) call errata (ier(1:3), "solve: dloca")
    !
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  dsolve_bm (a, b, imprime)
    integer (iin) :: i, j, k, p, eco
    real    (idp), dimension (:,:), intent (inout) :: a
    real    (idp), dimension (:,:), intent (inout) :: b
    logical      , optional       , intent (in)    :: imprime
    real    (idp), dimension   (:), allocatable :: x, y, v
    real    (idp), dimension   (:), allocatable :: d
    integer (iin), dimension   (:), allocatable :: indx
    integer (iin) :: n, m
    !
    !valida tamanios
    n = siza (size (b,1), size (a,1), size (a,2), "size (a)" )
    m =       size (b,2)
    !
    !aloca
    ier = 0
    allocate (x    (1:n), stat = ier (1) )
    allocate (y    (1:n), stat = ier (2) )
    allocate (v    (1:n), stat = ier (3) )
    allocate (d    (1:n), stat = ier (4) )
    allocate (indx (1:n), stat = ier (5) )
    if (any (ier .ne. 0)) call errata (ier(1:5), "solve: aloca")
    !
    !decide si imprimir
    if  ( present (imprime) ) then
      if  (imprime) then
        eco = 1
      else
        eco = 0
      end if
    end if
    !
    !re-inicia cronometro:
    time = 0.0
    !
    call  dlu_factor (a, d, indx)
    do  p = 1, m
      v (1:n) = b (1:n,p)
      call   dforward_sol (a, v, y, indx)
      call  dbackward_sol (a, y, x)
      b (1:n,p) = x (1:n)
    end do
    !
    !estadistica
    if  (eco == 1) then
      call  mflops_g (time, n, m, "gauss_pivo")
    end if
    !
    !dloca
    ier = 0
    deallocate (indx, stat = ier (5) )
    deallocate (d   , stat = ier (4) )
    deallocate (v   , stat = ier (3) )
    deallocate (y   , stat = ier (2) )
    deallocate (x   , stat = ier (1) )
    if (any(ier.ne.0)) call errata (ier(1:5), "solve: dloca")
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  dlu_factor (a, d, indx)
    integer (iin) :: i, j, k, n,kmax
    real    (idp), dimension (:,:), intent (inout) :: a
    real    (idp), dimension (:)  , intent (out)   :: d
    integer (iin), dimension (:)  , intent (out)   :: indx
    real    (idp), parameter :: tenue = 1.0d-20
    !
    !valida tamanios
    n = siza (size (a,1), size (a,2), "size (a)" )
    !
    !
    call system_clock (count = h1)
    !scaling
    indx = 0 
    d = maxval (abs (a), dim = 2) ! loop on rows 
    if (any (d == 0.0_idp) ) stop" singular matrix ... "  
    d = 1.0_idp / d
    do  k = 1, n
      !partial pivoting    
      kmax = (k - 1) + maxlocat (d (k:n) * abs ( a (k:n,k) ) )
      if (kmax .ne. k) then
        call  swap (a (kmax,:), a (k,:))
        d (kmax) = d (k)
      end if
      indx (k) = kmax
      if  (a (k,k) == 0.0_idp)  a (k,k) = tenue
      !reduction with outer product
      a (k+1:n,k:k)   = a (k+1:n,k:k) / a (k,k)
      a (k+1:n,k+1:n) = a (k+1:n,k+1:n) -                      &
              spread (a (k+1:n,k), dim = 2, ncopies = n-k) *   &
              spread (a (k,k+1:n), dim = 1, ncopies = n-k)
    end do
    call system_clock (count = h2)
    time (1) = dble (h2 - h1)
    !
    !
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  dforward_sol (a, b, y, indx)
    integer (iin) :: i, j, k, l, n
    real    (idp), dimension (:,:), intent (in)    :: a
    real    (idp), dimension (:)  , intent (inout) :: b
    real    (idp), dimension (:)  , intent (out)   :: y
    integer (iin), dimension (:)  , intent (inout) :: indx
    real    (idp) :: prima, s
    character (22) :: e = "size (a) | length (b,y)"
    !
    !valida tamanios    
    n = siza (size (b), size (y), size (a,1), size (a,2), e)
    if (any (indx < 1)) stop "error (forward_sol): index  < 1 "
    if (any (indx > n)) stop "error (forward_sol): index  > n "
    !
    !
    call system_clock (count = h1)
    k = 0
    do  i = 1, n
     l     = indx (i)
     prima = b (l)
     b (l) = b (i)
     s = 0.0_idp
     if     (k .ne. 0) then
       s = sum ( a (i,k:i-1) * y (k:i-1) )
     elseif (prima .ne. 0.0_idp) then
       k = i
     end if
     y (i) = prima - s
    end do
    call system_clock (count = h2)
    time (2) = dble (h2 - h1)
    !
    !
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  dbackward_sol (a, y, x)
    integer (iin) :: i, j, k, n
    real    (idp), dimension (:,:), intent (in)  :: a
    real    (idp), dimension (:)  , intent (in)  :: y
    real    (idp), dimension (:)  , intent (out) :: x
    real    (idp) :: s
    character (22) :: e = "size (a) | length (x,y)"
    !
    !valida tamanios
    n = siza (size (x), size (y), size (a,1), size (a,2),e)
    !
    !
    call system_clock (count = h1)
    do  i = n, 1, -1
      s = sum ( a (i,i+1:n) * x (i+1:n) )
      x (i) = (y (i) - s) / a (i,i)
    end do
    call system_clock (count = h2)
    time (3) = dble (h2 - h1)
    !
    !
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  zsolve_b1 (a, b, imprime)
    integer (iin) :: i, j, k, p, eco
    complex (idp), dimension (:,:), intent (inout) :: a
    complex (idp), dimension (:)  , intent (inout) :: b
    logical      , optional       , intent (in)    :: imprime
    complex (idp), dimension (:)  , allocatable :: y
    real    (idp), dimension (:)  , allocatable :: d
    integer (iin), dimension (:)  , allocatable :: indx
    integer (iin) :: n, m
    !
    !valida tamanios
    n = siza (size (b), size (a,1), size (a,2), "size (a)" )
    !
    !aloca
    ier = 0
    allocate (y    (1:n), stat = ier (1) )
    allocate (d    (1:n), stat = ier (2) )
    allocate (indx (1:n), stat = ier (3) )
    if (any (ier .ne. 0)) call errata (ier(1:3), "solve: aloca")
    !
    !decide si imprimir
    if  ( present (imprime) ) then 
      if (imprime) then
        eco = 1
      else
        eco = 0
      end if
    end if
    !
    !tareas
    call     zlu_factor (a, d, indx)
    call   zforward_sol (a, b, y, indx)
    call  zbackward_sol (a, y, b)
    !
    !estadistica
    if  (eco == 1) then
      m = 1
      call  mflops_g (time, n, m, "gauss_pivo")
    end if
    !
    !dloca
    ier = 0
    deallocate (indx, stat = ier (3) )
    deallocate (d   , stat = ier (2) )
    deallocate (y   , stat = ier (1) )
    if (any(ier.ne.0)) call errata (ier (1:3), "solve: dloca")
    !
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  zsolve_bm (a, b, imprime)
    integer (iin) :: i, j, k, p, eco
    complex (idp), dimension (:,:), intent (inout) :: a
    complex (idp), dimension (:,:), intent (inout) :: b
    logical      , optional       , intent (in)    :: imprime
    complex (idp), dimension   (:), allocatable :: x, y, v
    real    (idp), dimension   (:), allocatable :: d
    integer (iin), dimension   (:), allocatable :: indx
    integer (iin) :: n, m
    !
    !valida tamanios
    n = siza (size (b,1), size (a,1), size (a,2), "size (a)" )
    m =       size (b,2)
    !
    !aloca
    ier = 0
    allocate (x    (1:n), stat = ier (1) )
    allocate (y    (1:n), stat = ier (2) )
    allocate (v    (1:n), stat = ier (3) )
    allocate (d    (1:n), stat = ier (4) )
    allocate (indx (1:n), stat = ier (5) )
    if (any (ier .ne. 0)) call errata (ier(1:5), "solve: aloca")
    !
    !decide si imprimir
    if  ( present (imprime) ) then
      if  (imprime) then
        eco = 1
      else
        eco = 0
      end if
    end if
    !
    !re-inicia cronometro:
    time = 0.0
    !
    call  zlu_factor (a, d, indx)
    do  p = 1, m
      v (1:n) = b (1:n,p)
      call   zforward_sol (a, v, y, indx)
      call  zbackward_sol (a, y, x)
      b (1:n,p) = x (1:n)
    end do
    !
    !estadistica
    if  (eco == 1) then
      call  mflops_g (time, n, m, "gauss_pivo")
    end if
    !
    !dloca
    ier = 0
    deallocate (indx, stat = ier (5) )
    deallocate (d   , stat = ier (4) )
    deallocate (v   , stat = ier (3) )
    deallocate (y   , stat = ier (2) )
    deallocate (x   , stat = ier (1) )
    if (any(ier.ne.0)) call errata (ier(1:5), "solve: dloca")
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  zlu_factor (a, d, indx)
    integer (iin) :: i, j, k, n,kmax
    complex (idp), dimension (:,:), intent (inout) :: a
    real    (idp), dimension (:)  , intent (out)   :: d
    integer (iin), dimension (:)  , intent (out)   :: indx
    real    (idp), parameter :: tenue = 1.0d-20
    !
    !valida tamanios
    n = siza (size(a,1), size(a,2), "size (a)" )
    !
    !
    call system_clock (count = h1)
    !scaling
    indx = 0 
    d = maxval (abs (a), dim = 2) ! loop on rows 
    if (any (d == 0.0_idp) ) stop" singular matrix ... "  
    d = 1.0_idp / d
    do  k = 1, n
      !partial pivoting    
      kmax = (k - 1) + maxlocat (d (k:n) * abs (a(k:n,k)))
      if (kmax .ne. k) then
        call  swap (a (kmax,:), a (k,:))
        d (kmax) = d (k)
      end if
      indx (k) = kmax
      if  (abs (a(k,k)) == 0.0_idp) a (k,k) = cmplx (tenue, 0.0_idp)
      !reduction with outer product
      a (k+1:n,k:k)   = a (k+1:n,k:k) / a (k,k)
      a (k+1:n,k+1:n) = a (k+1:n,k+1:n) -                      &
              spread (a (k+1:n,k), dim = 2, ncopies = n-k) *   &
              spread (a (k,k+1:n), dim = 1, ncopies = n-k)
    end do
    call system_clock (count = h2)
    time (1) = dble (h2 - h1)
    !
    !
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  zforward_sol (a, b, y, indx)
    integer (iin) :: i, j, k, l, n
    complex (idp), dimension (:,:), intent (in)    :: a
    complex (idp), dimension (:)  , intent (inout) :: b
    complex (idp), dimension (:)  , intent (out)   :: y
    integer (iin), dimension (:)  , intent (inout) :: indx
    complex (idp) :: prima, s
    character (22) :: e = "size (a) | length (b,y)"
    !
    !valida tamanios
    n = siza (size (b), size (y), size (a,1), size (a,2), e)
    if (any (indx < 1)) stop "error (forward_sol): index  < 1 "
    if (any (indx > n)) stop "error (forward_sol): index  > n "
    !
    !
    call system_clock (count = h1)
    k = 0
    do  i = 1, n
     l     = indx (i)
     prima = b (l)
     b (l) = b (i)
     s     = cmplx (0.0_idp, 0.0_idp)
     if     (k .ne. 0) then
       s = sum ( a (i,k:i-1) * y (k:i-1) )
     elseif (prima .ne. 0.0_idp) then
       k = i
     end if
     y (i) = prima - s
    end do
    call system_clock (count = h2)
    time (2) = dble (h2 - h1)
    !
    !
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  zbackward_sol (a, y, x)
    integer (iin) :: i, j, k, n
    complex (idp), dimension (:,:), intent (in)  :: a
    complex (idp), dimension (:)  , intent (in)  :: y
    complex (idp), dimension (:)  , intent (out) :: x
    complex (idp) :: s
    character (22) :: e = "size (a) | length (x,y)"
    !
    !valida tamanios
    n = siza (size (x), size (y), size (a,1), size (a,2), e)
    !
    !
    call system_clock (count = h1)
    do  i = n, 1, -1
      s = sum ( a (i,i+1:n) * x (i+1:n) )
      x (i) = (y (i) - s) / a (i,i)
    end do
    call system_clock (count = h2)
    time (3) = dble (h2 - h1)
    !
    !
  end subroutine
  !
  !
  !----------------------------------------------------------------
  subroutine  dsolve_b1_adp (a, b, imprime)
    integer (iin) :: i, j, k, p, eco
    real    (idp), dimension (:,:), intent (inout) :: a
    real    (idp), dimension (:)  , intent (inout) :: b
    logical      , optional       , intent (in)    :: imprime
    real    (idp), dimension (:)  , allocatable :: y
    real    (idp), dimension (:)  , allocatable :: d
    integer (iin), dimension (:)  , allocatable :: indx
    integer (iin) :: n, m
    !hpf$ distribute * (*,block) :: a
    !hpf$ align (i)   with * a (i,*) :: b
    !hpf$ align (i)   with   a (i,*) :: y
    !hpf$ align (i)   with   a (i,*) :: d, indx
    !
    !valida tamanios
    n = siza (size (b), size (a,1), size (a,2), "size (a)" )
    !
    !aloca
    ier = 0
    allocate (y    (1:n), stat = ier (1) )
    allocate (d    (1:n), stat = ier (2) )
    allocate (indx (1:n), stat = ier (3) )
    if (any (ier .ne. 0)) call errata (ier(1:3), "solve: aloca")
    !
    !decide si imprimir
    if  ( present (imprime) ) then
      if (imprime) then
        eco = 1
      else
        eco = 0
      end if
    end if
    !
    !tareas
    call     dlu_factor_adp (a, d, indx)
    call   dforward_sol_adp (a, b, y, indx)
    call  dbackward_sol_adp (a, y, b)
    !
    !estadistica
    if  (eco == 1) then
      m = 1
      call  mflops_g (time, n, m, "gauss_pivo")
    end if
    !
    !dloca
    ier = 0
    deallocate (indx, stat = ier (3) )
    deallocate (d   , stat = ier (2) )
    deallocate (y   , stat = ier (1) )
    if (any(ier.ne.0)) call errata (ier(1:3), "solve: dloca")
    !
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  dsolve_bm_adp (a, b, imprime)
    integer (iin) :: i, j, k, p, eco
    real    (idp), dimension (:,:), intent (inout) :: a
    real    (idp), dimension (:,:), intent (inout) :: b
    logical      , optional       , intent (in)    :: imprime
    real    (idp), dimension   (:), allocatable :: x, y, v
    real    (idp), dimension   (:), allocatable :: d
    integer (iin), dimension   (:), allocatable :: indx
    integer (iin) :: n, m
    !hpf$ distribute * (*,block) :: a
    !hpf$ align (i,*) with * a (i,*) :: b
    !hpf$ align (i)   with   a (i,*) :: x, y, v
    !hpf$ align (i)   with   a (i,*) :: d, indx
    !
    !valida tamanios
    n = siza (size (b,1), size (a,1), size (a,2), "size (a)" )
    m =       size (b,2)
    !
    !aloca
    ier = 0
    allocate (x    (1:n), stat = ier (1) )
    allocate (y    (1:n), stat = ier (2) )
    allocate (v    (1:n), stat = ier (3) )
    allocate (d    (1:n), stat = ier (4) )
    allocate (indx (1:n), stat = ier (5) )
    if (any (ier .ne. 0)) call errata (ier(1:5), "solve: aloca")
    !
    !decide si imprimir
    if  ( present (imprime) ) then
      if  (imprime) then
        eco = 1
      else
        eco = 0
      end if
    end if
    !
    !re-inicia cronometro:
    time = 0.0
    !
    call  dlu_factor_adp (a, d, indx)
    do  p = 1, m
      v (1:n) = b (1:n,p)
      call   dforward_sol_adp (a, v, y, indx)
      call  dbackward_sol_adp (a, y, x)
      b (1:n,p) = x (1:n)
    end do
    !
    !estadistica
    if  (eco == 1) then
      call  mflops_g (time, n, m, "gauss_pivo")
    end if
    !
    !dloca
    ier = 0
    deallocate (indx, stat = ier (5) )
    deallocate (d   , stat = ier (4) )
    deallocate (v   , stat = ier (3) )
    deallocate (y   , stat = ier (2) )
    deallocate (x   , stat = ier (1) )
    if (any(ier.ne.0)) call errata (ier(1:5), "solve: dloca")
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  dlu_factor_adp (a, d, indx)
    integer (iin) :: i, j, k, n,kmax
    real    (idp), dimension (:,:), intent (inout) :: a
    real    (idp), dimension (:)  , intent (out)   :: d
    integer (iin), dimension (:)  , intent (out)   :: indx
    real    (idp), parameter :: tenue = 1.0d-20
    real    (idp), dimension (size(a,1)) :: t
    !hpf$ distribute * (*,block) :: a
    !hpf$ align (i)   with * a (i,*) :: d, indx
    !hpf$ align (i)   with   a (i,*) :: t
    !
    !valida tamanios
    n = siza (size (a,1), size (a,2), "size (a)" )
    !
    !
    call system_clock (count = h1)
    !scaling
    indx = 0 
    d = maxval (abs (a), dim = 2) ! loop on rows 
    if (any (d == 0.0_idp) ) stop" singular matrix ... "  
    d = 1.0_idp / d
    do  k = 1, n
      !partial pivoting    
      kmax = (k - 1) + maxlocat (d (k:n) * abs ( a (k:n,k) ) )
      if (kmax .ne. k) then
        t (1:n)      = a (kmax,1:n)
        a (kmax,1:n) = a (k,1:n)
        a (k,1:n)    = t (1:n)
!       call  swap (a (kmax,:), a (k,:))
        d (kmax) = d (k)
      end if
      indx (k) = kmax
      if  (a (k,k) == 0.0_idp)  a (k,k) = tenue
      !reduction with outer product
      a (k+1:n,k:k)   = a (k+1:n,k:k) / a (k,k)
      a (k+1:n,k+1:n) = a (k+1:n,k+1:n) -                      &
              spread (a (k+1:n,k), dim = 2, ncopies = n-k) *   &
              spread (a (k,k+1:n), dim = 1, ncopies = n-k)
    end do
    call system_clock (count = h2)
    time (1) = dble (h2 - h1)
    !
    !
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  dforward_sol_adp (a, b, y, indx)
    integer (iin) :: i, j, k, l, n
    real    (idp), dimension (:,:), intent (in)    :: a
    real    (idp), dimension (:)  , intent (inout) :: b
    real    (idp), dimension (:)  , intent (out)   :: y
    integer (iin), dimension (:)  , intent (inout) :: indx
    real    (idp) :: prima, s
    character (22) :: e = "size (a) | length (b,y)"
    !hpf$ distribute * (*,block) :: a
    !hpf$ align (i)   with * a (i,*) :: b, y, indx
    !
    !valida tamanios    
    n = siza (size (b), size (y), size (a,1), size (a,2), e)
    if (any (indx < 1)) stop "error (forward_sol): index  < 1 "
    if (any (indx > n)) stop "error (forward_sol): index  > n "
    !
    !
    call system_clock (count = h1)
    k = 0
    do  i = 1, n
     l     = indx (i)
     prima = b (l)
     b (l) = b (i)
     s = 0.0_idp
     if     (k .ne. 0) then
       s = sum ( a (i,k:i-1) * y (k:i-1) )
     elseif (prima .ne. 0.0_idp) then
       k = i
     end if
     y (i) = prima - s
    end do
    call system_clock (count = h2)
    time (2) = dble (h2 - h1)
    !
    !
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  dbackward_sol_adp (a, y, x)
    integer (iin) :: i, j, k, n
    real    (idp), dimension (:,:), intent (in)  :: a
    real    (idp), dimension (:)  , intent (in)  :: y
    real    (idp), dimension (:)  , intent (out) :: x
    real    (idp) :: s
    character (22) :: e = "size (a) | length (x,y)"
    !hpf$ distribute * (*,block) :: a
    !hpf$ align (i)   with * a (i,*) :: y, x
    !
    !valida tamanios
    n = siza (size (x), size (y), size (a,1), size (a,2),e)
    !
    !
    call system_clock (count = h1)
    do  i = n, 1, -1
      s = sum ( a (i,i+1:n) * x (i+1:n) )
      x (i) = (y (i) - s) / a (i,i)
    end do
    call system_clock (count = h2)
    time (3) = dble (h2 - h1)
    !
    !
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  zsolve_b1_adp (a, b, imprime)
    integer (iin) :: i, j, k, p, eco
    complex (idp), dimension (:,:), intent (inout) :: a
    complex (idp), dimension (:)  , intent (inout) :: b
    logical      , optional       , intent (in)    :: imprime
    complex (idp), dimension (:)  , allocatable :: y
    real    (idp), dimension (:)  , allocatable :: d
    integer (iin), dimension (:)  , allocatable :: indx
    integer (iin) :: n, m
    !hpf$ distribute * (*,block) :: a
    !hpf$ align (i)   with * a (i,*) :: b
    !hpf$ align (i)   with   a (i,*) :: y
    !hpf$ align (i)   with   a (i,*) :: d, indx
    !
    !valida tamanios
    n = siza (size (b), size (a,1), size (a,2), "size (a)" )
    !
    !aloca
    ier = 0
    allocate (y    (1:n), stat = ier (1) )
    allocate (d    (1:n), stat = ier (2) )
    allocate (indx (1:n), stat = ier (3) )
    if (any (ier .ne. 0)) call errata (ier(1:3), "solve: aloca")
    !
    !decide si imprimir
    if  ( present (imprime) ) then 
      if (imprime) then
        eco = 1
      else
        eco = 0
      end if
    end if
    !
    !tareas
    call     zlu_factor_adp (a, d, indx)
    call   zforward_sol_adp (a, b, y, indx)
    call  zbackward_sol_adp (a, y, b)
    !
    !estadistica
    if  (eco == 1) then
      m = 1
      call  mflops_g (time, n, m, "gauss_pivo")
    end if
    !
    !dloca
    ier = 0
    deallocate (indx, stat = ier (3) )
    deallocate (d   , stat = ier (2) )
    deallocate (y   , stat = ier (1) )
    if (any(ier.ne.0)) call errata (ier (1:3), "solve: dloca")
    !
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  zsolve_bm_adp (a, b, imprime)
    integer (iin) :: i, j, k, p, eco
    complex (idp), dimension (:,:), intent (inout) :: a
    complex (idp), dimension (:,:), intent (inout) :: b
    logical      , optional       , intent (in)    :: imprime
    complex (idp), dimension   (:), allocatable :: x, y, v
    real    (idp), dimension   (:), allocatable :: d
    integer (iin), dimension   (:), allocatable :: indx
    integer (iin) :: n, m
    !hpf$ distribute * (*,block) :: a
    !hpf$ align (i,*) with * a (i,*) :: b
    !hpf$ align (i)   with   a (i,*) :: x, y, v
    !hpf$ align (i)   with   a (i,*) :: d, indx
    !
    !valida tamanios
    n = siza (size (b,1), size (a,1), size (a,2), "size (a)" )
    m =       size (b,2)
    !
    !aloca
    ier = 0
    allocate (x    (1:n), stat = ier (1) )
    allocate (y    (1:n), stat = ier (2) )
    allocate (v    (1:n), stat = ier (3) )
    allocate (d    (1:n), stat = ier (4) )
    allocate (indx (1:n), stat = ier (5) )
    if (any (ier .ne. 0)) call errata (ier(1:5), "solve: aloca")
    !
    !decide si imprimir
    if  ( present (imprime) ) then
      if  (imprime) then
        eco = 1
      else
        eco = 0
      end if
    end if
    !
    !re-inicia cronometro:
    time = 0.0
    !
    call  zlu_factor_adp (a, d, indx)
    do  p = 1, m
      v (1:n) = b (1:n,p)
      call   zforward_sol_adp (a, v, y, indx)
      call  zbackward_sol_adp (a, y, x)
      b (1:n,p) = x (1:n)
    end do
    !
    !estadistica
    if  (eco == 1) then
      call  mflops_g (time, n, m, "gauss_pivo")
    end if
    !
    !dloca
    ier = 0
    deallocate (indx, stat = ier (5) )
    deallocate (d   , stat = ier (4) )
    deallocate (v   , stat = ier (3) )
    deallocate (y   , stat = ier (2) )
    deallocate (x   , stat = ier (1) )
    if (any(ier.ne.0)) call errata (ier(1:5), "solve: dloca")
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  zlu_factor_adp (a, d, indx)
    integer (iin) :: i, j, k, n,kmax
    complex (idp), dimension (:,:), intent (inout) :: a
    real    (idp), dimension (:)  , intent (out)   :: d
    integer (iin), dimension (:)  , intent (out)   :: indx
    real    (idp), parameter :: tenue = 1.0d-20
    complex (idp), dimension (size(a,1)) :: t
    !hpf$ distribute * (*,block) :: a
    !hpf$ align (i)   with * a (i,*) :: d, indx
    !hpf$ align (i)   with   a (*,i) :: t
    !
    !valida tamanios
    n = siza (size(a,1), size(a,2), "size (a)" )
    !
    !
    call system_clock (count = h1)
    !scaling
    indx = 0 
    d = maxval (abs (a), dim = 2) ! loop on rows 
    if (any (d == 0.0_idp) ) stop" singular matrix ... "  
    d = 1.0_idp / d
    do  k = 1, n
      !partial pivoting    
      kmax = (k - 1) + maxlocat (d (k:n) * abs (a(k:n,k)))
      if (kmax .ne. k) then
        t (1:n)      = a (kmax,1:n)
        a (kmax,1:n) = a (k,1:n)
        a (k,1:n)    = t (1:n)
!       call  swap (a (kmax,:), a (k,:))
        d (kmax) = d (k)
      end if
      indx (k) = kmax
      if  (abs (a(k,k)) == 0.0_idp) a (k,k) = cmplx (tenue, 0.0_idp)
      !reduction with outer product
      a (k+1:n,k:k)   = a (k+1:n,k:k) / a (k,k)
      a (k+1:n,k+1:n) = a (k+1:n,k+1:n) -                      &
              spread (a (k+1:n,k), dim = 2, ncopies = n-k) *   &
              spread (a (k,k+1:n), dim = 1, ncopies = n-k)
    end do
    call system_clock (count = h2)
    time (1) = dble (h2 - h1)
    !
    !
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  zforward_sol_adp (a, b, y, indx)
    integer (iin) :: i, j, k, l, n
    complex (idp), dimension (:,:), intent (in)    :: a
    complex (idp), dimension (:)  , intent (inout) :: b
    complex (idp), dimension (:)  , intent (out)   :: y
    integer (iin), dimension (:)  , intent (inout) :: indx
    complex (idp) :: prima, s
    character (22) :: e = "size (a) | length (b,y)"
    !hpf$ distribute * (*,block) :: a
    !hpf$ align (i)   with * a (i,*) :: b, y, indx
    !
    !valida tamanios
    n = siza (size (b), size (y), size (a,1), size (a,2), e)
    if (any (indx < 1)) stop "error (forward_sol): index  < 1 "
    if (any (indx > n)) stop "error (forward_sol): index  > n "
    !
    !
    call system_clock (count = h1)
    k = 0
    do  i = 1, n
     l     = indx (i)
     prima = b (l)
     b (l) = b (i)
     s     = cmplx (0.0_idp, 0.0_idp)
     if     (k .ne. 0) then
       s = sum ( a (i,k:i-1) * y (k:i-1) )
     elseif (prima .ne. 0.0_idp) then
       k = i
     end if
     y (i) = prima - s
    end do
    call system_clock (count = h2)
    time (2) = dble (h2 - h1)
    !
    !
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  zbackward_sol_adp (a, y, x)
    integer (iin) :: i, j, k, n
    complex (idp), dimension (:,:), intent (in)  :: a
    complex (idp), dimension (:)  , intent (in)  :: y
    complex (idp), dimension (:)  , intent (out) :: x
    complex (idp) :: s
    character (22) :: e = "size (a) | length (x,y)"
    !hpf$ distribute * (*,block) :: a
    !hpf$ align (i)   with * a (i,*) :: y, x
    !
    !valida tamanios
    n = siza (size (x), size (y), size (a,1), size (a,2), e)
    !
    !
    call system_clock (count = h1)
    do  i = n, 1, -1
      s = sum ( a (i,i+1:n) * x (i+1:n) )
      x (i) = (y (i) - s) / a (i,i)
    end do
    call system_clock (count = h2)
    time (3) = dble (h2 - h1)
    !
    !
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  mflops_g (time, n, m, raiz)
    real    (idp), dimension (:), intent (inout) :: time
    integer (iin), intent (in) :: n     ! orden de la matriz
    integer (iin), intent (in) :: m     ! nro de cargas
    character (*), intent (in) :: raiz  ! raiz archivo
    character (len(raiz)+4) :: arch     ! extension prefijada = 4
    character (4) :: exte = ".tim"      ! aqui la exten. es cte
    character (44):: s1, s2
    integer (iin) :: l, p
    integer (iin) :: ntime, h3 
    real    (idp) :: total, ops, mflops
    !
    !nro de procesadores
    !p = 1                               ! secuencial 
    p = number_of_processors ()          ! solo en HPF
    !
    !control
    ntime = size (time)
    if (ntime < 3) stop "error (mflops_g): size (time) < 3 "
    !
    !pasa a segundos
    call system_clock (count_rate = h3)  ! cte para pasar a seg
    if (h3 < 1) h3 = 1                   ! control
    time = time / dble (h3)              ! pasa de ciclos a seg
    !
    !tiempo total
    total = sum (time)
    !
    !estadistica  
    ops = (2.0/3.0) * dble (n) ** 3 + 2.0 * dble (n) ** 2
    mflops = ops / (1.0e6 * total)
    !
    !define archivo
    l = len_trim (raiz)        ! longitud omitiendo blancos
    arch = raiz (1:l) // exte  ! apendiza en ese orden sin blancos
    !
    !resumen a disco
    s1 ="  p        n   m   twall_factorLU   twall_"
    s2 ="forward  twall_back            Mflops     "
    print *
    print 100, " archivo timer: " // arch
    print 100, s1 // s2
    !
    open (1, file = arch         , &
             status = "unknown"  , &
             position = "append")
    write (*,110) p, n, m, time (1:3), mflops
    write (1,110) p, n, m, time (1:3), mflops
    close (1, status = 'keep')
    !
    print *
    print 120, total, mflops
    !
    ! formatos
    100 format (a)
    110 format (1x, i2, 1x, i8, 1x, i3, 3(1x,e16.8), 1x, f12.3 )
    120 format (" elapsed time = ",e16.8," seconds ;",f12.3," Mflops")
    !
  end subroutine
end module
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
