! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!   Gauss con (A, B), donde
!   A ; matriz de coeficientes
!   B ; matriz de cargas
! -----------------------------------------------------------------
module  m_krylov0     ! without mapping
  use   m_ctes
  use   m_temps
  use   m_tools0
  use   m_gauss0
  implicit none
  private              ! por omision todo es privado
  !precision doble     ! without mapping
  public  dsolve_fom
  !precision cuadruple ! without mapping due to ADAPTOR restrictions
  public  qsolve_fom
  !
  character (12), parameter :: c = "m_krylov0 > "
  !para medir tiempos
  real    (idp), dimension (1) :: time
  integer (iin) :: h1, h2
  contains
  !
  !----------------------------------------------------------------
  subroutine  dsolve_fom (a, b, m)
    integer (iin) :: i, j, k, p
    real    (idp), dimension (:,:), intent (inout) :: a
    real    (idp), dimension   (:), intent (inout) :: b
    integer (iin)                 , intent (inout) :: m
    real    (idp), dimension (:,:), allocatable :: h, v
    real    (idp), dimension (:)  , allocatable :: e1, y
    real    (idp), dimension (:)  , allocatable :: x0, r0, x, r
    !
    real    (idp), dimension (:,:), allocatable :: e
    real    (idp), dimension (:)  , allocatable :: q, t, u
    real    (idp) :: h0
    !
    real    (idp) :: beta0, beta1
    integer (iin) :: n
    !
    !valia tamanios
    n = siza (size(b), size(a,1), size(a,2), "size (a)" )
    !
    !aloca
    ier = 0
    allocate (x0 (n)    , stat = ier  (1) )
    allocate (r0 (n)    , stat = ier  (2) )
    allocate (x  (n)    , stat = ier  (3) )
    allocate (r  (n)    , stat = ier  (4) )
    allocate (v  (n,m+1), stat = ier  (5) )
    allocate (h  (m+1,m), stat = ier  (6) )
    allocate (e1 (m)    , stat = ier  (7) )
    allocate (y  (m)    , stat = ier  (8) )
    allocate (q (n)     , stat = ier  (9) )
    allocate (t (n)     , stat = ier (10) )
    allocate (u (n)     , stat = ier (11) )
    allocate (e (m,m+1) , stat = ier (12) )
    if (any (ier .ne. 0)) call errata (ier(1:12), "dsolve_fom: aloca")
    !
    !cronometra
    time = cero
    call system_clock (count = h1)
    !
    !solucion inicial
    x0 = 0.0_idp
    !
    !Vector residuo inicial "r0" y su norma L2 en "z0"
    r0 = b - matmul (a, x0)
    beta0 = sqrt (sum (r0 * r0) )  ! residuo inicial
    !
    if (beta0 > tol) then
      !
      !Algoritmo de Arnoldi: devuelve H (m+1,m) y V (n,m), con m < n
      !
      !1ra columna con el residuo inicial normalizado
      v (:,1) = r0 / beta0
      !
      !Lazo iterativo: llena columnas V_m = { v_1, v_2, ..., v_{m+1} } y
      !las entradas en la matriz F (m+1,m) 
      p = 0
      do  j = 1, m
        t = v (:,j)
        u = matmul (a, v (:,j))
        do  i = 1, j
          h (i,j) = sum (u * v (:,i)) ! f_ij = (Av_j, v_i) ; i=1,2,...,j 
        end do
        q = 0.0_idp
        do   i = 1, j
          q = q + h (i,j) * v (:,i)
        end do
        t = u - q
        h0 = sqrt ( sum (t * t) ) 
        h (j+1,j) = h0
        if  (h0 > tol) then
          p = j
          v (:,j+1) = t / h0
        else
          exit
        end if
      end do
      !actualiza "m"
      m = p
      if (m < 1) stop "error (solve_fom): m < 1 "
      !
      !solucion auxiliar por ortogonalizacion completa del residuo inicial
      !a partir del sistema cuadrado: Hm (m,m) * y (m,1) = e1 (m,1)
      e1 = 0.0_idp  ;  e1 (1) = beta0  ;  y = e1    
      call  solve_b1 (h(1:m,1:m), y(1:m))
      !
      !solucion iterada m-veces x^{(m)} del SEL
      x = x0 + matmul (v(1:n,1:m), y(1:m))
    else
      x = x0
    end if
    !
    r = b - matmul (a,x)
    beta1 = sqrt (sum (r * r) )
    !cronometrea
    call system_clock (count = h2)
    time (1) = dble (h2 - h1)
    !
    !estadistica
    call  mflops_k (time, n, m, "krylov")
    !
    print *
    print *, "unknows number          ;     n = ", n
    print *, "efective Krylov subspace;     m = ", m
    print *, "initial || r ||_2       ; beta0 = ", beta0
    print *, "last    || r ||_2       ; beta1 = ", beta1
    !
    !dloca
    ier = 0
    deallocate (e , stat = ier (12) )
    deallocate (u , stat = ier (11) )
    deallocate (t , stat = ier (10) )
    deallocate (q , stat = ier  (9) )
    deallocate (y , stat = ier  (8) )
    deallocate (e1, stat = ier  (7) )
    deallocate (h , stat = ier  (6) )
    deallocate (v , stat = ier  (5) )
    deallocate (r , stat = ier  (4) )
    deallocate (x , stat = ier  (3) )
    deallocate (r0, stat = ier  (2) )
    deallocate (x0, stat = ier  (1) )
    if (any(ier.ne.0)) call errata (ier(1:12), "dsolve_fom: dloca")
    !
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  qsolve_fom (a, b, m)
    integer (iin) :: i, j, k, p
    real    (iqp), dimension (:,:), intent (inout) :: a
    real    (iqp), dimension   (:), intent (inout) :: b
    integer (iin)                 , intent (inout) :: m
    real    (iqp), dimension (:,:), allocatable :: h, v
    real    (iqp), dimension (:)  , allocatable :: e1, y
    real    (iqp), dimension (:)  , allocatable :: x0, r0, x, r
    !
    real    (iqp), dimension (:,:), allocatable :: e
    real    (iqp), dimension (:)  , allocatable :: q, t, u
    real    (iqp) :: h0
    !
    real    (iqp) :: beta0, beta1
    integer (iin) :: n
    !
    !valia tamanios
    n = siza (size(b), size(a,1), size(a,2), "size (a)" )
    !
    !aloca
    ier = 0
    allocate (x0 (n)    , stat = ier  (1) )
    allocate (r0 (n)    , stat = ier  (2) )
    allocate (x  (n)    , stat = ier  (3) )
    allocate (r  (n)    , stat = ier  (4) )
    allocate (v  (n,m+1), stat = ier  (5) )
    allocate (h  (m+1,m), stat = ier  (6) )
    allocate (e1 (m)    , stat = ier  (7) )
    allocate (y  (m)    , stat = ier  (8) )
    allocate (q (n)     , stat = ier  (9) )
    allocate (t (n)     , stat = ier (10) )
    allocate (u (n)     , stat = ier (11) )
    allocate (e (m,m+1) , stat = ier (12) )
    if (any (ier .ne. 0)) call errata (ier(1:12), "qsolve_fom: aloca")
    !
    !cronometra
    time = cero
    call system_clock (count = h1)
    !
    !solucion inicial
    x0 = 0.0_iqp
    !
    !Vector residuo inicial "r0" y su norma L2 en "z0"
    r0 = b - matmul (a, x0)
    beta0 = sqrt (sum (r0 * r0) )  ! residuo inicial
    !
    if (beta0 > tol) then
      !
      !Algoritmo de Arnoldi: devuelve H (m+1,m) y V (n,m), con m < n
      !
      !1ra columna con el residuo inicial normalizado
      v (:,1) = r0 / beta0
      !
      !Lazo iterativo: llena columnas V_m = { v_1, v_2, ..., v_{m+1} } y
      !las entradas en la matriz F (m+1,m) 
      p = 0
      do  j = 1, m
        t = v (:,j)
        u = matmul (a, v (:,j))
        do  i = 1, j
          h (i,j) = sum (u * v (:,i)) ! f_ij = (Av_j, v_i) ; i=1,2,...,j 
        end do
        q = 0.0_iqp
        do   i = 1, j
          q = q + h (i,j) * v (:,i)
        end do
        t = u - q
        h0 = sqrt ( sum (t * t) ) 
        h (j+1,j) = h0
        if  (h0 > tol) then
          p = j
          v (:,j+1) = t / h0
        else
          exit
        end if
      end do
      !actualiza "m"
      m = p
      if (m < 1) stop "error (solve_fom): m < 1 "
      !
      !solucion auxiliar por ortogonalizacion completa del residuo inicial
      !a partir del sistema cuadrado: Hm (m,m) * y (m,1) = e1 (m,1)
      e1 = 0.0_iqp  ;  e1 (1) = beta0  ;  y = e1    
      call  solve_b1 (h(1:m,1:m), y(1:m))
      !
      !solucion iterada m-veces x^{(m)} del SEL
      x = x0 + matmul (v(1:n,1:m), y(1:m))
    else
      x = x0
    end if
    !
    r = b - matmul (a,x)
    beta1 = sqrt (sum (r * r) )
    !cronometrea
    call system_clock (count = h2)
    time (1) = dble (h2 - h1)
    !
    !estadistica
    call  mflops_k (time, n, m, "krylov")
    !
    print *
    print *, "unknows number          ;     n = ", n
    print *, "efective Krylov subspace;     m = ", m
    print *, "initial || r ||_2       ; beta0 = ", beta0
    print *, "last    || r ||_2       ; beta1 = ", beta1
    !
    !dloca
    ier = 0
    deallocate (e , stat = ier (12) )
    deallocate (u , stat = ier (11) )
    deallocate (t , stat = ier (10) )
    deallocate (q , stat = ier  (9) )
    deallocate (y , stat = ier  (8) )
    deallocate (e1, stat = ier  (7) )
    deallocate (h , stat = ier  (6) )
    deallocate (v , stat = ier  (5) )
    deallocate (r , stat = ier  (4) )
    deallocate (x , stat = ier  (3) )
    deallocate (r0, stat = ier  (2) )
    deallocate (x0, stat = ier  (1) )
    if (any(ier.ne.0)) call errata (ier(1:12), "qsolve_fom: dloca")
    !
  end subroutine
  !
  !----------------------------------------------------------------
  subroutine  mflops_k (time, n, m, raiz)
    real    (idp), dimension (:), intent (inout) :: time
    integer (iin), intent (in) :: n     ! orden de la matriz
    integer (iin), intent (in) :: m     ! subespacio de Krylov
    character (*), intent (in) :: raiz  ! raiz archivo
    character (len(raiz)+4) :: arch     ! extension prefijada = 4
    character (4) :: exte = ".tim"      ! aqui la exten. es cte
    character (44):: s1
    integer (iin) :: l, p
    integer (iin) :: ntime, h3 
    real    (idp) :: total, ops, mflops, c1, c2
    !
    !nro de procesadores
    !p = 1                               ! secuencial 
    p = number_of_processors ()          ! solo en HPF
    !
    !control
    ntime = size (time)
    if (ntime < 1) stop "error (mflops_k): size (time) < 1 "
    !
    !pasa a segundos
    call system_clock (count_rate = h3)  ! cte para pasar a seg
    if (h3 < 1) h3 = 1                   ! control
    time = time / dble (h3)              ! pasa de ciclos a seg
    !
    !tiempo total
    total = sum (time)
    !
    !estadistica
    c1 = (2.0/3.0) * dble (m) ** 3 + 2.0 * dble (m) ** 2
    c2 = dble (m + 2) * (n ** 2)
    ops = c1 + c2 
    mflops = ops / (1.0e6 * total)
    !
    !define archivo
    l = len_trim (raiz)        ! longitud omitiendo blancos
    arch = raiz (1:l) // exte  ! apendiza en ese orden sin blancos
    !
    !resumen a disco
    s1 ="  p        n   m   total_wall_time  Mflops   "
    print *
    print 100, " archivo timer: " // arch
    print 100, s1
    !
    open (1, file = arch         , &
             status = "unknown"  , &
             position = "append")
    write (*,110)  p, n, m, total, mflops
    write (1,110)  p, n, m, total, mflops
    close (1, status = 'keep')
    !
    print *
    print 120, total, mflops
    !
    ! formatos
    100 format (a)
    110 format (1x, i2, 1x, i8, 1x, i3, 1 (1x,e16.8), 1x, f12.3 )
    120 format (" elapsed time = ",e16.8," seconds ;",f12.3," Mflops")
    !
  end subroutine
  !
end module
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
