! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Compilation with IFORT:
!  ifort -xP -O3 -ipo -vms -c m_ctes.f90
!  ifort -xP -O3 -ipo -vms -c m_temps.f90
!  ifort -xP -O3 -ipo -vms -c m_tools.f90
!  ifort -xP -O3 -ipo -vms -c m_gauss0.f90
!  ifort -xP -O3 -ipo -vms -c m_krylov1.f90
!  ifort -xP -O3 -ipo -vms -c dfom.f90
!  ifort -xP -O3 -ipo -vms -o dfom_s.exe *.o
!
! Compilation with G95:
!  g95 -c m_ctes.f90
!  g95 -c m_temps.f90
!  g95 -c m_tools.f90
!  g95 -c m_gauss0.f90
!  g95 -c m_krylov1.f90
!  g95 -c dfom.f90
!  g95 -o dfom.exe *.o
!
! Compilation with (ADAPTOR + MPICH):
!  adaptor -settings
!  adaptor -hpf -dm -free -v -c m_ctes.f90
!  adaptor -hpf -dm -free -v -c m_temps.f90
!  adaptor -hpf -dm -free -v -c m_tools.f90
!  adaptor -hpf -dm -free -v -c m_gauss0.f90
!  adaptor -hpf -dm -free -v -c m_krylov1.f90
!  adaptor -hpf -dm -free -v -c dfom.f90
!  adaptor -hpf -dm -free -v -o dfom_p.exe *.o
!
! Running with MPICH:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpdringtest
!  mpiexec -machinefile ~/machi.dat -np 2 dfom_p.exe
!  mpiexec -machinefile ~/machi.dat -1 -np 4 dfom_p.exe
!  mpdallexit
!  pdsh -a hostname
!
! The same running but with MPI (old):
!  mpirun -machinefile ~/machi.dat -np 2 dfom_p.exe
!  mpirun -machinefile ~/machi.dat -nolocal -np 4 dfom_p.exe
!
! Some running details:
!  si n = 8 000 entonces m = n^2, RAM = m * 8 /1e6 =  512 [Mbytes]
!  pero como aqui hay una copia de A se tiene ram = 1,024 [Gbytes]
!
! output example:
!  dfom 
!  number of unknowns ; n = ? 
!  max diff =  5.83895598538220E-10
!  time   =  576.0575  seconds
!  mflops =  374.9626  on  8  processors
!  mflops =  46.87032  for one processor
! -----------------------------------------------------------------
program  dfom
  use m_ctes
  use m_temps
  use m_tools1
  use m_krylov1
  implicit none
  integer (iin) :: j
  real (idp), dimension (:,:), allocatable ::  a
  real (idp), dimension (:)  , allocatable ::  b, x
  !hpf$ distribute (block,*) :: a
  !hpf$ align (i)   with a (i,*) :: b
  integer (iin) :: n, m
  integer (iin) :: i, k, falla = 1
  !
  integer (iin) :: n_unknowns, m_subspace
  namelist /krylov_basic_data/ n_unknowns, m_subspace
  !
  ! lee del disco
  open  (1, file = "krylov_basic.dat", status = "old", err = 100)
  read  (1,nml = krylov_basic_data)
  write (*,nml = krylov_basic_data)  
  close (1, status = "keep")
  n = n_unknowns
  m = m_subspace
  if (m > n .or. m < 1) m = n 
  !
  !carteles
  write (*,*)
  write (*,*) "solution of a SEL : FOM "
  write (*,*) "number of unknows           ; n = ", n
  write (*,*) "size of the Krylov subspace ; m = ", m
  !
  !aloca sistema
  ier = 0
  allocate ( a (1:n,1:n), stat = ier (1) )
  allocate ( b (1:n)    , stat = ier (2) )
  allocate ( x (1:n)    , stat = ier (3) )
  if ( any (ier .ne. 0) ) call errata (ier(1:3), "aloca" )
  !
  !define un SEAL A X = B donde A es la matriz del sistema,
  !X, B matrices solucion y carga, respectivamente. 
  do  j = 1, n
    call  random_number (x)
    a (1:n,j) = x (1:n)
  end do
  call  random_number (x)
  b = x
  !
  !hpf$ independent
  forall (i=1:n)  a (i,i) = a (i,i) + dble (n)
  !
  !solucion fom
  call  dsolve_fom (a, b, m)
  !
  !eventual impresion
  ! print *
  ! print *, "matriz A "
  ! do  i = 1, n
  !   print *, real ( a (i,:) )
  ! end do ! i
  ! print *
  ! print *, "carga  b "
  ! do  i = 1, n
  !   print *, real ( x (i,:) )
  ! end do ! i
  ! print *
  ! print *, "soluc       x "
  ! do  i = 1, n
  !   print *, real ( b (i) )
  ! end do ! i
  !
  !dloca 
  ier = 0
  deallocate ( x, stat = ier (1) )
  deallocate ( b, stat = ier (2) )
  deallocate ( a, stat = ier (3) )
  if (any (ier .ne. 0))  call errata (ier(1:3), "dloca" )
  !
  falla = 0
  100 if (falla .ne. 0) stop "error: namelist file was not found ..."
  !
  print *
  print *, "fine ... "
  !
  ! formatos
  110 format (a)
  120 format (1x, i4, 1x, i20, 1x, e24.12)
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
