! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Compilation with IFORT:
!  ifort -xP -O3 -ipo -vms -c m_ctes.f90
!  ifort -xP -O3 -ipo -vms -c m_temps.f90
!  ifort -xP -O3 -ipo -vms -c m_tools.f90
!  ifort -xP -O3 -ipo -vms -c m_gauss0.f90
!  ifort -xP -O3 -ipo -vms -c zgauss0m.f90
!  ifort -xP -O3 -ipo -vms -o zgauss0m_s.exe *.o
!
! Compilation with G95:
!  g95 -c m_ctes.f90
!  g95 -c m_temps.f90
!  g95 -c m_tools.f90
!  g95 -c m_gauss0.f90
!  g95 -c gauss1.f90
!  g95 -o gauss1.exe *.o
!
! -----------------------------------------------------------------
program  zgauss0m
  use m_ctes
  use m_temps
  use m_tools0
  use m_gauss0
  implicit none
  integer (iin), parameter :: ncut = 1000
  integer (iin) :: j
  complex (idp), dimension (:,:), allocatable ::  a,  b, r
  complex (idp), dimension (:,:), allocatable :: a0, b0
  real    (idp), dimension (:)  , allocatable ::  x, y, u, v
  !
  integer (iin) :: n, m
  integer (iin) :: i, k, falla = 1
  !
  integer (iin) :: n_unknowns, n_loads
  namelist /gauss_basic_data/ n_unknowns, n_loads
  !
  ! lee del disco
  open  (1, file = "gauss_basic.dat", status = "old", err = 100)
  read  (1,nml = gauss_basic_data)
  write (*,nml = gauss_basic_data)  
  close (1, status = "keep")
  n = n_unknowns
  m = n_loads
  !
  !carteles
  write (*,*)
  write (*,*) "Gauss solution of a SEL with dynamic RAM "
  write (*,*) "number of unknows ; n = ", n
  write (*,*) "number of loads   ; m = ", m
  !
  !aloca sistema
  ier = 0
  allocate ( a (1:n,1:n), stat = ier (1) )
  allocate ( b (1:n,1:m), stat = ier (2) )
  allocate (a0 (1:n,1:n), stat = ier (3) )
  allocate (b0 (1:n,1:m), stat = ier (4) )
  allocate ( r (1:n,1:m), stat = ier (5) )
  allocate ( x (1:n)    , stat = ier (6) )
  allocate ( y (1:n)    , stat = ier (7) )
  allocate ( u (1:m)    , stat = ier (8) )
  allocate ( v (1:m)    , stat = ier (9) )
  if ( any (ier .ne. 0) ) call errata (ier(1:9), "aloca" )
  !
  !define un SEAL A X = B donde A es la matriz del sistema,
  !X, B matrices solucion y carga, respectivamente. 
  do  j = 1, n
    call  random_number (x)
    call  random_number (y)
    a (1:n,j) = cmplx ( x (1:n) , y (1:n) )
    call  random_number (u)
    call  random_number (v)
    b (j,1:m) = cmplx ( u (1:m) , v (1:m) )
  end do
  !hpf$ independent
  forall (i=1:n)  a (i,i) = a (i,i) + cmplx (dble (n), 0.0_idp)
  !
  !copia para posterior verificacion de los resultados
  if (n < ncut) then
    a0 = a
    b0 = b
  end if
  !
  !solucion gaussiana
  call  solve_bm (a, b, imprime = .true.)
  !
  !verifica matriz solucion X con R = B - A X
  if (n < ncut) then
    print *
    print *, "verification; r = A x - b "
    r = matmul (a0,b) - b0
    u = sqrt (sum (abs (r * r), dim = 1))
    print *, "maxval (r) = ", maxval (u)
  end if
  !
  !eventual impresion
    print *
    print *, "matriz dato A "
    do  i = 1, n
      print *,  a0 (i,:)
    end do ! i
    print *
    print *, "carga B "
    do  i = 1, n
      print *,  b0 (i,:)
    end do ! i
    print *
    print *, "matriz factorizada A "
    do  i = 1, n
      print *, a (i,:)
    end do ! i
    print *
    print *, "solucion X "
    do  i = 1, n
      print *, b (i,:)
    end do ! i
    print *
    print *, "residuo  R "
    do  i = 1, n
      print *, r (i,:)
    end do ! j
  !
  !dloca 
  ier = 0
  deallocate ( v, stat = ier (9) )
  deallocate ( u, stat = ier (8) )
  deallocate ( y, stat = ier (7) )
  deallocate ( x, stat = ier (6) )
  deallocate ( r, stat = ier (5) )
  deallocate (b0, stat = ier (4) )
  deallocate (a0, stat = ier (3) )
  deallocate ( b, stat = ier (2) )
  deallocate ( a, stat = ier (1) )
  if (any (ier .ne. 0))  call errata (ier(1:9), "dloca" )
  !
  falla = 0
  100 if (falla .ne. 0) stop "error: namelist file was not found ..."
  !
  print *
  print *, "fine ... "
  !
  ! formatos
  110 format (a)
  120 format (1x, i4, 1x, i20, 1x, e24.12)
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
