! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Problema del Conjunto de Mandelbrot
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Compilation with INTEL:
!  ifort -vms -c mandel.f90
!  ifort -vms -o mandel.exe1 *.o
!
! Compilation with GNU:
!  g95 -c circulo.f90
!  g95 -o mandel.exe2 *.o
!
! Compilation with (ADAPTOR + MPICH):
!  adaptor -hpf -dm -free -c mandel.f90
!  adaptor -hpf -dm -free -o mandel.exe3 *.o
!
! Running with MPICH:
!  mpdboot -n 4 -f ~/mpd.hosts
!  mpdtrace
!  mpdringtest
!  mpiexec -machinefile ~/machi.dat -np 4 mandel.exe3
!  mpiexec -machinefile ~/machi.dat -1 -np 4 mandel.exe3
!  mpdallexit
!
! Para ver Salida Grafica Color con
!    xv -expand 0.5 mandel_co.pgm &
!
! Para ver Salida Grafica Blanco y Negro con
!    xv -expand 0.5 mandel_bw.pgm &
!
! ------------------------------------------------------------------
! Programa HPF para el conjunto de Mandelbrot.
! Ref. Liverpool HPF course, Marshall A.C., 1997.
! ------------------------------------------------------------------
program mandel
  implicit none
  integer, parameter :: n = 2048
  integer, parameter :: resolucion = 516
  integer, dimension (n,n) :: color
  real   , dimension (n,n) :: zr, zi, cr, ci, ur, ui
  !hpf$ distribute (block,block) :: zr, zi, cr, ci, ur, ui, color
  real    :: s10, s11, s15, s20, s25, s30, s40, s50, s60
  integer :: i, j, k, ini, fin
  integer :: c, rojo, verde, azul
  !
  write (*,*)
  write (*,*)" Conjunto de Mandelbrot ... "  
  write (*,*)" n = ", n
  write (*,*)
  !
  ! inicializa
  forall (j=1:n, i=1:n)
    zr (i,j) = real (i-1) / real (n-1)
    zi (i,j) = real (j-1) / real (n-1)
  end forall
  cr = zr       ; ci = zi
  ur = zr * zr  ; ui = zi * zi
  color = 0
  call system_clock (ini)
  !
  ! lazo
  do   k = 0, resolucion
    where ( (ur + ui) .le. 4.0 )
      ur = zr * zr
      ui = zi * zi
      zi = 2.0 * zr * zi + ci
      zr = (ur - ui)     + cr
      color = k
    end where
  end do
  call system_clock (fin)
  !
  ! archivo gv monocromatico. Usar gv -expand 0.5 mandel_bw.pgm
  open  (10, file = 'mandel_bw.pgm', status = 'unknown')
  write (10,100)  n, n, resolucion
  write (10,110)  color
  close (10, status = 'keep')
  !
  ! archivo gv color. Usar gv -expand 0.5 mandel2_co.pgm
  s10 = 0.039216 * resolucion ! 10
  s11 = 0.043137 * resolucion ! 11
  s15 = 0.058824 * resolucion ! 15
  s20 = 0.078431 * resolucion ! 20
  s25 = 0.098039 * resolucion ! 25
  s30 = 0.117650 * resolucion ! 30
  s40 = 0.156860 * resolucion ! 40
  s50 = 0.196080 * resolucion ! 50
  s60 = 0.235290 * resolucion ! 60
  !
  open  (11, file = 'mandel_co.pgm', status = 'unknown')
  write (11,120)  n, n, resolucion
  do  i = 1, n 
  do  j = 1, n
    c = color (i,j)
    if     ( c .eq. resolucion ) then
      rojo = 0       ; verde = 0       ; azul = 0 
    elseif ( c .ge.   0 .and. c .le. s10 ) then
      rojo = c * s25 ; verde = c * s20 ; azul = c * s10 
    elseif ( c .ge. s11 .and. c .le. s15 ) then
      rojo = c * s60 ; verde = c * s15 ; azul = c * s30
    else
      rojo  = resolucion / 1
      azul  = resolucion / 2
      verde = resolucion / 4 
    end if
    write (11,130) rojo, verde, azul
  end do
  end do
  close (11, status = 'keep')
  !
  ! formatos
  100 format ('P2' / i5, 2x, i5 / i5)
  110 format ( 11 (1x, i5) )
  120 format ('P3' / i5, 2x, i5 / i5)
  130 format ( 11 (1x, i5) )
end program
! ==================================================================
