! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Aproximacion de numero pi
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Compilation with INTEL:
!  ifort -vms -c m_ctes.f90
!  ifort -vms -c m_temps.f90
!  ifort -vms -c pi.f90
!  ifort -vms -o pi.exe1 *.o
!
! Compilation with GNU:
!  g95 -c m_ctes.f90
!  g95 -c m_temps.f90
!  g95 -c pi.f90
!  g95 -o pi.exe2 *.o
!
! Compilation with (ADAPTOR + MPICH):
!  adaptor -hpf -dm -free -c m_ctes.f90
!  adaptor -hpf -dm -free -c m_temps.f90
!  adaptor -hpf -dm -free -c pi.f90
!  adaptor -hpf -dm -free -o pi.exe3 *.o
!
! Running with MPICH:
!  mpdboot -n 4 -f ~/mpd.hosts
!  mpdtrace
!  mpdringtest
!  mpiexec -machinefile ~/machi.dat -np 2 pi.exe3
!  mpiexec -machinefile ~/machi.dat -1 -np 4 pi.exe3
!  mpdallexit
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program  aproxima_pi
 use m_ctes
 use m_temps
 integer (iin), parameter :: p = 8, n = 10**p  ! nro de intervalos
 real    (idp), parameter :: h = 1.0d0/n        ! ancho 
 real    (idp), dimension (n) :: area
 real    (idp) :: f, x, pi1
 integer (iin) :: m=1, i
 !hpf$ distribute (block) :: area
 !
 ! inicia cronometro;  
 call system_clock (count = h1)   ! t1 = second ()
 !
 ! tarea
 !hpf$ independent, new (f,x)
 do i = 1, n 
   x = h * (1.0*i - 0.5d0)
   f = 4.0d0 / (1.0d0 + x * x)
   area (i) = h * f               ! deberia ser f (x) pero no-anda
 end do
 pi1 = sum (area)
 !
 ! consulta cronometro
 call system_clock (count = h2)   ! t2 = second ()
 time (1) = h2 - h1
 !
 ! salida
 write (*,*)
 write (*,*) "p  = ", p
 write (*,*) "n  = ", n
 write (*,*) "pi = ", pi1
 call timer (n, m, "pi")
 !
 !contains
  !
  !pure function f (x)
  ! real (idp), intent (in) :: x
  ! real (idp) :: f
  ! f = 4.0d0 / (1.0d0 + x * x)
  !end function
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
