! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! Compilation with (adaptor + mpich):
!  adaptor -hpf -dm -v -free -o prime_basic.exe prime_basic.f90
!
! Running with mpich:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 2 prime_basic.exe
!  mpdallexit
!
! Common output:
!    threads =            1
!    cblocks =            1
!    there are      5761455  primes until    100000000
!
! Specific output:
!
!  mpiexec -machinefile ~/machi.dat -np 1 prime_basic.exe
!    nodes   =            1
!    time needed :    7.388600    
!
!  mpiexec -machinefile ~/machi.dat -np 2 prime_basic.exe
!    nodes   =            2
!    time needed :    3.975300    
!
!  mpiexec -machinefile ~/machi.dat -np 4 prime_basic.exe
!    nodes   =            4 
!    time needed :    2.112800    
!
!  mpiexec -machinefile ~/machi.dat -np 8 prime_basic.exe
!    nodes   =            8 
!    time needed :    1.189800    
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! data parallel source program for sieving primes
program prime_basic
  implicit none
  integer, parameter :: n1 = 10000
  integer, parameter :: n  = n1 * n1
  logical, dimension (:), allocatable :: a
  !hpf$ distribute (block) :: a
  integer :: k, s
  integer :: h1, h2, h3, ier = 0
  real    :: time
  !
  print *
  print *, "counting primes in range 2 to n, normal version: "
  print *, "with communication to find out the number to be sieved"
  print *, "nodes   = ", number_of_nodes   ()
  print *, "threads = ", number_of_threads ()
  print *, "cblocks = ", number_of_cblocks ()
  ! 
  allocate ( a (n), stat = ier)
  if (ier .ne. 0) stop " error aloca: a "
  ! 
  call system_clock (count=h1)
  !
  a = .true.                 ! presuponemos que todos son primos
  a (1) = .false.            ! pero 1 no es un numero primo
  k = 2                      ! pero 2 lo es y desde ahi empezamos
  do while (k*k .le. n)      ! cota inferior teorica para buscar
    a (k*k:n:k) = .false.    ! los multiplos de "k" no son primos
    k = k + 1                ! pasamos al sgte numero
    do while (.not. a (k))   ! busca el sgte numero no marcado
      k = k + 1
    end do
  end do
  s = count (a)              ! cuenta solo los que tienen TRUE
  !
  call system_clock (count=h2, count_rate=h3)
  h2 = h2 - h1
  time = float (h2) / float (h3)
  !
  print *, "there are ", s, " primes until ", n
  print *, "time needed : ", time
  !
  deallocate (a, stat = ier)
  if (ier .ne. 0) stop " error dloca: a "
  !
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
