! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! compilation with (adaptor + mpich):
!  adaptor -hpf -dm -v -free -o prime_better1.exe prime_better.f90
!  adaptor -hpf -dm -v -cb -free -o prime_better2.exe prime_better.f90
!
! running with mpich:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 2 prime_better1.exe
!  mpiexec -machinefile ~/machi.dat -np 8 prime_better2.exe -nb 400
!  mpdallexit
!
! Common output:
!    threads =            1
!    cblocks =            1
!    there are      5761455  primes until    100000000
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Specific outputs without -cb:
!   mpiexec -machinefile ~/machi.dat -np 1 prime_better1.exe
!     nodes   =            1
!     time needed :    5.089800    
!   mpiexec -machinefile ~/machi.dat -np 2 prime_better1.exe
!     nodes   =            2
!     time needed :    2.707800    
!   mpiexec -machinefile ~/machi.dat -np 4 prime_better1.exe
!     nodes   =            4
!     time needed :    1.377400    
!   mpiexec -machinefile ~/machi.dat -np 8 prime_better1.exe
!     nodes   =            8
!     time needed :   0.6808000    
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Specific outputs with -cb:
!   mpiexec -machinefile ~/machi.dat -np 1 prime_better2.exe -nb 400
!     nodes   =            1
!     time needed :    2.201400    
!   mpiexec -machinefile ~/machi.dat -np 2 prime_better2.exe -nb 400
!     nodes   =            2
!     time needed :    1.208800    
!   mpiexec -machinefile ~/machi.dat -np 4 prime_better2.exe -nb 400
!     nodes   =            4
!     time needed :   0.7175000    
!   mpiexec -machinefile ~/machi.dat -np 8 prime_better2.exe -nb 400
!     nodes   =            8
!     time needed :   0.4746000    
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! data parallel source program for sieving primes
program prime_better
  implicit none
  integer, parameter :: n1 = 10000
  integer, parameter :: n  = n1 * n1
  logical, dimension (:), allocatable :: a 
  logical, dimension (n1) :: a1    ! replicated
  !hpf$ distribute a (block)
  integer :: k, i, s
  integer :: h1, h2, h3, ier = 0
  real    :: time
  !
  print *
  print *, "counting primes in range 2 to n, improved version: "
  print *, "replicated computation of primes until sqrt (n)    "
  print *, "nodes   = ", number_of_nodes   ()
  print *, "threads = ", number_of_threads ()
  print *, "cblocks = ", number_of_cblocks ()
  ! 
  allocate ( a (n), stat = ier)
  if (ier .ne. 0) stop " error aloca: a "
  ! 
  call system_clock (count = h1)
  !
  !print *, "replicated sieving up to ", n1
  a1 = .true.
  a1 (1) = .false.
  k = 2
  do while (k*k .le. n1)
     a1 (k*k:n1:k) = .false.
     k = k + 1
     do while (.not. a1 (k))
       k = k + 1
     end do
  end do
  !
  !print *, "distributed sieving up to ", n
  s = 0
  a (1) = .false.
  !hpf$ parallel, reduction (s) begin
  a (2:n) = .true.
  do  k = 2, n1
    if (a1 (k)) then
      ! a (k*k:n:k) = .false.
      !hpf$ independent
      do i = k*k, n, k
        if ( a (i) )  a (i) = .false.
      end do
    end if
  end do
  !hpf$ independent
  do  i = 1, n
   if ( a (i) ) s = s + 1
  end do
  !hpf$ end parallel
  !
  call system_clock (count=h2, count_rate=h3)
  h2 = h2 - h1
  time = float (h2) / float (h3)
  !
  print *, "there are ", s, " primes until ", n
  print *, "time needed : ", time
  !
  deallocate (a, stat = ier)
  if (ier .ne. 0) stop " error dloca: a "
  !
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
