! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! compilation with (adaptor + mpich):
!  adaptor -hpf -dm -v -free -o prime_local1.exe prime_local.f90
!  adaptor -hpf -dm -v -cb -free -o prime_local2.exe prime_local.f90
!
! running with mpich:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 2 prime_local1.exe
!  mpiexec -machinefile ~/machi.dat -np 10 prime_local2.exe -nb 400
!  mpdallexit
!
! Common output:
!    threads =            1
!    cblocks =            1
!    there are      5761455  primes until    100000000
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Specific outputs without -cb:
!   mpiexec -machinefile ~/machi.dat -np 1 prime_local1.exe
!     nodes   =            1
!     time needed :    5.141800    
!   mpiexec -machinefile ~/machi.dat -np 2 prime_local1.exe
!     nodes   =            2
!     time needed :    2.718000    
!   mpiexec -machinefile ~/machi.dat -np 4 prime_local1.exe
!     nodes   =            4
!     time needed :    1.381100    
!   mpiexec -machinefile ~/machi.dat -np 8 prime_local1.exe
!     nodes   =            8
!     time needed :   0.6817000   
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Specific outputs with -cb:
!   mpiexec -machinefile ~/machi.dat -np 1 prime_local2.exe -nb 400
!     nodes   =            1
!     time needed :    1.818100    
!   mpiexec -machinefile ~/machi.dat -np 2 prime_local2.exe -nb 400
!     nodes   =            2
!     time needed :   0.8939000    
!   mpiexec -machinefile ~/machi.dat -np 4 prime_local2.exe -nb 400
!     nodes   =            4
!     time needed :   0.4150000    
!   mpiexec -machinefile ~/machi.dat -np 8 prime_local2.exe -nb 400
!     nodes   =            8
!     time needed :   0.2285000    
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! data parallel source program for sieving primes
program prime_local
  implicit none
  integer, parameter :: n1 = 10000
  integer, parameter :: n  = n1 * n1
  logical, dimension (n)  :: a 
  logical, dimension (n1) :: a1    ! replicated
  integer, dimension (number_of_processors ()) :: s1
  !hpf$ distribute (block) :: a, s1
  interface
    extrinsic (hpf_local) subroutine criba (a, n, s, a1, n1)
      integer :: n, n1
      logical, dimension (:) :: a
      integer, dimension (:) :: s
      !hpf$ distribute (block) :: a, s
      logical, dimension (n1) :: a1
    end subroutine
  end interface
  !
  integer :: i, k, s
  integer :: h1, h2, h3, ier = 0
  real    :: time
  !
  print *
  print *, "counting primes in range 2 to n, local HPF version: "
  print *, "with HPF local computation                          "
  print *, "nodes   = ", number_of_nodes   ()
  print *, "threads = ", number_of_threads ()
  print *, "cblocks = ", number_of_cblocks ()
  !
  call system_clock (count=h1)
  !
  !print *, 'replicated sieving up to ', n1
  a1 = .true.
  a1 (1) = .false.
  k = 2
  do while (k*k .le. n1)
     a1 (k*k:n1:k) = .false.
     k = k + 1
     do while (.not. a1 (k) )
       k = k + 1
     end do
  end do
  !
  !print *, 'distributed sieving up to ', n
  call criba (a, n, s1, a1, n1)
  s = sum (s1)
  !
  call system_clock (count=h2, count_rate=h3)
  h2 = h2 - h1
  time = float (h2) / float (h3)
  !
  print *, "there are ", s, " primes until ", n
  print *, "time needed : ", time
  !
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
extrinsic (hpf_local) subroutine criba (a, n, s, a1, n1)
  integer :: n, n1
  logical, dimension (:) :: a
  integer, dimension (:) :: s
  !hpf$ distribute (block) :: a
  !hpf$ distribute (block) :: s
  logical, dimension (n1) :: a1
  !
  integer :: s1
  integer :: lb, ub, lb1, ub1
  integer :: i, k
  !
  lb = lbound (a, 1)
  ub = ubound (a, 1)
  s1 = 0
  do  i = lb, ub
    a (i) = .true.
  end do
  if (lb == 1) a (1) = .false.
  do  k = 2, n1
    if ( a1 (k) ) then
      if (k*k < lb) then
        lb1 = k*k + ((lb - k*k - 1) / k + 1) * k
      else
        lb1 = k*k
      end if
      ub1 = min (ub, n)
      do i = lb1, ub1, k
        if ( a(i) )  a(i) = .false.
      end do
    end if
  end do
  do i = lb, ub
    if ( a (i) ) s1 = s1 + 1
  end do
  s (:) = s1
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
