! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Compilation with INTEL:
!  ifort -w -o primo.exe1 primo_hpf.f90
!
! Compilation with GNU:
!  g95 -w -o primo.exe2 primo_hpf.f90
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! ADAPTOR: compilation of a HPF input source file and FREE format.
!
! Some options:
!
! 1) HPF compilation for a single node: 
!    adaptor -hpf_1 -free -o primo_hpf.exe3 primo_hpf.f90
!
! 2) HPF compilation for Distributed Memory Machines using MPICH/MPI:
!    adaptor -hpf -dm -free -o primo_hpf.exe4 primo_hpf.f90
!    mpdboot -n 4 -f ~/mpd.hosts
!    mpiexec -machinefile ~/machi.dat -1 -np 3 primo_hpf.exe4
!    mpirun -machinefile ~/machi.dat -nolocal -np 3 primo_hpf.exe4
!
! 3) HPF compilation for Shared Memory Machines (SMM) using PThreads
!    adaptor -hpf -sm -free -o primo_hpf.exe5 primo_hpf.f90
!
! 4) HPF compilation for Cached Architectures
!    adaptor -hpf -cb -free -o primo_hpf.exe6 primo_hpf.f90
!    primo_hpf.exe6 -nb 2
!
! 5) HPF compilation for a cluster of SMP nodes using MPI and PThreads
!    adaptor -hpf -dm -sm -free -o primo_hpf.exe7 primo_hpf.f90
!    mpirun -machinefile ~/machi.dat -np 4 primo_hpf.exe7
!
! 6) OpenMP compilation for shared memory machines using PThreads
!    adaptor -openmp -free -o primo_hpf.exe8 primo_hpf.f90
!    export OMP_NUM_THREADS=2 ; echo $OMP_NUM_THREADS
!    primo_hpf.exe8
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program primo
  implicit none
  logical, dimension (:), allocatable :: a
  !hpf$ distribute (block) :: a
  integer :: m=1, n, h, k, ier=0
  integer :: h1, h2, h3, p
  real    :: time
  character (13) :: arch = "primo_hpf.tim"
  character (60) :: s
  !
  write (*,*)
  write (*,*) "input n for counting primes in range 2 to n "
  write (*,*) "n ? " ; read (*,*) n
  !
  call system_clock (count=h1)
  ! 
  allocate (a(n), stat = ier)
  if (ier .ne. 0) stop " error aloca: a "
  ! 
  a = .true.                 ! presuponemos que todos son primos
  a (1) = .false.            ! pero 1 no es un numero primo
  k = 2                      ! pero 2 lo es y desde ahi empezamos
  do 
    p = k * k                ! cota inferior teorica para buscar
    if (p > n) exit          ! no hace falta mas revisar
    a (p:n:k) = .false.      ! los multiplos de "k" no son primos
    k = k + 1                ! pasamos al sgte numero
    do
      if (a (k)) exit        ! busca el sgte numero no marcado
      k = k + 1
    end do
  end do
  h = count (a)              ! cuenta solo los que tienen TRUE
  !
  call system_clock (count=h2,count_rate=h3)
  time = float (h2 - h1) / float (h3)
  !
  !nro de procesadores
  p = 1                                ! secuencial
  p = number_of_processors ()          ! solo en HPF
  !
  !resumen a disco
  write (*,*)
  write (*,100) " archivo timer: " // arch
  write (*,*)
  s="  p            n m   time1           time2           time_all"
  write (*,100) s
  !
  open (1, file = arch         , &
           status = "unknown"  , &
           position = "append")
  write (*,110)  p, n, m, time
  write (1,110)  p, n, m, time
  close (1, status = 'keep')
  write (*,*)
  !  
  write (*,*)
  write (*,*) "hay",h," primos hasta", n
  write (*,*) "wall time = ", time 
  write (*,*)
  !
  ! formatos
  100 format (a)
  110 format (1x, i2, 1x, i12, 1x, i2, 3 (1x, e15.7))
  !
  deallocate (a, stat = ier)
  if (ier .ne. 0) stop " error dloca: a "
  !
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
