! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! Compilation with (ADAPTOR + MPICH):
!  adaptor -hpf -dm -free -c loc_tasklib.f90
!  adaptor -hpf -dm -free -o loc_tasklib.exe loc_tasklib.o
!  adaptor -hpf -dm -free -o loc_tasklib.exe loc_tasklib.f90
!
! Running with MPICH:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 2 loc_tasklib.exe
!  mpdallexit
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
!     Test Program to check the task library functionality in LOCAL
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program  tareas_locales
  implicit none
  integer :: np
  integer, dimension (1:number_of_processors ()) :: ier
  !hpf$ distribute (block) :: ier
  integer errata
  np = number_of_processors ()
  if (np < 2) then
     print *, "application must run on at least 2 processors"
  else
     ier = 0
     print *
     print *, "Test HPF_TASK_LIBRARY (local) on ",np," procs"
     print *, "============================================="
     print *
     call local (ier)
     errata = sum (ier)
     if (errata .eq. 0) then
        print *, "HPFLocalTaskLib NP =", np, ": all tests PASSED"
     else
        print *, "HPFLocalTaskLib NP =", np, ": FAILED"
     end if
  end if
  !
  write (*,*)
  write (*,*)"FINE ... "
  write (*,*)
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
!     LOCAL : SPMD code with message passing via TASK library
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
extrinsic (HPF_LOCAL) subroutine local (ier)
  implicit none
  use HPF_TASK_LIBRARY
  integer, dimension (:) :: ier
  !hpf$ distribute (block) :: ier
  integer :: siza, rango
  integer :: valor
  !
  call HPF_TASK_INIT ()
  call HPF_TASK_RANK (rank = rango)
  call HPF_TASK_SIZE (size = siza)
  print *, "I am ", rango, " of ", siza
  !
  ! PING PONG Test 
  if     (rango == 1) then
     call tarea1 (10)
  elseif (rango == 2) then
     call tarea2 (10, ier)
  end if
  !
  ! global TEST 
  print *, "bcast between processors in local mode"
  valor = 2
  if (rango == 1) valor = 5
  call HPF_BCAST (valor)
  if (valor .ne. 5) ier = 1
  !
  call HPF_TASK_EXIT ()
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
!     TASK1 : task that will send data 
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine tarea1 (n)
  implicit none
  use HPF_TASK_LIBRARY
  integer, intent (in)   :: n
  integer, dimension (n) :: a, ra, a1
  real   , dimension (n) :: s
  integer, dimension (n,n) :: b
  !
  print *, "send/recv of a scalar"
  call HPF_SEND (data = n, dest = 2)
  !
  print *, "send distributed => recv distributed"
  a = 1
  call HPF_SEND (data = a, dest = 2)
  !
  ! now send an array for comparison
  call random_number (s)
  a = s * 100
  print *, "send A = ", a
  call HPF_SEND (data = a, dest = 2)
  print *, "send_recv "
  call HPF_SEND_RECV (send_data =  a, dest   = 2,  &
                      recv_data = a1, source = 2)
  !
  print *, "send_recv back"
  call HPF_SEND_RECV (send_data = a1, dest   = 2,  &
                      recv_data =  a, source = 2)
  !
  print *, "send/recv two-dimensional"
  b = spread (a, dim = 1, ncopies = n)
  call HPF_SEND (data = b, dest = 2)
  !
  print *, "send/recv two-dimensional with transpose"
  call HPF_SEND (data = b, dest = 2, order = [2, 1])
  !
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
!     TASK2 : task that will receive data 
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine tarea2 (n, ier)
  implicit none
  use HPF_TASK_LIBRARY
  integer, dimension (1)   :: ier
  integer                  :: n
  integer, dimension (n)   :: a, ra, z
  integer, dimension (n,n) :: b, res2, res2t
  integer :: errata = 0 , n1
  !
  a = 0
  call HPF_TASK_INIT ()
  call HPF_RECV (data = n1, source = 1)
  if (n1 .ne. n) then
    print *, "received illegal scalar val N"
    errata = errata + 1
  end if
  call HPF_RECV (data = a, source = 1)
  if (sum (a) .ne. n) then
    print *, "received illegal integer array A (1)"
    errata = errata + 1
  end if
  ! receive an array for comparison
  call HPF_RECV (data = z, source = 1)
  print *, "received A as Z = ", z
  print *, "send_recv "
  a = 0
  call HPF_SEND_RECV (send_data = z, dest = 1,  &
                      recv_data = a, source = 1)
  if (any (a .ne. z)) then
    print *, "received illegal integer array A (send_recv 1)"
    errata = errata + 1
  end if
  !
  call HPF_SEND_RECV (send_data = a, dest = 1,  &
                      recv_data = z, source = 1)
  if (any (a .ne. z)) then
    print *, "received illegal integer array A (send_recv 2)"
    errata = errata + 1
  end if
  !
  ! print *, "send/recv two-dimensional"
  res2 = spread (z, dim=1, ncopies = n)
  call HPF_RECV (data = B, source = 1)
  if (any (b .ne. res2)) then
    print *, "received illegal integer array B"
    print *, "received: ", b
    print *, "expected: ", res2
    errata = errata + 1
  end if
  !
  print *, "send/recv two-dimensional with transpose"
  call HPF_RECV (data = b, source = 1)
  res2t = transpose (res2)
  if (any (b .ne. res2t)) then
    print *, "received illegal integer array B (transpose)"
    print *, "received: ", b
    print *, "expected: ", res2t
    errata = errata + 1
  end if
  print *, "TASK2 : errors = ", errata
  ier (1) = errata
  !
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
