! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Compilation with (ADAPTOR + MPICH):
!  adaptor -hpf -dm -free -c tasklib.f90
!  adaptor -hpf -dm -free -o tasklib.exe tasklib.o
!  adaptor -hpf -dm -free -o tasklib.exe tasklib.f90
!
! Running with MPICH:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 2 tasklib.exe
!  mpdallexit
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
!     Test Program to check the task library functionality
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program comunica_tasks
  implicit none
  integer np, n1, n2, errata
  np = number_of_processors ()
  if (np < 2) then
     print *, "application must run on at least 2 processors"
     stop
  end if
  n1 = np/2
  n2 = np - n1
  print *
  print *, "Test HPF_TASK_LIBRARY on ",np," procs"
  print *, "====================================="
  print *
  call tareas (np, n1, n2, errata)
  if (errata .eq. 0) then
     print *, "HPFtaskLib NP =", np, ": all tests PASSED"
    else
     print *, "HPFtaskLib NP =", np, ": at least one test FAILED"
  end if
  !
  write (*,*)
  write (*,*)"FINE ... "
  write (*,*)
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
!     TASKING: starts two concurrent data parallel tasks   
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine  tareas (p, p1, p2, errata)
  implicit none
  ! runs two data parallel tasks on P1 and P2 processors
  integer, intent (in)   :: p, p1, p2, errata
  integer, parameter     :: n = 4             ! problem size
  integer, dimension (1) :: ier
  !hpf$ distribute ier (block) onto proc (p:p)
  !hpf$ processors proc (P)
  print *, "run TASK1 on ", 1,   " - ", p1
  print *, "run TASK2 on ", p1+1," - ", p
  ! define a task region, otherwise home will be ignored
  !hpf$ independent task_region
  !hpf$   on (proc(1:p1))  , resident
            call  tarea1 (n)
  !hpf$   on (proc(p1+1:p)), resident
            call  tarea2 (n, ier)
  !hpf$ end task_region
  errata = ier (1)
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
!     TASK1 : task that will send data 
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine  tarea1 (n)
  implicit none
  use HPF_TASK_LIBRARY
  integer, intent (in)       :: n
  integer, dimension (n)     :: a, ra, a1
  real   , dimension (n)     :: r
  !hpf$ distribute (block)   :: a, a1
  integer, dimension (n,n)   :: b
  !hpf$ distribute (*,block) :: b
  integer :: kontrol
  !
  call HPF_TASK_INIT ()
  !
  print *, "send/recv of a scalar"
  call HPF_SEND (data = n, dest = 2)
  !
  a = 1
  print *, "send distributed => recv distributed"
  call HPF_SEND (data = a, dest = 2)
  !
  ! now send an array for comparison
  call random_number (r)
  a = r * 100
  print *, "send A = ", a
  call HPF_SEND (data = a, dest = 2)
  !
  print *, "send replicated => recv distributed"
  ra = a
  call HPF_SEND (data = ra, dest = 2)
  !
  print *, "send distributed => recv replicated"
  call HPF_SEND (data =  a, dest = 2)
  !
  print *, "send replicate => recv replicated"
  call HPF_SEND (data = ra, dest = 2)
  !
  print *, "send/recv with REQUEST"
  call HPF_SEND_INIT (data = a, dest = 2, request = kontrol)
  call HPF_TASK_COMM (kontrol)
  !
  print *, "send_recv "
  call HPF_SEND_RECV (send_data =  a, dest   = 2,  &                     
                      recv_data = a1, source = 2)
  !
  print *, "send_recv back"
  call HPF_SEND_RECV (send_data = a1, dest   = 2,  &
                      recv_data =  a, source = 2)
  !
  print *, "send/recv two-dimensional"
  b = spread (a, dim=1, ncopies=n)
  call HPF_SEND (data = b, dest = 2)
  !
  print *, "send/recv two-dimensional with transpose"
  call HPF_SEND (data = b, dest = 2, order = [2, 1])
  !
  call HPF_TASK_EXIT ()  
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
!     TASK2 : task that will receive data 
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine tarea2 (n, ier)
  implicit none
  use HPF_TASK_LIBRARY
  integer, dimension (1)     :: ier
  integer                    :: n
  !hpf$ inherit              :: ier
  integer, dimension (n)     :: a, ra, z
  !hpf$ distribute (block)   :: a, z
  integer, dimension (n,n)   :: b, res2, res2t
  !hpf$ distribute (*,block) :: b, res2, res2t
  integer errata = 0, kontrol, n1
  !
  a = 0
  call HPF_TASK_INIT ()
  !
  call HPF_RECV (data = n1, source = 1)
  if (n1 .ne. n) then
      print *, "received illegal scalar val n"
      errata = errata + 1
  end if
  !
  call HPF_RECV (data = a, source = 1)
  if (sum (a) .ne. n) then
      print *, "received illegal integer array A (1)"
      errata = errata + 1
  end if
  !
  ! receive an array for comparison
  call HPF_RECV (data = z, source = 1)
  print *, "received A as Z = ", z
  !
  ! print *, "send replicated => recv distributed"
  call HPF_RECV (data = a, source = 1)
  if (any (a .ne. z)) then
      print *, "received some illegal integer array A (2)"
      errata = errata + 1
  end if
  !
  ! print *, "send distributed => recv replicated"
  ra = 0
  call HPF_RECV (data = ra, source = 1)
  if (any (ra .ne. z)) then
      print *, "received some illegal integer array RA (1)"
      errata = errata + 1
  end if
  !
  ! print *, "send replicated => recv replicated"
  ra = 0
  call HPF_RECV (data = ra, source = 1)
  if (any (ra .ne. z)) then
      print *, "received some illegal integer array RA (2)"
      errata = errata + 1
  end if
  !
  ! print *, "send/recv with REQUEST"
  a = 0
  call HPF_RECV_INIT (data = a, source = 1, request = kontrol)
  call HPF_TASK_COMM (kontrol)
  if (any (a .ne. z)) then
     print *, "received some illegal integer array A (request)"
     errata = errata + 1
  end if
  !
  print *, "send_recv "
  a = 0
  call HPF_SEND_RECV (send_data = z, dest   = 1,  &
                      recv_data = a, source = 1)
  if (any (a .ne. z)) then
     print *, "received some illegal integer array A (send_recv 1)"
     errata = errata + 1
  end if
  !
  call HPF_SEND_RECV (send_data = a, dest   = 1,  &
                      recv_data = z, source = 1)
  if (any (a .ne. z)) then
     print *, "received illegal integer array A (send_recv 2)"
     errata = errata + 1
  end if
  !
  ! print *, "send/recv two-dimensional"
  res2 = spread (z, dim = 1, ncopies = n)
  call HPF_RECV (data = b, source = 1)
  if (any (b .ne. res2)) then
     print *, "received illegal integer array B"
     print *, "received: ", b
     print *, "expected: ", res2
     errata = errata + 1
  end if
  !
  print *, "send/recv two-dimensional with transpose"
  call HPF_RECV (data = b, source = 1)
  res2t = transpose (res2)
  if (any (b .ne. res2t)) then
     print *, "received illegal integer array B (transpose)"
     print *, "received: ", b
     print *, "expected: ", res2t
     errata = errata + 1
  end if
  print *, "TASK2 : errors = ", errata
  ier (1) = errata
  call HPF_TASK_EXIT ()
  !
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
