! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Problema del Juego de la Vida
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Compilation with INTEL:
!  ifort -vms -c vida.f90
!  ifort -vms -o vida.exe1 *.o
!
! Compilation with GNU:
!  g95 -c circulo.f90
!  g95 -o vida.exe2 *.o
!
! Compilation with (ADAPTOR + MPICH):
!  adaptor -hpf -dm -free -c vida.f90
!  adaptor -hpf -dm -free -o vida.exe3 *.o
!
! Running with MPICH:
!  mpdboot -n 4 -f ~/mpd.hosts
!  mpdtrace
!  mpdringtest
!  mpiexec -machinefile ~/machi.dat -np 4 vida.exe3
!  mpiexec -machinefile ~/machi.dat -1 -np 4 vida.exe3
!  mpdallexit
!
! Para ver alguna "instantanea", e.g. la #10
!    xv -expand 0.5 gvida.010.pgm &
!
! Opcional:
!    a) Convertir las "instantaneas" *.pgm en *.gif con:
!       for f in *.pgm
!        do echo convert $f `basename $f /pgm`.gif ;
!                convert $f `basename $f /pgm`.gif ;
!       done
!    b) Crear el unico archivo de animacion (de pequenio tamanio
!       porque solo guarda las diferencias entre las fotos):
!       anim.pl -p gvida -t
!       anim.pl -p gvida
!    c) Ejecutar la animacion con: gifview gvida_anim.gif &
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Programa HPF para simular el juego de la vida en un tablero 2D
! Animacion con xv -expand 10 -wait 0.1 -rw *.pgm
! Ref. Liverpool HPF course, Marshall A.C., 1997.
! ------------------------------------------------------------------
program vida
  implicit none
  ! 
  !parametros del "experimento biologico"
  integer, parameter :: n_generaciones = 100
  integer, parameter :: n = 64 ! tablero 2D
  integer, parameter :: h = 16 ! patron inicial
  !
  !eventual impresion txt (mejor evitarla pues es MUY LENTA !!)
  integer, parameter :: cmax =  70 ! max col.impr.
  logical, parameter :: imprime = .false.
  !
  !declara y distribuye datos
  integer, dimension (n,n) :: tablero
  integer, dimension (n,n) :: vecinos
  !hpf$ distribute (cyclic,cyclic) :: tablero, vecinos
  ! 
  integer        :: nx, ny, ntasks, nvivas
  integer        :: i, j, m, p
  character (16) :: pic, txt
  character (1)  :: s
  !
  !variables para cronometro
  real   , dimension (3) :: time
  integer                :: h1, h2, h3
  real   , parameter     :: cero = 0.0
  !
  !titulos
  ntasks = number_of_processors ()
  write (*,*)
  write (*,*)' juego de la vida con HPF y con salida a xv'
  write (*,*)' number_of_processors = ', ntasks
  !
  !inicia cronometro;  t1 = second ()
  time = cero
  call system_clock (count = h1, count_rate = h3)
  !
  ! inicializa tablero
  nx = size (tablero,1)
  ny = size (tablero,2)
  !hpf$  independent
  forall (i=1:nx, j=1:ny) tablero (i,j) = 0
  m = n / 2
  !hpf$  independent
  forall (i = m - h/2 + 1 : m + h/2 )  tablero (i,m) = 1
  !
  !tablero: impresion para "xv"
  write (pic,100) 0
  open  (10, file = pic, status = 'unknown')
  write (10,110)  nx, ny, 1
  write (10,120)  tablero
  close (10, status = 'keep') 
  ! 
  !tablero: eventual impresion txt (mejor evitarlo)
  if ( imprime .and. n .le. cmax ) then
    write (txt,90) 0
    open  (11, file = txt, status = 'unknown')
    do  i = 1, nx
    do  j = 1, ny
    s = '.'
    if ( tablero (i,j) .eq. 1 )  s = '*'
    write (11,80,advance='no') s
    end do ! j
    write (11,*)
    end do ! i
    close (11, status='keep') 
  end if
  !
  !inicia cronometro
  call system_clock (count = h2)     ! t2 = second ()
  time (1) = h2 - h1                 ! time (1) = t2 - t1
  call system_clock (count = h1)     ! t1 = second ()
  !
  !lazo sobre las generaciones
  generacion : do  p = 1, n_generaciones
    !vecinos
    vecinos =      tablero                        &
        + cshift ( tablero, shift = -1, dim = 1)  &
        + cshift ( tablero, shift = +1, dim = 1)
    vecinos =      vecinos                        &
        + cshift ( vecinos, shift = -1, dim = 2)  &
        + cshift ( vecinos, shift = +1, dim = 2)
    vecinos = vecinos - tablero
    !
    ! actualiza tablero segun reglas de juego
    where ( vecinos < 2 .or. vecinos > 3 )  tablero = 0
    where ( vecinos == 3 )                  tablero = 1
    !
    !tablero: impresion para xv
    write (pic,100) p
    open  (10, file = pic, status = 'unknown')
    write (10,110)  nx, ny, 1
    write (10,120)  tablero
    close (10, status = 'keep') 
    !
    !tablero: eventual impresion txt (evitarlo para n grande)
    if  (imprime .and. n .le. cmax) then
      write (txt,90) p
      open  (11, file = txt, status = 'unknown')
      write (11,'(i5)') p
      do  i = 1, nx
      do  j = 1, ny
      s = '.'
      if ( tablero (i,j) .eq. 1 )  s = '*'
      write (11,80,advance='no') s
      end do ! j
      write (11,*)
      end do ! i
      close (11, status='keep') 
    end if
  end do generacion
  !
  !cronometro
  call system_clock (count = h2)     ! t2 = second ()
  time (2) = h2 - h1                 ! time (2) = t2 - t1
  time (3) = time (1) + time (2)     ! total_time
  time = time / h3                   ! pasa de ciclos a seg
  !
  !numero de celulas vivas remanentes
  nvivas = sum (tablero, mask = tablero .eq. 1)
  write (*,*)
  write (*,*)' nro de celulas vivas     ; nvivas = ', nvivas
  write (*,*)' al cabo de la generacion ;      p = ', p
  !
  !imprime
  write (*,*)
  write (*,*)' tpo de inicial.  [s] ; t1 = ', real (time(1))
  write (*,*)' tpo de barrido   [s] ; t2 = ', real (time(2))
  write (*,*)' tiempo_total     [s] ; t3 = ', real (time(3))
  write (*,*)
  !
  !imprime en archivo historico
  open  (12, file = 'vida.tim', status = 'unknown')
  write (12,60)  ntasks, time (1:3)
  close (12, status = 'keep')
  !
  !formatos
  60  format (1x, i4, 3 (1x, e19.9) )
  70  format (i1)
  80  format (a1)
  90  format ('tvida', i3.3, '.txt')
  !formatos p/archivo xv. Usar xv -expand 10 -wait 0.1 -rw *.pgm
  100 format ('gvida.', i3.3, '.pgm')
  110 format ('P2' / i4 , 2x , i4 / i4)
  120 format ( 35 (1x, i1) )
end program
! ==================================================================
