#
# ADAPTOR versions 10.1 and 10.2. 
# Installation with the fortran compilers: 
#   g95
#   ifort
#   gfortran
#   g77 

# Verify compiler and version and that it works well
which g95
which ifort
which gfortran 
which g77

# Set dynamic libraries path and path: Intel case
if [ -z "${LD_LIBRARY_PATH}" ]
then
  export LD_LIBRARY_PATH="/opt/intel/fc/9.0/lib"
else
  export LD_LIBRARY_PATH="/opt/intel/fc/9.0/lib:$LD_LIBRARY_PATH"
fi
if [ -z "${PATH}" ]
then
  export PATH="/opt/intel/fc/9.0/bin"
else
  export PATH="/opt/intel/fc/9.0/bin:$PATH"
fi

# Set dynamic libraries path and path: g95 case
if [ -z "${LD_LIBRARY_PATH}" ]
then
  export LD_LIBRARY_PATH="/usr/local/g95/lib"
else
  export LD_LIBRARY_PATH="$LD_LIBRARY_PATH:/usr/local/g95/lib"
fi
if [ -z "${PATH}" ]
then
  export PATH="/usr/local/g95/bin"
else
  export PATH="$PATH:/usr/local/g95/bin"
fi

# environment variable for MPICH
#xport MPI_HOME="/usr/local/mpich2/1.0.3-ifc"       # mpich2-1.0.3-ifc
#xport MPI_HOME="/usr/local/mpich2/1.0.3-g95"       # mpich2-1.0.3-g95 
#xport MPI_HOME="/usr/local/mpich2/1.0.3-gfortran"  # mpich2-1.0.3-gfortran
export MPI_HOME="/usr/local/mpich2/1.0.4-ifc"       # mpich2-1.0.4-ifc
export MPI_INC="-I$MPI_HOME/include"
#xport MPI_LIB="-lc -L$MPI_HOME/lib -lmpichf90 -lmpich -lf95 -lm"   # g95
export MPI_LIB="-lc -L$MPI_HOME/lib -lmpichf90 -lmpichf90 -lmpich"  # ifc 
echo $MPI_HOME ; echo $MPI_LIB ; echo $MPI_INC

# Set dynamic libraries path and path: MPICH2
if [ -z "${LD_LIBRARY_PATH}" ]
then
  export LD_LIBRARY_PATH="$MPI_HOME/lib"
else
  export LD_LIBRARY_PATH="$MPI_HOME/lib:$LD_LIBRARY_PATH"
fi
if [ -z "${PATH}" ]
then
  export PATH="$MPI_HOME/bin:"
else
  export PATH="$MPI_HOME/bin:$PATH"
fi

# control
echo "LD_LIBRARY_PATH = $LD_LIBRARY_PATH"
echo "PATH = $PATH"

# some tests as root
cd /usr/local/paquetes/tests
mpif90 conftest.f90
mpif90 -o hola_par_f90.exe hola_par_f90.f90

# another tests as some user:
su -l pepe
echo $LD_LIBRARY_PATH  
echo $PATH  
which mpdboot
cd /usr/local/paquetes/tests
mpif90 -o hola_par_f90.exe hola_par_f90.f90
mpdboot -n 21 -f ~/mpd.hosts
mpdtrace
pdsh -a hostname
mpdringtest
mpiexec -machinefile ~/machi.dat -np 20 hola_f90.exe
mpdallexit
exit

# Install the MPICH2 package with the same F90 compiler that it will 
# be used with adaptor

# Set ADAPTOR version, compiler and mpich-version
export VER="10.1"; export FIL="adp_10.1.tar.gz" ; echo $VER ; echo $FIL
#xport VER="10.2"; export FIL="adp_10.2.tar.gz" ; echo $VER ; echo $FIL
#xport COM="g95-103" ; echo $COM
#xport COM="ifc-103" ; echo $COM
export COM="ifc-104" ; echo $COM

# Untar the adaptor source file
cd /usr/local
mkdir adaptor
cd /usr/local/adaptor ; pwd
mkdir $VER
cd $VER ; pwd
mkdir $COM ; ls -all
cd $COM ; pwd
tar -zxvf /usr/local/paquetes/adaptor_install/$FIL
cd ..
chgrp -R root $COM
chown -R root $COM
cd $COM ; pwd
set

# environment variable for PHOME
export PHOME=/usr/local/adaptor/$VER/$COM  ; echo $PHOME

# environment variables for the CC compiler
export CC=gcc ; echo $CC
export CFLAGS="-w -O" ; echo $CFLAGS

# environment variables for the F77 compiler
#
# a) g95 compiler:
export F77=mpif90 ; echo $F77
export F77_EXTEND="-ffixed-form, -ffixed-line-length-132" ; echo $F77_EXTEND 
export F77_NOWARN="-w" ; echo $F77_NOWARN
export F77_OPT="-O4 -ffree-line-length-huge" ; echo $F77_OPT 
export F77_FLAGS="-fPIC -I$MPI_HOME/include" ; echo $F77_FLAGS
export F77_MP=" " ; echo $F77_MP
#
# b) ifort compiler: NOTAR que -Wp DEBE ir primero
export F77=mpif90 ; echo $F77
export F77_EXTEND="-extend-source" ; echo $F77_EXTEND 
export F77_NOWARN="-w" ; echo $F77_NOWARN
export F77_OPT="-Wp,-xP,-O3,-ipo,-vms,-recursive" ; echo $F77_OPT 
export F77_FLAGS="-fPIC -I$MPI_HOME/include" ; echo $F77_FLAGS
export F77_MP="-auto" ; echo $F77_MP
#
# c) gfortran compiler: 
export F77=mpif90 ; echo $F77
export F77_EXTEND="-ffixed-form -ffixed-line-length-none" ; echo $F77_EXTEND 
export F77_NOWARN="-w" ; echo $F77_NOWARN
export F77_OPT="-ffree-line-length-none" ; echo $F77_OPT 
export F77_FLAGS="-fPIC -I$MPI_HOME/include" ; echo $F77_FLAGS
export F77_MP="fautomatic" ; echo $F77_MP
#
# e) g77 compiler:
export F77=mpif90 ; echo $F77
export F77_EXTEND="-Mextend" ; echo $F77_EXTEND 
export F77_NOWARN="-w" ; echo $F77_NOWARN
export F77_OPT="-recursive" ; echo $F77_OPT 
export F77_FLAGS="-fPIC -I$MPI_HOME/include" ; echo $F77_FLAGS
#
# e) pgi compiler:
export F77=pgf90 ; echo $F77
export F77_EXTEND=-Mextend ; echo $F77_EXTEND 
export F77_OPT="-Mrecursive"; echo $F77_OPT 
export F77_FLAGS="-fPIC -I$MPI_HOME/include" ; echo $F77_FLAGS

# environment variable for the thread library
export THR_LIB=-lpthread ; echo $THR_LIB  

# BEFORE to start edit the "configure" file.
#
# 1) Set dynamic libraries path and path
#    THIS IS A CRITICAL POINT.
#
# 2) Check "extended LENGTH" (line 2428)
#     gfortran: adds "-ffixed-line-length-132"
#     g95     : adds "-ffixed-line-length-none"
#     ifort   : adds "-extend-source"
#
# 3) Check "FREE source format" (line 2564)
#     gfortran: adds "-ffree-form"
#     g95     : adds "--free-form"
#     ifort   :      "-free"

# System's configuration:
#
# A configuration must be done as:
cd $PHOME ; pwd
make clean
./configure 2>&1 | tee configure.log
cat config.log

# Looks in the .adaptorrc file. If its LD_FLAGS is empty then 
# find the gcc library and then insert the gcc library in 
# the LD_FLAGS flag
gcc -v

Reading specs from /usr/lib/gcc-lib/i386-redhat-linux/2.96/specs
gcc version 2.96 20000731 (Red Hat Linux 7.1 2.96-81)

# Looks in the ".adaptorrc" file the variables:

# Libraries for distributed memory message passing:
DM:mpi = -lc -L/usr/local/mpich2/1.0.3/lib -lmpichf90 -lmpichf90 -lmpich
DM:mpi = -lc -L/usr/local/mpich2/g95/1.0.3/lib -lmpichf90 -lmpich -lf95 -lm

# a) g95 compiler: 
FC=mpif90 -fPIC -I/usr/local/mpich2/1.0.3-g95/include
FC_FREE=-ffree-form
FC_EXTEND=-ffixed-line-length-132
FC_EXTEND_LENGTH=132
FC_OPT=O4 -ffree-line-length-huge
FC_MP=
FC_FLAGS=-w
LD_FLAGS=-L/usr/lib/gcc/i386-redhat-linux/3.4.4/specs -lgcc
         
# b) ifort compiler:
FC=mpif90 -fPIC -I/usr/local/mpich2/1.0.3/include
FC_FREE=-free
FC_EXTEND=-extend-source
FC_EXTEND_LENGTH=132
FC_OPT=-Wp,-xP,-O3,-ipo,-vms,-recursive
FC_MP=-auto
FC_FLAGS=-w
LD_FLAGS=

# c) gfortran compiler: 
FC=mpif90 -fPIC -I/usr/local/mpich2/1.0.3/include
FC_FREE=-ffree-form
FC_EXTEND=-ffixed-line-length-none
FC_EXTEND_LENGTH=134
FC_OPT=-ffree-line-length-none
FC_MP=
FC_FLAGS=-w
LD_FLAGS=-L/usr/lib/gcc/i386-redhat-linux/3.4.4/specs -lgcc

# Then, make the System's construction
cd $PHOME ; pwd
make clean
make 2>&1 | tee make.log

# Traductor's construction
cd $PHOME/src ; pwd
make clean
make 2>&1 | tee make_src.log

# Before testing the compiler
cd $PHOME
chmod 777 -R test_examples
chmod 777 -R hpf_examples
chgrp -R users test_examples

# Settings
$PHOME/bin/adaptor -settings
$PHOME/bin/fadapt -settings

# in each user enviroment (e.g. .bashrc)
export PHOME=/usr/local/adaptor/g95/10.1
export PATH=$PATH:$PHOME/bin 
export MANPATH=$MANPATH:$PHOME/man

# ADAPTOR's bin and man in the PATH
export PATH=$PATH:$PHOME/bin ; echo $PATH
export MANPATH=$MANPATH:$PHOME/man ; echo $MANPATH

# as some user
su -l username
echo $LD_LIBRARY_PATH ; which mpdboot ; echo $MPI_ROOT ; echo $PHOME
mpdboot -n 23 -f ~/mpd.hosts
mpdtrace
mpdringtest
export PHOME=/usr/local/adaptor/10.1; echo $PHOME
export PATH=$PATH:$PHOME/bin ; echo $PATH
export MANPATH=$MANPATH:$PHOME/man ; echo $MANPATH

# Sec. 9.2  Testing the Compiler, page 19
# cd /usr/local/adaptor/test_examples/HPF_CB/SIMPLE <- before in "iguide.pdf"
cd $PHOME/hpf_examples ; pwd                        <- now
# 
adaptor -v -hpf_1 -c simple.hpf
adaptor -v -hpf_1 -o simple1.exe simple.o
simple1.exe > simple_test.out
#
adaptor -v -openmp -c simple.hpf
adaptor -v -openmp -o simple2.exe simple.o
simple2.exe >> simple_test.out
#
adaptor -v -hpf -c simple.hpf
adaptor -v -hpf -o simple3.exe simple.o
simple3.exe -nb 1 >> simple_test.out
cat simple_test.out

# Sec. 9.3    Test Programs

# Sec. 9.3.1  Serial HPF Test Programs
# make &> hpf_serial_test.log
# cat     hpf_serial_test.log
# make run &> hpf_serial_test.out
# cat hpf_serial_test.out
#
cd $PHOME/test_examples ; pwd
make clean
make 2>&1 | tee hpf_serial_test.log
make run 2>&1 | tee hpf_serial_test.out

# Sec. 9.3.2  Parallel HPF Test Programs

# In $PHOME/test_examples/HPF_MPI/hpf_halo
# with "ifort" modify "makefile"  (tests crashing)
# ELIMINATE   "halo" 
#             "dynamic"
#
cd $PHOME/test_examples/HPF_MPI ; pwd
make clean
make 2>&1 | tee hpf_parallel_test.log
run 2>&1 | tee hpf_parallel_test.out

# Sec. 9.3.3  Parallel OMP Test Programs
cd $PHOME/test_examples/OMP
make clean
make &> omp_parall_test.log
cat omp_parall_test.log
run &> omp_parall_test.out
cat omp_parall_test.out

# Don't forget finalize the "mpd" session !!!
mpdallexit
exit

#end
