! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! compilation with (adaptor + mpich):
!  adaptor -hpf -dm -free -o dynamic.exe dynamic.f90
!
! running with mpich:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 2 dynamic.exe
!  mpdallexit
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
      program Halo
!
!     simple test program for HALO library
!
      integer ERRORS, NP

      ERRORS = 0
      NP     = number_of_processors ()

      print *, ' '
      print *, 'TEST of HALO Library on ', NP, ' procs'
      print *, '======================================'
      print *, ' '

      call UPDATE (30, 10, ERRORS)
      call UPDATE (10, 30, ERRORS)

      if (ERRORS .eq. 0) then
         print *, 'Halo NP =', NP, ': all tests PASSED'
        else
         print *, 'Halo NP =', NP, ': at least one test FAILED'
      end if

      end program Halo

!************************************************************************
!     subroutine UPDATE     (tests HALO_UPDATE)
!************************************************************************

      subroutine UPDATE (M, N, ERRORS)

      use HPF_HALO_LIBRARY

      integer, intent(in)    :: M, N
      integer, intent(inout) :: ERRORS 

      integer, dimension (N) :: B, B1, P   ! problem values
      real, dimension (N)    :: R

      integer :: I, S_ID
      integer :: SIZE

      integer, pointer, dimension(:) :: A
!hpf$ distribute (BLOCK) :: A

!hpf$ template T(M)   
!hpf$ distribute (BLOCK) :: T

!hpf$ distribute (block) :: B
!hpf$ align(I) with B(I) :: P, R, B1

!     init P by random
      call random_number (R)
      P = R * M + 1

      call HALO_DEFINE (s_id, A, P, UB=M)

      call HALO_MAX_SIZE (s_id, SIZE)

      call MY_ALLOCATE (A, M, SIZE)

      call HALO_LOCAL_INDEXES (s_id, P)

      call HALO_UPDATE (s_id, A)

!hpf$ independent, on home (B(I)), resident
      do i = 1, n
         B(i) = A(P(I))
      end do

      call HALO_GLOBAL_INDEXES (s_id, P)

      call HALO_FREE (s_id)

      B1 = A(P)

      if (any (B .ne. B1)) then
         ERRORS = ERRORS + 1
         print *, 'ERROR, UPDATE, (N,M) = ', N, M
      end if

      end subroutine UPDATE

      subroutine MY_ALLOCATE (A, M, SIZE)

      integer :: M, SIZE

      integer, dimension(:), pointer :: A
!hpf$ shadow (0:*SIZE) :: A
!hpf$ distribute (block) :: A

      allocate (A(M))

      end subroutine MY_ALLOCATE

