! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! compilation with (adaptor + mpich):
!  adaptor -hpf -dm -free -o halo.exe halo.f90
!
! running with mpich:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 2 halo.exe
!  mpdallexit
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!     simple test program for HALO library
program halo
  implicit none
  integer errata, np
  errata = 0
  np = number_of_processors ()
  print *
  print *, 'TEST of HALO Library on ', np, ' procs'
  print *, '======================================'
  print *
  call update (30, 10, errata)  ! 30 10
  call update (10, 30, errata)  ! 10 30
  call reduce (20, 20, errata)  ! 20 20
  call reduce (10, 30, errata)  ! 10 30
  if (errata .eq. 0) then
    print *, 'Halo np =', np, ': all tests PASSED'
  else
    print *, 'Halo np =', np, ': at least one test FAILED'
  end if
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!  tests HALO_update
subroutine update (m, n, errata)
  implicit none
  use HPF_HALO_LIBRARY
  integer, intent(in)    :: m, n
  integer, intent(inout) :: errata 
  integer, dimension (m) :: a    ! indirectly accessed
  integer, dimension (n) :: b, b1, p   ! problem values
  real   , dimension (n) :: r
  integer :: i, s_id
  !hpf$ distribute (block) :: a
  !hpf$ distribute (block) :: b, b1
  !hpf$ align (i) with b (i) :: p, r
  !hpf$ shadow a (0:*17)
  forall (i=1:m) a (i) = i
  ! init p by random
  call random_number (r)
  p = r * m + 1
  call HALO_ALL_DEFINE (s_id, a, p)
  call HALO_update (s_id, a)
  !hpf$ independent, on home (b(i)), resident
  do i = 1, n
    b(i) = a(p(i))
  end do
  call HALO_GLOBAL_INDEXES (s_id, p)
  call HALO_FREE (s_id)
  b1 = a(p)
  if (any (b .ne. b1)) then
    errata = errata + 1
    print *, 'ERROR, update, (n,m) = ', n, m
  end if
  !
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! tests HALO_SUM_reduce)
subroutine reduce (m, n, errata)
  implicit none
  use HPF_HALO_LIBRARY
  integer, intent(in)    :: m, n
  integer, intent(inout) :: errata 
  integer, dimension (m) :: a    ! indirectly accessed
  integer, dimension (n) :: b, b1, p   ! problem values
  real, dimension (n)    :: r
  integer :: i, s_id
  !hpf$ distribute (block) :: a
  !hpf$ distribute (block) :: b
  !hpf$ align (i) with b (i) :: p, r
  !hpf$ shadow a (3:*17)
  integer :: s1, s2
  forall (i=1:n) b(i) = i
  ! init p by random
  call random_number (r)
  p = r * m + 1
  call HALO_ALL_DEFINE (s_id, a, p)
  a = 0
  call HALO_INIT       (s_id, a, 0)
  !hpf$ independent, on home (b(i)), resident
  do i = 1, n
     a(p(i)) = a(p(i)) + b(i)
  end do
  call HALO_REDUCE_SUM     (s_id, a)
  call HALO_GLOBAL_INDEXES (s_id, p)
  call HALO_FREE (s_id)
  s1 = sum (a)
  s2 = n * (n+1) / 2
  if (s1 .ne. s2) then
    errata = errata + 1
    print 10, n, m
    10      format ('ERROR reduce, n = ',i4,', m = ',i4)
  end if
  !
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
