! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Compilation with INTEL:
!  ifort -w -o ptask.exe1 ptask.f90
!
! Compilation with GNU:
!  g95 -w -o ptask.exe2 ptask.f90
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! ADAPTOR: compilation of a HPF input source file and FREE format.
!
! Some options:
!
! 1) HPF compilation for a single node: 
!    adaptor -hpf_1 -free -o ptask.exe3 ptask.f90
!
! 2) HPF compilation for Distributed Memory Machines using MPICH/MPI:
!    adaptor -hpf -dm -free -o ptask.exe4 ptask.f90
!    mpdboot -n 4 -f ~/mpd.hosts
!    mpdtrace
!    mpiexec -machinefile ~/machi.dat -1 -np 3 ptask.exe4
!    mpirun -machinefile ~/machi.dat -nolocal -np 3 ptask.exe4
!    mpdallexit
!
! 3) HPF compilation for Shared Memory Machines (SMM) using PThreads
!    adaptor -hpf -sm -free -o ptask.exe5 ptask.f90
!
! 4) HPF compilation for Cached Architectures
!    adaptor -hpf -cb -free -o ptask.exe6 ptask.f90
!    ptask.exe6 -nb 2
!
! 5) HPF compilation for a cluster of SMP nodes using MPI and PThreads
!    adaptor -hpf -dm -sm -free -o ptask.exe7 ptask.f90
!    mpirun -machinefile ~/machi.dat -np 4 ptask.exe7
!
! 6) OpenMP compilation for shared memory machines using PThreads
!    adaptor -openmp -free -o ptask.exe8 ptask.f90
!    export OMP_NUM_THREADS=2 ; echo $OMP_NUM_THREADS
!    ptask.exe8
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program ptask
  implicit none
  integer, parameter :: nax = 8
  integer, dimension (nax) :: a1, a2
  !hpf$ processors proc (4)
  !hpf$ distribute a1 (block) onto proc (1:2)
  !hpf$ distribute a2 (block) onto proc (3:4)
  integer, dimension (2) :: ier
  integer :: n1, n2
  ! 
  !hpf$ task_region
  !hpf$ on home (a1), resident
   call task1 (n1)
  !hpf$ on home (a2), resident
   call task2 (n2)
  !hpf$ end task_region
  !
  write (*,*) "n1", n1
  write (*,*) "n2", n2
  !
  contains
    subroutine task1 (n)
!    integer, dimension (:), intent (out) :: a1
     integer               , intent (out) :: n
!    !hpf$ inherit :: a1
!    !hpf$ range  a1 (cyclic)
     n = 1
    end subroutine
    pure subroutine task2 (n)
!    integer, dimension (:), intent (out) :: a2
     integer               , intent (out) :: n
!    !hpf$ inherit :: a2
!    !hpf$ range a2 (block)
     n = 2
    end subroutine
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
