! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Compilation with INTEL:
!  ifort -w -o pura.exe1 pura.f90
!
! Compilation with GNU:
!  g95 -w -o pura.exe2 pura.f90
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! ADAPTOR: compilation of a HPF input source file and FREE format.
!
! Some options:
!
! 1) HPF compilation for a single node: 
!    adaptor -hpf_1 -free -o pura.exe3 pura.f90
!
! 2) HPF compilation for Distributed Memory Machines using MPICH/MPI:
!    adaptor -hpf -dm -free -o pura.exe4 pura.f90
!    mpdboot -n 4 -f ~/mpd.hosts
!    mpdtrace
!    mpiexec -machinefile ~/machi.dat -1 -np 3 pura.exe4
!    mpirun -machinefile ~/machi.dat -nolocal -np 3 pura.exe4
!    mpdallexit
!
! 3) HPF compilation for Shared Memory Machines (SMM) using PThreads
!    adaptor -hpf -sm -free -o pura.exe5 pura.f90
!
! 4) HPF compilation for Cached Architectures
!    adaptor -hpf -cb -free -o pura.exe6 pura.f90
!    pura.exe6 -nb 2
!
! 5) HPF compilation for a cluster of SMP nodes using MPI and PThreads
!    adaptor -hpf -dm -sm -free -o pura.exe7 pura.f90
!    mpirun -machinefile ~/machi.dat -np 4 pura.exe7
!
! 6) OpenMP compilation for shared memory machines using PThreads
!    adaptor -openmp -free -o pura.exe8 pura.f90
!    export OMP_NUM_THREADS=2 ; echo $OMP_NUM_THREADS
!    pura.exe8
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
      pure function suma1 (x,y) result (z)
        integer, intent (in) :: x, y
        integer :: z
        z = x + y
      end function      
!
      pure subroutine suma2 (x,y,z)
      integer, intent (in)  :: x, y
      integer, intent (out) :: z
      z = x + y
      end subroutine
!
      program pura
      integer, parameter :: nax = 6
      integer, dimension (nax) :: a, b, c
      !hpf$ distribute (block) :: a, b, c
      integer :: i, n
!
      write (*,*)
      n = nax
! 
      do i = 1, n
        a (i) = i
        b (i) = n - i + 1
        c (i) = 0
      end do
!
!     !hpf$ independent, resident, on home (c(i))
      do i = 1, n
        c (i) = suma1 (a(i),b(i))
      end do
!
      write (*,*) "a", a
      write (*,*) "b", b
      write (*,*) "c", c
!
      end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
