      program GEN_BLOCK_TEST

!     general block distributions

      implicit NONE

      use HPF_LIBRARY   !   import HPF_SUBGRID_INFO

      integer, parameter :: NCOL = 10
      real, parameter    :: EPS  = 0.00001

      integer, dimension (NCOL) :: COLSIZE, COLOFFSET

!hpf$ distribute COLSIZE (block)

      integer :: NP 

      integer, dimension (NUMBER_OF_PROCESSORS()) :: PSIZE, DSIZE
      integer, dimension(number_of_processors(),1) :: LB, UB

!hpf$ distribute (block)   :: DSIZE
!hpf$ distribute (block,*) :: LB, UB

      integer ROW, COL, NVALS
      integer ERRORS 

      real :: S1, S2
      integer :: I, IERR
      integer :: UB0, LB1, UB1

      real, allocatable :: VALS (:)

      integer, allocatable :: INDEX (:)
      real, allocatable    :: VAL1 (:)

!hpf$ distribute (block) :: VAL1
!hpf$ align INDEX(I) with VAL1(I)

!hpf$ distribute vals (block)
!hpf$ dynamic vals
      
      print *, ' '
      print *, ' Test GEN_BLOCK () distribution'
      print *, ' =============================='
      print *, ' '

      COLSIZE = (/ (COL, COL = 1, NCOL) /)   !  column i has i entries

      print *, 'before LOCAL_SUM, COLSIZE = ', COLSIZE

      call LOCAL_SUM (DSIZE, COLSIZE)
      PSIZE = DSIZE

      COLOFFSET (1) = 1
      do COL = 2, NCOL
         COLOFFSET (COL) = COLOFFSET (COL-1) + COLSIZE(COL-1)
      end do

      NVALS = sum (COLSIZE)

      print *, 'sparse matrix with ', NCOL, ' columns'

      print 10, COLSIZE
      print 11, COLOFFSET
      print 12, PSIZE

  10  format (' col sizes   : ',15I4)
  11  format (' col offsets : ',15I4)
  12  format (' proc sizes  : ',15I4)

      ERRORS = 0
      NP     = number_of_processors ()

!hpf$ redistribute vals (gen_block(PSIZE))

      allocate (VALS (NVALS))

!     show the distribution of VALS

      call SHOW_DISTRIBUTION (VALS)

!     get subgrid info

      print *, 'check subgrid info'

      call HPF_SUBGRID_INFO (VALS, IERR, lb=LB, ub=UB)

      do I = 1, number_of_processors ()
         LB1 = LB(I,1)
         UB1 = UB(I,1)
         if (UB1 - LB1 + 1 .ne. PSIZE(I)) ERRORS = ERRORS + 1
         if (I .gt. 1) then
           if (UB0 + 1 .ne. LB1) ERRORS = ERRORS + 1
         end if
         UB0 = UB1
      end do

      if (ERRORS .gt. 0) then
         print *, '--> error for SUBGRID info'
      end if

      print 13, LB(:,1)
      print 14, UB(:,1)

  13  format (' subgrid LB  : ',15I4)
  14  format (' subgrid UB  : ',15I4)

      print *, 'check local sizes of array VALS'

      DSIZE = 0
      call GET_SIZES (VALS, DSIZE)
      if (any (DSIZE .ne. PSIZE)) then
         ERRORS = ERRORS + 1
         print *, '--> size error : ', DSIZE
      end if

!hpf$ independent, on home (COLSIZE(COL)), resident
      do COL = 1, NCOL
         do ROW = 1, COLSIZE(COL)
            VALS (COLOFFSET(COL) + ROW-1) = ROW
         end do
      end do

!     equivalent computation

      S1 = 0.0
      do COL = 1, NCOL
         S1 = S1 + COL * (COL+1) / 2
      end do

      S2 = sum (VALS)

      print *, 'check local computations on array VALS'

      if (abs (S1 - S2) .gt. EPS) then
         ERRORS = ERRORS + 1
         print *, '--> wrong result'
         print *, 'S1 = ', S1
         print *, 'S2 = ', S2
      end if

!     test indirect access to general block distributed array

      allocate (VAL1(NVALS), INDEX(NVALS))

      INDEX = (/ (I, I = NVALS, 1, -1) /)
      VAL1 = VALS(INDEX)

      print *, 'check indirect access on array VALS'

      S2 = sum (VAL1)
      if (abs (S1 - S2) .gt. EPS) then
         ERRORS = ERRORS + 1
         print *, '--> wrong result'
         print *, 'S1 = ', S1
         print *, 'S2 = ', S2
         print *, 'VALS = ', VALS
         print *, 'INDEX = ', INDEX
         print *, 'VAL1 = VALS(INDEX) = ', VAL1
      end if

      deallocate (VAL1, INDEX)

      deallocate (VALS)

      print *, 'check local computations on reallocated VALS'

      allocate (VALS(NVALS))

!hpf$ independent, on home (COLSIZE(COL)), resident
      do COL = 1, NCOL
         do ROW = 1, COLSIZE(COL)
            VALS (COLOFFSET(COL) + ROW-1) = ROW
         end do
      end do

      S2 = sum (VALS)

      if (abs (S1 - S2) .gt. EPS) then
         ERRORS = ERRORS + 1
         print *, '--> wrong result'
         print *, 'S1 = ', S1
         print *, 'S2 = ', S2
      end if

      deallocate (VALS)

      print *, ' '

      if (ERRORS .eq. 0) then
         print *, 'GenBlock, NP =', NP,' PASSED'
        else
         print *, 'GenBlock, NP =', NP,' FAILED'
      end if

      end

      extrinsic (hpf_local) 
     &subroutine GET_SIZES (VALS, DSIZE)
 
      real, dimension (:)    :: VALS
      integer, dimension (:) :: DSIZE

!hpf$ distribute (gen_block()) :: VALS
!hpf$ distribute (block)       :: DSIZE

      DSIZE (lbound(DSIZE,1)) = ubound(VALS,1) - lbound(VALS,1) + 1

      end

      extrinsic (hpf_local) 
     &subroutine SHOW_DISTRIBUTION (X)

      use HPF_LOCAL_LIBRARY

      real, dimension (:) :: X(:)
!hpf$ distribute (gen_block()) :: X
      character*10 AXIS_TYPE (7)

      integer, dimension(7) :: AXIS_INFO, SHAPE

      integer I, RANK, PID

      PID = my_processor ()

      axis_type = '            '
      axis_info = 0

      call GLOBAL_DISTRIBUTION (x, axis_type, axis_info,
     $                          rank, shape)

      print  *, 'GLOBAL distribution of array on PID = ', PID
      print 10, PID, AXIS_TYPE(1)
      print 20, PID, AXIS_INFO(1)
      print 40, PID, RANK
      print 30, PID, (SHAPE (i), i=1,rank)

  10  format (' ',I3,' AXIS_TYPE  : ',A12)
  20  format (' ',I3,' AXIS_INFO  : ',I5)
  30  format (' ',I3,' PROC_SHAPE : ',3I5)
  40  format (' ',I3,' PROC_RANK  : ',I5)
      end

      extrinsic (HPF_LOCAL) 
     &  subroutine LOCAL_SUM (PROCSIZE, COLSIZE)
 
!hpf$ distribute (BLOCK) :: PROCSIZE, COLSIZE

      integer, dimension (:) :: PROCSIZE, COLSIZE

      print *, 'LOCAL_SUM:, COLSIZE = ', COLSIZE

      PROCSIZE (lbound(PROCSIZE,1)) = sum (COLSIZE(:))

      end subroutine LOCAL_SUM

