!     numerical integration to calculate pi

      program PI_TEST

      implicit NONE

      integer, parameter :: N = 100000
      real, parameter    :: EPS = 0.0001

      integer :: NP

      real, allocatable :: V(:)
      real              :: S1, S2, S3

!hpf$ distribute v(block)
!hpf$ dynamic v

      integer TICKSTART, TICKSTOP, TICKRATE
      real TIME, MFLOPS
      integer, dimension(128) :: SIZES
      integer :: I

      NP = number_of_processors ()

      ! realizes old implementation of adaptor (version 4.0 and older)

      do I = 1, NP
         SIZES(i) = I * N / NP - (I-1) * N / NP
      end do

      call system_clock (tickstart, tickrate)
      allocate (V(N))
      call CALC_PI (V, N, S1)
      deallocate (v)
      call system_clock (tickstop)

      print  *, 'V block distributed'
      TIME   = float (TICKSTOP - TICKSTART) / float(TICKRATE)
      MFLOPS = 9 * N / (1000000.0 * TIME)
      print  5, S1
      print 10, TIME
      print 20, MFLOPS, NP
      print 30, MFLOPS/NP

      print *, 'redistribute gen_block (', SIZES(1:NP), ')'
!hpf$ redistribute V(gen_block(SIZES))

      call system_clock (TICKSTART, TICKRATE)
      allocate (V(N))
      call CALC_PI (V, N, S2)
      deallocate (V)
      call system_clock (TICKSTOP)

      TIME   = float (TICKSTOP - TICKSTART) / float(TICKRATE)
      MFLOPS = 9 * N / (1000000.0 * TIME)
      print  *, 'V gen_block distributed'
      print  5, S2
      print 10, TIME
      print 20, MFLOPS, NP
      print 30, MFLOPS/NP

!hpf$ redistribute V(*)

      call system_clock (TICKSTART, TICKRATE)
      allocate (V(N))
      call CALC_PI (V, N, S3)
      deallocate (V)
      call system_clock (TICKSTOP)

      TIME   = float (TICKSTOP - TICKSTART) / float(TICKRATE)
      MFLOPS = 9 * N / (1000000.0 * TIME)
      print  *, 'V replicated'
      print  5, S3
      print 10, TIME
      print 30, MFLOPS

   5  format (' pi ist approximated with ', F9.6)
  10  format (' time   = ', F7.3, ' seconds')
  20  format (' mflops = ', F7.3, ' on ',I3,' processors')
  30  format (' mflops = ', F7.3, ' for one processors')

      print *, ' '
      if ((abs (S1-S2) .lt. EPS) .and. (abs(S1-S3) .lt. EPS)) then
         print *, 'PI range test, NP =', NP, ' PASSED'
        else
         print *, 'PI range test, NP =', NP, ' FAILED'
         print *, 'Result 1 = ', S1
         print *, 'Result 2 = ', S2
         print *, 'Result 3 = ', S3
      end if

      end

      subroutine CALC_PI (V, N, S)

      integer, intent(in) :: N
      real, intent(out)   :: S
      real, dimension (N) :: V
!hpf$ inherit             :: V
!hpf$ range (block())     :: V

      integer :: I
      real    :: W

      W = 1.0 / float(N)
      forall (I=1:N) V(I) = I
      V = (V - 0.5 )* W
      V = 4.0 / (1.0 + V * V)
      S = sum (V)
      S = S * W

      end subroutine CALC_PI

