! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Compilation with INTEL:
!  ifort -w -c set1.f90
!  ifort -w -o set1.exe set1.o
!
! Compilation with GNU:
!  g95 -w -o set1.exe set1.f90
!
! Compilation with (ADAPTOR + MPICH):
!  adaptor -hpf -dm -free -c set1.f90
!  adaptor -hpf -dm -free -o set1.exe set1.f90
!
! Running with MPICH:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 2 set1.exe
!  mpiexec -machinefile ~/machi.dat -1 -np 4 set1.exe
!  mpdallexit
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program set_test
  implicit none
  integer, parameter :: n = 64
  real, parameter :: eps = 0.0001
  real, dimension (n,n) :: t
  !hpf$ distribute (block, block) :: t
  real    :: x, s, s1
  integer :: i, j, m = 10, errors = 0
  !
  type grid
    integer :: i, j
    real, dimension (:,:), pointer :: u
    !hpf$ align (i,j) with t (5*j,5*i-1) :: u
  end type
  !
  type (grid) :: level1
  real, dimension (:,:), allocatable :: f
  !
  print *
  print *, " Test set for distribution in derived types"
  print *, " =========================================="
  print *
  !
  allocate (level1 % U (m,m))
  allocate ( f (m,m) )
  !
  call random_number (level1 % u)
  f = level1 % u
  !
  s1 = sum (f)
  print *, "sum of F (serial)   ", s1
  !
  s = sum (level1 % u)
  print *, "sum of U (parallel) ", s
  !
  if (abs (s - s1) > eps) errors = errors + 1
  !
  s = 0.0
  do  j = 1, m
    do  i = 1, m
      x = level1 % u (i,j) 
      s = s + x
    end do
  end do
  !
  print *, "sum of U (serial)   ", s
  !
  if (abs (s - s1) > eps) errors = errors + 1
  !
  deallocate (level1 % u)
  deallocate (f)
  !
  print *
  if (errors .eq. 0) then
    print *, "distribution in DT: all tests PASSED"
  else
    print *, "distribution in DT: at least one test FAILED"
  end if
  print *
  !
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
