! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Compilation with INTEL:
!  ifort -w -c type_fn.f90
!  ifort -w -o type_fn.exe type_fn.o
!
! Compilation with GNU:
!  g95 -w -o type_fn.exe type_fn.f90
!
! Compilation with (ADAPTOR + MPICH):
!  adaptor -hpf -dm -free -c type_fn.f90
!  adaptor -hpf -dm -free -o type_fn.exe type_fn.f90 
!
! Running with MPICH:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 2 type_fn.exe
!  mpiexec -machinefile ~/machi.dat -1 -np 4 type_fn.exe
!  mpdallexit
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
module definitions
  implicit none
  type bloque
    real x, y
  end type bloque
  contains
    function init_bloque (x, y)
      type (bloque) :: init_bloque
      real, intent (in) :: x, y
      init_bloque % x = x
      init_bloque % y = y
    end function
    function all_init_bloque (n, x, y)
      integer, intent (in) :: n
      real   , intent (in) :: x, y
      type (bloque), dimension (n) :: all_init_bloque
      !hpf$ distribute (block) :: all_init_bloque
      all_init_bloque % x = x
      all_init_bloque % y = y
    end function
end module DEFINITIONS
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program ppal
  use definitions
  implicit none
  !
  integer, parameter :: n = 10
  real   , parameter :: v1 = 2.0, v2 = 0.5
  !
  type (bloque), dimension (n) :: f
  real        , dimension (n) :: x
  !hpf$ distribute (block) :: x, f
  type (bloque) b
  integer :: errors = 0
  real    :: res
  !
  print *
  print *, " Test derived type functions"
  print *, " ==========================="
  print *
  !
  b = init_bloque (v1, v2)
  !
  f = b
  !
  x = f % x
  call real_check (sum (x), n*v1, errors)

  x = f % Y
  call real_check (sum (x), n*v2, errors)
  !
  f = all_init_bloque (n, v2, v1)
  !
  x = f % x
  call real_check (sum (x), n*v2, errors)
  !
  x = f % y
  call real_check (sum (x), n*v1, errors)
  !
  print *
  if (errors .eq. 0) then
    print *, "map array of type: all tests PASSED"
  else
    print *, "map array of type: at least one test FAILED"
  end if
  print *
  !
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine real_check (r1, r2, errors)
  real, parameter       :: eps = 0.000001
  real   , intent (in)  :: r1, r2
  integer, intent (out) :: errors
  !
  if ( abs (r1 - r2) > eps) then
     errors = errors + 1
     print *, '--> ERROR, result: ', r1, ', shoud be ', r2
  end if
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
