! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! compilation with (adaptor + mpich):
!  adaptor -hpf -dm -free -o bc_redist.exe bc_redist.f90
!
! running with mpich:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 2 bc_redist.exe
!  mpdallexit
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program redistribution
  implicit none
  print *, "cyclic(n) redistribution (integer)"
  print *, " (C2,C3) -> (C1,*) -> (B,C4) -> (C2,C3)"
  call redist2 ()
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine redist2 ()
  implicit none
  integer :: n1, n2, i, j
  parameter (n1=18, n2=13)
  !hpf$ template   ta (n1,n2)
  !hpf$ distribute ta (cyclic(2),cyclic(3))
  !hpf$ template   tb (n1,n2)
  !hpf$ distribute tb (cyclic,*)
  !hpf$ template   tc (n1,n2)
  !hpf$ distribute tc (block,cyclic(4))
  integer, dimension (n1,n2) :: a1, a2
  integer, dimension (n1,n2) ::  b,  c
  integer, dimension (n2)    :: r1, r2
  !hpf$ align with ta :: a1, a2
  !hpf$ align with tb :: b
  !hpf$ align with tc :: c
  integer :: s, d
  !
  forall (j=1:n2, i=1:n1)
    a1 (i,j) = i + j
  end forall
  !
  b = a1
  b = b + 1    ! do some work in topology of B
  c = b
  c = c + 2    ! do some work in topology of C
  a2 = c
  a2 = a2 - a1
  s = sum (a2)
  print *, "sum is ", s, " should be ", 3*n1*n2
  !
  print *, "compare all rows of A and B (checks replication)"
  do i = 1, n1
     r1 = a1(i,n2)
     r2 = b (i,n2)
     d  = sum (r2 - r1)
     if (d .ne. n2) then
        print *, "ROWDIIFF error in row ", i
        print *, "R1   = ", sum (r1)
        print *, "R2   = ", sum (r2)
        print *, "diff = ", d
        print *, "should be ", n2
     end if
  end do
  !
  print *, "compare all rows of B and C"
  do i = 1, n1
     r1 = b (i,n2)
     r2 = c (i,n2)
     d  = sum (r2 - r1)
     if (d .ne. 2*n2) then
        print *, "ROWDIIFF error in row ", i
        print *, "R1   = ", sum (r1)
        print *, "R2   = ", sum (r2)
        print *, "diff = ", d
        print *, "should be ", 2*n2
     end if
  end do
  !
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
