! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! compilation with (adaptor + mpich):
!  adaptor -hpf -dm -free -o copy.exe copy.f90
!
! running with mpich:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 2 copy.exe
!  mpdallexit
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program test_copy
  implicit none
  integer, parameter :: n = 10
  integer, parameter :: np = 16
  real, dimension (n,n) :: a, b
  real, dimension (2) :: y
  real :: x
  !hpf$ processors p (2,number_of_processors()/2)
  !hpf$ distribute a (block,block) onto p
  !hpf$ distribute b (block,block) 
  integer, dimension (np) :: res
  !hpf$ distribute res (block)
  integer :: i, j
  integer :: errata, np1
  !
  np1 = number_of_processors ()
  errata = 0
  print *
  print *, "Test copy reduction on ",np1," procs"
  print *, "==================================="
  print *
  a = 0.0
  a (9,1) = 1.0
  a (4,8) = 2.0
  b = 0.0
  b (9,1) = 1.0
  b (4,8) = 2.0
  x = 0.0
  !full reduction, generates dalib_top_merge 
  !hpf$ independent, reduction (x)
  do  j = 1, n
    !hpf$ independent
    do  i = 1, n
      if (b (i,j) .gt. 0.0)  x = b (i,j)
    end do
  end do
  !
  !prove that x has correct value on host
  if ((x .ne. 1.0) .and. (x .ne. 2.0)) then
     print *, "copy reduction for scalar fails (wrong result)"
     print *, "x = ", x, " should be 1.0 or 2.0"
     errata = errata + 1
  end if
  !
  !prove that x is consistent among all procs
  res = x
  if (np * x .ne. sum (res)) then
     errata = errata + 1
     print *, "copy reduction for scalar fails (inconsistent)"
  end if      
  x = 0.0
  !generates dalib_array_merge (2) + broadcast
  !hpf$ independent, reduction (x)
  do j = 1, n
    !hpf$ independent
     do i = 1, n
       if (a(i,j) .gt. 0.0) x = a(i,j)
     end do
  end do
  !
  !prove that x has correct value on host
  if ((x .ne. 1.0) .and. (x .ne. 2.0)) then
     print *, "copy reduction for scalar fails (wrong result)"
     print *, "x = ", x, " should be 1.0 or 2.0"
     errata = errata + 1
  end if
  !
  !prove that x is consistent among all procs
  res = x
  if (np * x .ne. sum (res)) then
     errata = errata + 1
     print *, "copy reduction for scalar fails (inconsistent)"
  end if      
  y = 0.0
  ! full reduction, generates dalib_top_section_merge 
  !hpf$ independent, reduction (y)
  do j = 1, n
    !hpf$ independent
     do i = 1, n
        if (b(i,j) .gt. 1.0) then
           y(1) = b (i,j)
         else if (a (i,j) .gt. 0.0) then
           y(2) = b (i,j)
        end if
     end do
  end do
  !
  !prove that y has correct value on host
  if ((y(1) .ne. 2.0) .or. (y(2) .ne. 1.0)) then
     print *, "copy reduction for array fails (wrong result)"
     print *, "y = ", y, " should be 2.0, 1.0"
     errata = errata + 1
  end if
  !
  !prove that y is consistent among all procs
  res = y(1) + y(2)
  if (np * (y(1)+y(2)) .ne. sum (res)) then
     errata = errata + 1
     print *, "copy reduction for array fails (inconsistent)"
  end if      
  y = 0.0
  !generates dalib_array_section_merge (2) + broadcast
  !hpf$ independent, reduction (y)
  do j = 1, n
    !hpf$ independent
    do i = 1, n
      if (a(i,j) .gt. 1.0) then
        y (1) = a (i,j)
      else if (a (i,j) .gt. 0.0) then
        y (2) = a (i,j)
      end if
    end do
  end do
  !prove that y has correct value on host
  if ((y(1) .ne. 2.0) .or. (y(2) .ne. 1.0)) then
     print *, "copy reduction for array fails (wrong result)"
     print *, "y = ", y, " should be 2.0, 1.0"
     errata = errata + 1
  end if
  ! prove that y is consistent among all procs
  res = y(1) + y(2)
  if (np * (y(1) + y(2)) .ne. sum (res) ) then
     errata = errata + 1
     print *, "copy reduction for array fails (inconsistent)"
  end if      
  if (errata == 0) then
     print *, "copy reduction test: PASSED"
   else
     print *, "copy reduction test: FAILED"
  end if
end program TEST_COpy
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
