! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! compilation with (adaptor + mpich):
!  adaptor -hpf -dm -free -o group.exe group.f90
!
! running with mpich:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 6 group.exe
!  mpdallexit
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program group_test
  implicit none
  !verifies the implementation of irregular groups in DALIB
  !hpf$ processors p (6)
  integer, dimension (6) :: ind  = [1, 4, 2, 3, 6, 5]
  integer, dimension (3) :: ind1 = [4, 6, 2]
  integer, dimension (3) :: ind2 = [1, 5, 3]
  integer :: errata, np
  print *
  print *, "Test PROCESSOR subsets"
  print *, "======================"
  print *
  errata = 0
  np = number_of_processors ()
  call redist_testa (errata)
  !hpf$ on (p(ind))
  call comuni_testa (errata)
  !hpf$ independent task_region
  !hpf$   on (p(ind1)), resident
            call tarea (errata)
  !hpf$   on (p(ind2)), resident
            call tarea (errata)
  !hpf$ end task_region
   print *
  if (errata .eq. 0) then
    print *, "Groups np =", np, ": all tests PASSED"
  else
    print *, "Groups np =", np, ": at least one test FAILED"
  end if
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine redist_testa (errata)
  implicit none
  integer, intent (inout) :: errata
  integer, parameter :: n = 20
  integer, dimension (4) :: ind1 = [2, 3, 1, 4]
  integer, dimension (3) :: ind2 = [5, 6, 1]
  integer, dimension (n) :: a
  integer :: i
  !hpf$ processors p(6)
  !hpf$ distribute (block) onto p :: a
  !hpf$ dynamic :: a
  a = [ (i, i=1,n) ]
  !hpf$ redistribute a(block) onto p (1:4)
  a = a + 1
  !hpf$ redistribute a(block) onto p
  a = a - 2
  !hpf$ redistribute a(block) onto p(ind1)
  a = a + 3
  !hpf$ redistribute a(block) onto p
  a = a - 4
  !hpf$ redistribute a(block) onto p(ind2)
  a = a + 5
  !hpf$ redistribute a(block) onto p
  a = a - 3
  if (any (a .ne. [ (i, i=1,n) ])) then
     errata = errata + 1
     print *, "REDISTRIBUTION error"
     print *, a
  end if
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine comuni_testa (errata)
  implicit none
  integer, intent (inout) :: errata
  integer, parameter :: n = 100
  integer, dimension (n) :: x1, x2, x3
  integer, dimension (n) :: ind
  integer :: i
  !hpf$ distribute (block)     :: x1, x2, ind   ! on irregular topology
  !hpf$ distribute (cyclic(3)) :: x3
  x1 = [ (i, i=1,n) ] 
  do i = 1, n
     x2 = cshift (x1, 1, dim=1)
     x1 = x2
  end do
  if (any (x1 .ne. [ (i, i=1,n) ])) then
     errata = errata + 1
     print *, "CYCLIC test failed"
     print *, x1
  end if
  x1  = [ (i, i=1,n)    ] 
  x3 = x1
  x3 = x3 + 1
  x1 = x3
  x1 = x1 - 1
  if (any (x1 .ne. [ (i, i=1,n) ])) then
     errata = errata + 1
     print *, "ASSIGN test failed"
     print *, x1
  end if
  x1  = [ (i, i=1,n)    ] 
  ind = [ (i, i=n,1,-1) ] 
  x2 = x1(ind)
  x1 = x2(ind)
  if (any (x1 .ne. [(i, i=1,n)])) then
     errata = errata + 1
     print *, "IRREG test failed"
     print *, x1
  end if
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine tarea (errata)
  use HPF_TASK_LIBRARY
  implicit none
  integer, intent(inout) :: errata
  integer, parameter :: n = 100
  integer :: rango
  integer, dimension(n) :: a
  !hpf$ distribute (block) :: a
  !
  call HPF_TASK_INIT ()
  call hpf_task_rank (rank=rango)
  if (rango == 1) then
    a = 1
    call hpf_send (data=a, dest=2)
  else
    a = 0
    call hpf_recv (data=a, source=1)
    if (sum (a) .ne. n) then
       errata = errata + 1
       print *, "tarea test failed"
    end if
  end if
  call HPF_TASK_EXIT ()
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
