! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! compilation with (adaptor + mpich):
!  adaptor -hpf -dm -free -o matmul.exe matmul.f90
!
! running with mpich:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 6 matmul.exe
!  mpdallexit
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!     Test Program for F90 Intrinsic MATMUL
program matmul_test
  implicit none
  real, parameter :: eps = 0.001
  integer, parameter :: n = 100
  real   , dimension (n,n) :: a, b, c, c1, c2
  !hpf$ distribute (block,block) :: a, b, c, c1, c2
  integer :: h1, h2, h3
  integer :: error1, error2
  integer :: errata
  integer i
  !
  print *
  print *, "Test of Parallel MATMUL"
  print *, "======================="
  print *
  errata = 0
  print *, "generate random arrays a, b "
  !generate random arrays a(n,n) and b(n,n)
  call random_number (a)
  call random_number (b)
  a = 1.0
  b = 2.0
  !computing matmul in one step
  print *, "full matrix multiplication"
  call system_clock (h1, h3)
  c = matmul (a, b)
  call system_clock (h2)
  h1 = h2 - h1
  !computing matmuls one column after the other
  print *, "column matrix multiplication"
  call system_clock (h1)
  do i = 1, n
     c1(:,i)  = matmul (a, b(:,i))
  end do
  call system_clock (h2)
  h2 = h2 - h1
  !computing matmuls one row after the other
  print *, "row matrix multiplication"
  c2 = 0.0   ! better debugging
  call system_clock (h1)
  do i = 1, n
    c2(i,:)  = matmul (a(i,:), b)
  end do
  call system_clock (h2)
  h3 = h2 - h1
  !verification that all matrices are the same
  print *, "now check the results"
  error1 = count (abs (c - c1) > eps)
  error2 = count (abs (c - c2) > eps)
  if (error1 > 0) then
     print *, "Error in column matrix multiplication ", error1
     errata = errata + 1
  end if
  if (error2 > 0) then
     print *, "Error in row matrix multiplication ", error2
     errata = errata + 1
  end if
 !print timings
 !print *, "time: matmul full ", float(t1)/float(h3), " sec"
 !print *, "time: matmul col  ", float(t2)/float(h3), " sec"
 !print *, "time: matmul row  ", float(t3)/float(h3), " sec"
  if (errata .eq. 0) then
     print *, "MATMUL (Parallel) : all tests PASSED"
   else
     print *, "MATMUL (Parallel) : at least one test FAILED"
  end if
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
