! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! compilation with (adaptor + mpich):
!  adaptor -hpf -dm -free -o pipelining.exe pipelining.f90
!
! running with mpich:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 6 pipelining.exe
!  mpdallexit
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program pipelining_2d
  implicit none
  integer, parameter :: n = 1000
  integer, parameter :: nt = 3       ! number of iterations
  integer, parameter :: siza = 50    ! block size
  real,    parameter :: eps = 0.0001 ! due to compiler optimizations
  real, dimension (n,n) :: a, a1, a2
  integer :: i, j, k
  integer :: deps (2)
  !hpf$ distribute (*,block) :: a, a2
  !hpf$ shadow a  (1:0,1:0)
  !hpf$ shadow a2 (1:0,1:0)
  integer h1, h2, h3
  real     :: time, time1, time2
  integer  :: dummy
  external :: bloque
  integer  :: errata=0, np
  integer  :: diffs
  !
  np     = number_of_processors ()
  print *, " "
  print *, "test pipelining on ",np," procs"
  print *, "================================"
  print *, " "
  forall (j=1:n, i=1:n)
    a (i,j) = 0.25 * i + 0.75 * j / float (n) 
  end forall
  a2 = a
  a1 = a 
  !
  !serial computation on array a1 (replicated)
  print *, "serial computation started"
  call dalib_barrier ()
  call system_clock (h1, h3)
  do k = 1, nt
    do j = 2, n
      do i = 2, n
        a1 (i,j) = (a1(i,j) + a1(i-1,j) + a1(i,j-1) ) * 0.3
      end do
    end do
  end do
  call dalib_barrier ()
  call system_clock (h2)
  print *, "serial computation finished"
  time1 = float (h2 - h1) / float (h3)
  time1 = time1 / float (nt)
  print *, "serial time = ", time1
  print *, "result a1  = ", sum (a1)
  !
  !pipelining with full blocks (no overlap)
  print *, "standard computation started" 
  deps = [1,1]
  call dalib_barrier ()
  call system_clock (h1, h3)
  do  k = 1, nt
    call dalib_barrier
    call dalib_loccs_driver (bloque, n, 1, 0, a2(2:n,2:n), deps)
  end do
  call dalib_barrier
  call system_clock (h2) 
  print *, "standard computation finished" 
  time2 = float(h2 - h1) / float(h3)
  time2 = time2 / float(nt)
  print *, "time standard = ", time2
  print *, "result a2  = ", sum (a2)
  diffs = count (abs (a1 - a2) > eps)
  if (diffs > 0) then
    errata = errata + 1
    print *, "error full pipeline, wrong results : ", diffs
  end if
  !
  print *, "macro-pipeline computation started"
  deps = [1,1] 
  call dalib_barrier ()
  call system_clock (h1, h3)
  !
  do k = 1, nt
    call dalib_barrier
    call dalib_loccs_driver (bloque, siza, 1, 0, a(2:n,2:n), deps)
  end do
  call dalib_barrier 
  !
  call system_clock (h2)
  print *, "macro-pipeline computation finished" 
  time = float(h2 - h1) / float(h3)
  time = time / float(nt)
  !
  print *, "result a = ", sum (a)
  print *, "time pipeline = ", time
  diffs = count (abs (a1 - a2) > eps)
  if (diffs > 0) then
      errata = errata + 1
      print *, "error block pipeline, wrong results : ", diffs
  end if
  !
  print *, "macro-pipeline computation started"
  print *, "speedup seq/pip = ", time1 / time 
  print *, "speedup standard/seq = ", time2 / time1
  print *, "speedup standard/pip = ", time2 / time
  !
  if (errata .eq. 0) then
     print *, "pipelining np =", np, ": all tests passed"
    else
     print *, "pipelining np =", np, ": at least one test failed"
  end if
  end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
extrinsic (hpf_local) subroutine bloque (a)
  implicit none
  real, dimension (:,:), intent (inout) :: a
  integer :: i, j
  !hpf$ distribute a (*,*)
  !print 10, lbound (a,1), ubound (a,1), lbound (a,2), ubound (a,2)
  do j = lbound (a,2), ubound (a,2)
     do i = lbound (a,1), ubound (a,1)
        a (i,j) = ( a(i,j) + a(i-1,j) + a(i,j-1) ) * 0.3 
     end do
  end do
  !10  format ("bloque job of ",i4,":",i4," , ",i4,":",i4)
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
