! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! compilation with (adaptor + mpich):
!  adaptor -hpf -dm -free -o scatter.exe scatter.f90
!
! running with mpich:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 6 scatter.exe
!  mpdallexit
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program test_scatter
  use HPF_LIBRARY
  implicit none
  logical, parameter :: T = .true.
  logical, parameter :: F = .false.
  logical, dimension (4) :: maska
  integer, dimension (4) :: arreg
  logical, dimension (3) :: lbase, lres
  integer, dimension (3) :: ibase, ires
  integer, dimension (4) :: indx1
  !hpf$ distribute (block) :: maska, arreg, lbase, lres
  !hpf$ distribute (block) :: ibase, ires, indx1
  !adp$ trace indx1
  integer :: errata = 0
  !
  print *
  print *, " Test SCATTER (HPF) (dalib)"
  print *, " =========================="
  print *
  !
  maska = [ T, T, T, F ]
  lbase = [ T, T, T ]
  indx1 = [ 1, 1, 2, 2 ]
  !
  lres = all_scatter (maska, base=lbase,indx1=indx1)
  print 53, "ALL_SCATTER", maska, lbase, indx1, lres
  call check_log_results (lres, [ T, F, T] , 3, errata)
  !
  maska = [ T, F, F, F ]
  lbase = [ F, F, T ]
  !
  lres = any_scatter (maska, base=lbase,indx1=indx1)
  print 53, "ANY_SCATTER", maska, lbase, indx1, lres
  call check_log_results (lres, [ T, F, T] , 3, errata)
  !
  arreg = [ 1, 2, 3, 4 ]
  ibase = [ 7, 8, 9 ]
  !
  ires = copy_scatter (arreg, base=ibase,indx1=indx1)
  print 51, "copy_SCATTER", arreg, ibase, indx1, ires
  call check_copy_results (ires, [ 1, 3, 9], [ 2, 4, 9 ],3, errata)
  !
  maska = [ T, T, T, F ]
  ibase = [ 1, -1, 0 ]
  !
  ires = count_scatter (maska, base=ibase,indx1=indx1)
  print 52, "COUNT_SCATTER", maska, ibase, indx1, ires
  call check_int_results (ires, [ 3, 0, 0] , 3, errata)
  !
  arreg = [ 1, 2, 3, 6 ]
  ibase = [ 1, 3, 7 ]
  !
  !same data for IALL_SCATTER and IANY_SCATTER
  !
  ires = iall_scatter (arreg, base=ibase,indx1=indx1)
  print 51, "IALL_SCATTER", arreg, ibase, indx1, ires
  call check_int_results (ires, [ 0, 2, 7] , 3, errata)
  !
  ires = iany_scatter (arreg, base=ibase,indx1=indx1)
  print 51, "IANY_SCATTER", arreg, ibase, indx1, ires
  call check_int_results (ires, [ 3, 7, 7] , 3, errata)
  !
  ires = iparity_scatter (arreg, base=ibase,indx1=indx1)
  print 51, "IPARITY_SCATTER", arreg, ibase, indx1, ires
  call check_int_results (ires, [ 2, 6, 7] , 3, errata)
  !
  arreg = [ 1, 2, 3, 1 ]
  ibase = [ 4, -5, 7 ]
  !
  ires = maxval_scatter (arreg, base=ibase,indx1=indx1)
  print 51, "MAXVAL_SCATTER", arreg, ibase, indx1, ires
  call check_int_results (ires, [ 4, 3, 7] , 3, errata)
  !
  arreg = [ 1, -2, -3, 6 ]
  ibase = [ 4, 3, 7 ]
  !
  ires = minval_scatter (arreg, base=ibase,indx1=indx1)
  print 51, "MINVAL_SCATTER", arreg, ibase, indx1, ires
  call check_int_results (ires, [ -2, -3, 7] , 3, errata)
  !
  maska = [ T, T, T, T ]
  lbase = [ T, F, F ]
  indx1 = [ 1, 1, 1, 2 ]
  !
  lres = parity_scatter (maska, base=lbase,indx1=indx1)
  print 53, "PARITY_SCATTER", maska, lbase, indx1, lres
  call check_log_results (lres, [ F, T, F] , 3, errata)
  !
  arreg = [ 1, 2, 3, 1 ]
  ibase = [ 4, -5, 7 ]
  indx1 = [ 1, 1, 2, 2 ]
  !
  ires = product_scatter (arreg, base=ibase,indx1=indx1)
  print 51, "PRODUCT_SCATTER", arreg, ibase, indx1, ires
  call check_int_results (ires, [ 8, -15, 7] , 3, errata)
  !
  ires = sum_scatter (arreg, base=ibase,indx1=indx1)
  print 51, "SUM_SCATTER", arreg, ibase, indx1, ires
  call check_int_results (ires, [ 7, -1, 7] , 3, errata)
  !
  print *
  if (errata .eq. 0) then
     print *, "Scatter (dalib): all tests PASSED"
  else
     print *, "Scatter (dalib): at least one test FAILED"
  end if
  51   format (A15," (",4I3,", base = ",3I3,", index =",4I2,") =",3I4)
  52   format (A15," (",4L3,", base = ",3I3,", index =",4I2,") =",3I4)
  53   format (A15," (",4L3,", base = ",3L3,", index =",4I2,") =",3L4)
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine check_int_results (v1, v2, n, errata)
  implicit none
  integer, intent (in)    :: n
  integer, intent (inout) :: errata
  integer, dimension (n), intent(in) :: v1, v2
  !hpf$ distribute (block) :: v1
  if (any (v1 .ne. v2)) then
     errata = errata + 1
     print *, "--> ERROR, result: ", v1, ", shoud be ", v2
  end if
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine check_log_results (v1, v2, n, errata)
  implicit none
  integer, intent (in)    :: n
  integer, intent (inout) :: errata
  logical, dimension (n), intent(in) :: v1, v2
  !hpf$ distribute (block) :: v1
  if (any (v1 .neqv. v2)) then
     errata = errata + 1
     print *, "--> ERROR, result: ", v1, ", shoud be ", v2
  end if
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine check_copy_results (v1, v2a, v2b, n, errata)
  implicit none
  integer, intent (in)    :: n
  integer, intent (inout) :: errata
  integer, dimension (n), intent(in) :: v1, v2a, v2b
  !hpf$ distribute (block) :: v1
  if (any ( (v1 .ne. v2a) .and. (v1 .ne. v2b)) ) then
     errata = errata + 1
     print *, "--> ERROR, result: ",v1, ", shoud be ",v2a, " or ",v2b
  end if 
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
