! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! compilation with (adaptor + mpich):
!  adaptor -hpf -dm -free -o spread.exe spread.f90
!
! running with mpich:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 6 spread.exe
!  mpdallexit
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program spread_test
  implicit none
  integer, parameter :: siza = 13, ncopias = 11
  integer, dimension (siza)         :: a
  integer, dimension (ncopias,siza) :: b1, b2
  integer, dimension (siza,ncopias) :: c1, c2
  !hpf$ distribute (block) :: a
  !hpf$ distribute (block, block) :: b1, b2, c1, c2
  integer :: i
  integer :: errata = 0
  !
  print *
  print *, " Test intrinsic SPREAD (DALIB)"
  print *, " ============================="
  print *
  !
  a = [ (i, i = 1, siza ) ]
  !
  b1 = spread (a, dim=1, ncopies = ncopias)
  do  i = 1, ncopias
    b2 (i,:) = a
  end do 
  print 10, siza, 1, ncopias
  call print_results (b1, ncopias, siza)
  call check_results (b1, b2, ncopias, siza, errata)
  !
  c1 = spread (a, dim=2, ncopies = ncopias)
  do  i = 1, ncopias
     c2(:,i) = a
  end do 
  print 10, siza, 2, ncopias
  !
  call print_results (c1, siza, ncopias)
  call check_results (c1, c2, siza, ncopias, errata)
  !
  print *
  if (errata .eq. 0) then
     print *, "all tests PASSED"
  else
     print *, "at least one test FAILED"
  end if
  10 format (" SPREAD ([1,2,..., ",i2,"], dim=", i1, ", ncopies=",i2, ")")
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine print_results (v1, n1, n2)
  implicit none
  integer, intent (in) :: n1, n2
  integer, dimension (n1, n2), intent(in) :: v1
  integer :: i
  do  i = 1, n1
     print 10, v1 (i,:)
  end do
  10  format (20i3)
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine check_results (v1, v2, n1, n2, errata)
  implicit none
  integer, intent (in)    :: n1, n2
  integer, intent (inout) :: errata
  integer, dimension (n1, n2), intent(in) :: v1, v2
  !hpf$ distribute (block,block) :: v1, v2
  if (any (v1 .ne. v2)) then
     errata = errata + 1
     print *, "--> ERROR, result: ", v1, ", shoud be ", v2
  end if
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
