! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! compilation with (adaptor + mpich):
!  adaptor -hpf -dm -free -o red_halo.exe red_halo.f90
!
! running with mpich:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 2 red_halo.exe
!  mpdallexit
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program red_halo_test
  implicit none
  use HPF_LIBRARY
  use HPF_HALO_LIBRARY
  logical, parameter :: T = .true.
  logical, parameter :: F = .false.
  logical, dimension (4) :: maska
  integer, dimension (4) :: arreg
  logical, dimension (3) :: lbase, lres
  !hpf$ shadow (0:*3) :: lres
  integer, dimension (3) :: ibase, ires
  !hpf$ shadow (0:*3) :: ires
  integer, dimension (4) :: indx1
  !hpf$ distribute (block) :: maska, arreg, lbase, lres
  !hpf$ distribute (block) :: ibase, ires, indx1
  integer :: i, id
  integer :: errata = 0
  !
  print *
  print *, " Test HALO_REDUCE_XXX (HPF) (halo)"
  print *, " ================================="
  print *
  !
  maska = [ T, T, T, F ]
  lbase = [ T, T, T ]
  indx1 = [ 1, 1, 2, 2 ]
  ! lres = all_scatter (maska, base=lbase,indx1=indx1)
  lres = lbase
  call HALO_ALL_DEFINE (id, lres, indx1)
  call HALO_INIT       (id, lres, T)
  !hpf$ independent on home (indx1(I))  resident
  do  i = 1, 4
    lres (indx1(i)) = lres(indx1(i)) .and. maska (i)
  end do
  call HALO_REDUCE_ALL     (id, lres)
  call HALO_GLOBAL_INDEXES (id, indx1)
  print 53, "REDUCE_ALL", maska, lbase, indx1, lres
  call CHECK_LOG_RESULTS (lres, [T, F, T] , 3, errata)
  !
  maska = [ T, F, F, F ]
  lbase = [ F, F, T ]
  ! lres = any_scatter (maska, base=lbase,indx1=indx1)
  lres = lbase
  call HALO_LOCAL_INDEXES (id, indx1)
  call HALO_INIT          (id, lres, F)
  !hpf$ independent on home (indx1(i))  resident
  do  i = 1, 4
    lres (indx1(i)) = lres(indx1(i)) .or. maska (i)
  end do
  call HALO_REDUCE_ANY     (id, lres)
  call HALO_GLOBAL_INDEXES (id, indx1)
  print 53, "REDUCE_ANY", maska, lbase, indx1, lres
  call CHECK_LOG_RESULTS (lres, [T, F, T] , 3, errata)
  !
  arreg = [ 1, 2, 3, 4 ]
  ibase = [ 7, 8, 9 ]
  ! ires = copy_scatter (arreg, base=ibase,indx1=indx1)
  call HALO_LOCAL_INDEXES (id, indx1)
  ires = ibase
  !hpf$ independent on home (indx1(i))  resident
  do  i = 1, 4
    ires (indx1(i)) =  arreg(i)
  end do
  call HALO_REDUCE_COPY    (id, ires)
  call HALO_GLOBAL_INDEXES (id, indx1)
  print 51, "REDUCE_COPY", arreg, ibase, indx1, ires
  call CHECK_COPY_RESULTS (ires, [1, 3, 9], [2, 4, 9], 3, errata)
  !
  arreg = [ 1, 2, 3, 6 ]
  ibase = [ 1, 3, 7 ]
  !  same data for REDUCE_IALL and REDUCE_IANY
  !  ires = iall_scatter (arreg, base=ibase,indx1=indx1)
  call HALO_LOCAL_INDEXES (id, indx1)
  call HALO_INIT          (id, ires, -1)
  ires = ibase
  !hpf$ independent on home (indx1(i))  resident
  do  i = 1, 4
    ires (indx1(i)) = iand (ires(indx1(i)), arreg(i))
  end do
  call HALO_REDUCE_IALL    (id, ires)
  call HALO_GLOBAL_INDEXES (id, indx1)
  print 51, "REDUCE_IALL", arreg, ibase, indx1, ires
  call CHECK_INT_RESULTS (ires, [0, 2, 7] , 3, errata)
  ! ires = iany_scatter (arreg, base=ibase,indx1=indx1)
  call HALO_LOCAL_INDEXES (id, indx1)
  call HALO_INIT          (id, ires, 0)
  ires = ibase
  !hpf$ independent on home (indx1(i))  resident
  do  i = 1, 4
    ires (indx1(i)) = ior (ires(indx1(i)), arreg(i))
  end do
  call HALO_REDUCE_IANY    (id, ires)
  call HALO_GLOBAL_INDEXES (id, indx1)
  print 51, "REDUCE_IANY", arreg, ibase, indx1, ires
  call CHECK_INT_RESULTS (ires, [3, 7, 7] , 3, errata)
  !     ires = iparity_scatter (arreg, base=ibase,indx1=indx1)
  call HALO_LOCAL_INDEXES (id, indx1)
  call HALO_INIT (id, ires, 0)
  ires = ibase
  !hpf$ independent on home (indx1(I))  resident
  do  i = 1, 4
    ires (indx1(i)) = ieor (ires(indx1(i)), arreg(i))
  end do
  call HALO_REDUCE_IPARITY (id, ires)
  call HALO_GLOBAL_INDEXES (id, indx1)
  print 51, "REDUCE_IPARITY", arreg, ibase, indx1, ires
  call CHECK_INT_RESULTS (ires, [2, 6, 7] , 3, errata)
  !
  arreg = [ 1,  2, 3, 1 ]
  ibase = [ 4, -5, 7 ]
  ! ires = maxval_scatter (arreg, base=ibase,indx1=indx1)
  call HALO_LOCAL_INDEXES (id, indx1)
  call HALO_INIT          (id, ires, -2147483647)
  ires = ibase
  !hpf$ independent on home (indx1(i))  resident
  do  i = 1, 4
    ires (indx1(i)) = max (ires(indx1(i)), arreg(i))
  end do
  call HALO_REDUCE_MAXVAL  (id, ires)
  call HALO_GLOBAL_INDEXES (id, indx1)
  print 51, "REDUCE_MAXVAL", arreg, ibase, indx1, ires
  call CHECK_INT_RESULTS (ires, [4, 3, 7] , 3, errata)
  !
  arreg = [ 1, -2, -3, 6 ]
  ibase = [ 4, 3, 7 ]
  !ires = minval_scatter (arreg, base=ibase,indx1=indx1)
  call HALO_LOCAL_INDEXES (id, indx1)
  call HALO_INIT          (id, ires, 2147483647)
  ires = ibase
  !hpf$ independent on home (indx1(I))  resident
  do  i = 1, 4
     ires (indx1(i)) =  min (ires(indx1(i)), arreg(i))
  end do
  call HALO_REDUCE_MINVAL  (id, ires)
  call HALO_GLOBAL_INDEXES (id, indx1)
  call HALO_FREE (id)
  print 51, "REDUCE_MINVAL", arreg, ibase, indx1, ires
  call CHECK_INT_RESULTS (ires, [-2, -3, 7] , 3, errata)
  !
  maska = [ T, T, T, T ]
  lbase = [ T, F, F ]
  indx1 = [ 1, 1, 1, 2 ]
  ! lres = parity_scatter (maska, base=lbase,indx1=indx1)
  call HALO_ALL_DEFINE (id, lres, indx1)
  lres = lbase
  call HALO_INIT (id, lres, F)   ! F is neutral element for PARITY
  !hpf$ independent on home (indx1(i))  resident
  do  i = 1, 4
    lres (indx1(i)) =  lres(indx1(i)) .neqv. maska (i)
  end do
  call HALO_REDUCE_PARITY  (id, lres)
  call HALO_GLOBAL_INDEXES (id, indx1)
  call HALO_FREE (id)
  print 53, "REDUCE_PARITY", maska, lbase, indx1, lres
  call CHECK_LOG_RESULTS (lres, [F, T, F] , 3, errata)
  !
  arreg = [ 1, 2, 3, 1 ]
  ibase = [ 4, -5, 7 ]
  indx1 = [ 1, 1, 2, 2 ]
  ! ires = product_scatter (arreg, base=ibase,indx1=indx1)
  call HALO_ALL_DEFINE (id, ires, indx1)
  call HALO_INIT       (id, ires, 1)
  ires = ibase
  !hpf$ independent on home (indx1(i))  resident
  do I = 1, 4
     ires (indx1(I)) =  ires(indx1(i)) * arreg (i)
  end do
  call HALO_REDUCE_PRODUCT (id, ires)
  call HALO_GLOBAL_INDEXES (id, indx1)
  print 51, "REDUCE_PRODUCT", arreg, ibase, indx1, ires
  call CHECK_INT_RESULTS (ires, [ 8, -15, 7] , 3, errata)
  ! ires = sum_scatter (arreg, base=ibase,indx1=indx1)
  call HALO_LOCAL_INDEXES (id, indx1)
  call HALO_INIT          (id, ires, 0)
  ires = ibase
  !hpf$ independent on home (indx1(i))  resident
  do  i = 1, 4
    ires (indx1(i)) =  ires(indx1(i)) +  arreg (i)
  end do
  call HALO_REDUCE_SUM     (id, ires)
  call HALO_GLOBAL_INDEXES (id, indx1)
  call HALO_FREE (id)
  print 51, "REDUCE_SUM", arreg, ibase, indx1, ires
  !
  call CHECK_INT_RESULTS (ires, [7, -1, 7] , 3, errata)
  print *
  if  (errata .eq. 0) then
    print *, "reduce (halo): all tests PASSED"
  else
    print *, "reduce (halo): at least one test FAILED"
  end if
  51 format (A15," (",4I3,", base = ",3I3,", index =",4I2,") =",3I4)
  52 format (A15," (",4L3,", base = ",3I3,", index =",4I2,") =",3I4)
  53 format (A15," (",4L3,", base = ",3L3,", index =",4I2,") =",3L4)
  !
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine CHECK_INT_RESULTS (v1, v2, n, errata)
  implicit none
  integer, intent (in)    :: n
  integer, intent (inout) :: errata
  integer, dimension (n), intent(in) :: v1, v2
  !hpf$ distribute (block) :: v1
  if (any (v1 .ne. v2)) then
     errata = errata + 1
     print *, "--> ERROR, result: ", v1, ", shoud be ", v2
  end if
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine CHECK_LOG_RESULTS (v1, v2, n, errata)
  implicit none
  integer, intent(in)    :: n
  integer, intent(inout) :: errata
  logical, dimension (n), intent(in) :: v1, v2
  !hpf$ distribute (block) :: v1
  if (any (v1 .neqv. v2)) then
     errata = errata + 1
     print *, "--> ERROR, result: ", v1, ", shoud be ", v2
  end if
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine CHECK_COPY_RESULTS (v1, v2a, v2b, n, errata)
  implicit none
  integer, intent(in)    :: n
  integer, intent(inout) :: errata
  integer, dimension (n), intent(in) :: v1, v2A, v2B
  !hpf$ distribute (block) :: v1
  if (any ( (v1 .ne. v2a) .and. (v1 .ne. v2b)) ) then
    errata = errata + 1
    print *, "--> ERROR, result: ",v1,", shoud be ",v2a," or ",v2b
  end if 
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----

