! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! compilation with (adaptor + mpich):
!  adaptor -hpf -dm -free -o serial.exe serial.f90
!
! running with mpich:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 2 serial.exe
!  mpdallexit
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!Testing of HPF_SERIAL Interface
! serial: tests call of serial routine with a replicated and
!         with a distributed array
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program test_hpf_serial
  implicit none
  integer, parameter :: n  = 512
  integer, parameter :: np =  16    
  integer, dimension (n) ::  b, b1
  ! B will have an incarnation on all processors
  integer, dimension (np) :: res
  !hpf$ distribute (block) :: res
  !hpf$ distribute (block) :: b1
  integer :: res1, s, np1
  integer errata = 0, i
  !
  errata = 0
  np1 = number_of_processors ()
  print *
  print *, "Test HPF_SERIAL call on ",np1," procs"
  print *, "====================================="
  print *
  !
  ! do the tests serveral times to be sure about
  ! not pending send and receives of replication
  !
  do  i = 1, 5
    ! verify that serial call broadcasts values
    b = [1:n]
    call tarea (b, n)    ! initialize B with serial routine
    b = b - [1:n]
    ! verify that serial mechanism has bcast correctly
    ! check that B == 1 on all processors 
    res1 = sum (b)
    res  = res1
    s    = sum (res)
    if (s .ne. np * n) then
       print *, "Call of serial routine for rep array failed"
       print *, "RES = ", res
       print *, "should have been ", n , " * ", np, " = ", np*n
       errata = errata + 1
    end if
  end do
  !
  do  i = 1, 5
    ! verify that serial call broadcasts values
    b1 = [1:n]           ! B1 is a distributed array 
    call tarea (b1, n)   ! work on B1 within serial routine
    b1 = b1 - [1:n]
    ! verify that serial mechanism has bcast correctly
    ! check that B1 == 1 
    res1 = sum (b1)
    res  = res1
    s    = sum (res)
    if (s .ne. np * n) then
       print *, "Call of serial routine for dist array failed"
       print *, "RES = ", res
       print *, "should have been ", n , " * ", np, " = ", np*n
       errata = errata + 1
    end if
  end do
  !
  if (errata == 0) then
     print *, "HPF serial test: PASSED"
   else
     print *, "HPF serial test: FAILED"
  end if
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
extrinsic (hpf_serial) subroutine tarea (a, n)
 integer :: n
 integer, dimension (n) :: a
 a = a + 1
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
