! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! compilation with (adaptor + mpich):
!  adaptor -hpf -dm -free -o single.exe single.f90
!
! running with mpich:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 2 single.exe
!  mpdallexit
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!single:   Tests the correct behavior of the SINGLE directive
!          (only one incarnation of a not mapped variable)
!          hpfrun 4 single
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program simple
  implicit none
  integer, parameter :: n =  512
  integer, parameter :: np = 16    
  integer, dimension (n) ::  a, b
  !A has only one incarnation on the HOST
  !B will have an incarnation on all processors
  !adp$ single a 
  integer,  dimension (np) :: res
  !hpf$ distribute (block) :: res
  integer :: res1, s
  integer :: errata=0, np1
  !
  np1 = number_of_processors ()
  print *
  print *, "Test single layout on ",np1," procs"
  print *, "=================================="
  print *
  !
  !verify that values of single version are replicated
  b = 0
  a = [1:n] + 1
  b = a
  b = b - [1:n]
  !
  !check that B == 1 on all processors 
  res1 = sum (b)
  res  = res1
  s    = sum (res)
  if (s .ne. np * n) then
     print *, "res = ", res
     print *, "should have been ", n , " * ", np, " = ", np*n
     errata = errata + 1
  end if
  !
  !verify that remapping from replicated to single works fine
  b = 0
  call tarea1 (b, n)    ! initialize B on a single version
  !verify that copy out has broadcast the values
  !
  !check that B == 1 on all processors 
  res1 = sum (b)
  res  = res1
  s    = sum (res)
  !
  if (s .ne. np * n) then
     print *, "Call of single routine for rep array failed"
     print *, "res = ", res
     print *, "should have been ", n , " * ", np, " = ", np*n
     errata = errata + 1
  end if
  !
  !make sure that single version is remapped to replicated
  a = [1:n]
  call tarea2 (a, b, n)
  b = b - [1:n]
  !
  !check that B == 1 on all processors 
  !
  res1 = sum (b)
  res  = res1
  s    = sum (res)
  !
  if (s .ne. np * n) then
     print *, "Call of rep routine for single array failed"
     print *, "res = ", res
     print *, "should have been ", n , " * ", np, " = ", np*n
     errata = errata + 1
  end if
  !
  if (errata == 0) then
    print *, "single layout test: PASSED"
  else
    print *, "single layout test: FAILED"
  end if
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine tarea1 (a, n)
  implicit none
  integer :: n
  integer, dimension (n) :: a
  !adp$ single a
  a = 1
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine tarea2 (a, b, n)
  implicit none
  integer :: n
  integer, dimension (n) :: a, b
  a = a + 1
  b = a
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
