! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! Compilation with (ADAPTOR + MPICH):
!  adaptor -hpf -dm -free -o fft.exe fft.f90
!
! Running with MPICH:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 2 fft.exe
!  mpdallexit
! =================================================================
! running details:
!  si n = 8 000 entonces m = n^2, ram = m * 8 /1e6 = 512 [mbytes]
!  pero como aqui hay una copia de a se tiene ram = 1024 mbytes
! -----------------------------------------------------------------
! Programm zur Berechnung der Potenzen der Einheitswurzeln
! n  Problemgroesse; 2er Potenz
! n2 Anzahl der zu berechnenden Potenzen; n2 = n/2
! Direkt Fouriersynthese bzw. Fourieranalyse (1/-1)
! w (n) Feld zur Aufnahme der Potenzen der Einheitswurzeln
! -----------------------------------------------------------------
program fast_fourier_transform
  implicit none
  integer, parameter  :: k1  = 8        ! first problem size
  integer, parameter  :: nit = 9        ! number of problems
  real, parameter     :: eps = 0.002    ! maximal error allowed
  integer :: k          !  logarithm of problem size
  integer :: n          !  problem size = 2**k
  integer :: n2         !  is n/2
  complex, allocatable, dimension (:) ::  w, c
  real, allocatable, dimension (:) ::  f      
  !hpf$ distribute (block) :: c, f
  real :: maxf
  integer :: i, it
  real :: direkt
  integer :: errors
  errors = 0
  print *, 'test of fft (inspector/executor)'
  !
  k = k1                    !  first problem
  do it = 1, nit
     n  = 2**k
     n2 = 2**(k-1)
     allocate (w(0:n2-1), c(0:n-1), f(0:n-1))
     ! random initialization
     call random_number (f)
     !
     !hpf$ independent
     do i=0,n-1
       c(i) = f(i)
     end do
     !
     direkt = -1
     call presort (n,k,c)
     call roots (w,n2,n,direkt)
     call transform (c,w,n,n2,k)
     ! inverse fft
     direkt = 1
     call presort (n,k,c)
     call roots (w,n2,n,direkt)
     call transform (c,w,n,n2,k)
     !hpf$ independent
     do i=0,n-1 
       f(i) = abs (c(i)/n - f(i))
     end do
     maxf = 0.0
     !hpf$ independent, reduction (maxf)
     do i = 0, n-1
        maxf = max (maxf, f(i))
     end do
     write (*,*)'fft, k= ', k,', n= ', n,', maxdiff = ', maxf
     if (maxf > eps) then
        errors = errors + 1
     end if
     !
     deallocate (f, c, w)
     k = k+1
  end do
  if (errors .eq. 0) then
     print *, 'fft (unstr): all tests passed'
  else
     print *, 'fft (unstr): at least one test failed'
  end if
  !
end program
! -----------------------------------------------------------
! ---------------- Umordnung der Speicherplaetze ------------
! ----------------     nach Cooley und Tukey     ------------
! -----------------------------------------------------------
subroutine presort (n,k,c)
  integer, intent (in) :: n, k
  complex c(0:n-1)
  complex h(0:n-1)
  !hpf$ distribute (block) :: h, c, index
  integer kl,kn
  integer hi, hj, index(0:n-1), ix
  integer :: i, l
  ! initialization of index
  !hpf$ independent
  do i = 0, n-1 
    index(i) = i
  end do
  ! executing bit reverse for all elements of index
  do l=1,k
    kl = 2**(l-1)       
    kn = n / (kl + kl)    
    !hpf$ independent, new (ix, hi, hj)
    do i = 0, n-1 
      ix = index(i)
      ! hi = ix / 2
      hi = ishft (ix,-1)
      ! hj = hi / kn
      hj = ishft (hi,l-k)
      ! hi = mod (hi,kn)
      hi = iand (hi,kn-1)
      index (i) = hi + 2*kn*hj + mod(ix,2)*kn
    end do
  end do
  h = c(index)
  !hpf$ independent
  do i = 0, n-1 
    c (i) = h (i)
  end do
end subroutine
! -----------------------------------------------------------
! --------- Berechnung der Potenzen der Einheitswurzeln -----
! --------- der Alg. spiegelt eine Rekurrenz erster     -----
! --------- Ordnung wieder.                             -----
! -----------------------------------------------------------
subroutine roots (w,n2,n,direkt)
  integer, intent(in) :: n, n2
  complex, dimension (0:n2-1) :: w
  complex :: omega
  real :: phi, direkt
  integer i
  !
  phi = direkt * 8.0 * atan(1.0) / real(n)
  omega = cmplx(cos(phi),sin(phi))
  !     
  w(0) = 1
  do i = 1,n2-1
     w(i) = w(i-1) * omega
  end do
end subroutine
! -----------------------------------------------------------
! -------- Transformation nach Cooley-Tukey -----------------
! -----------------------------------------------------------
! Feld c repraesentiert die Fourierkoeffizienten
! Feld w beinhaltet die Potenzen der Einheitswurzeln
! -----------------------------------------------------------
subroutine transform (c,w,n,n2,k)
  integer :: n, n2, k
  complex :: c (0:n-1), w (0:n2-1), h (0:n-1)
  !hpf$ distribute (block) :: h, c
  integer, dimension (0:n-1) :: index
  !hpf$ distribute (block) :: index
  complex :: t
  integer :: kl, kn, kl1, hi, ni
  integer :: i, l      
  do l=1,k
    kl = 2**(l-1)
    kn = n2/kl
    kl1 = 2 * kl
    !hpf$ independent 
    do i = 0, n-1
       index(i) = ieor (i,kl)
    end do
    h = c(index)
    !hpf$ independent, new(hi, ni)
    do i = 0, n-1 
      ! hi = mod (i,kl1)
      hi = iand (i,kl1-1)
      ! ni = mod (i,kl)
      ni = iand (i,kl-1)
      if (hi .ge. kl) then
        c (i) = h (i) - c (i) * w (ni*kn)
      else
        c (i) = c (i) + w (ni*kn) * h (i)
      end if
    end do
  end do
end subroutine
! -----------------------------------------------------------
