! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! Compilation with (ADAPTOR + MPICH):
!  adaptor -hpf -dm -free -o lumm_arb.exe lumm_arb.f90
!
! Running with MPICH:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 2 lumm_arb.exe
!  mpdallexit
!
! -----------------------------------------------------------------
program lumm_arb_test
  implicit none
  real,    parameter :: eps = 0.001
  integer, parameter :: n = 300   ! nro de incognitas
  integer, parameter :: h =   6   ! nro de ...
  integer, dimension (h) :: siza, map1, map2
  real   , dimension (h) :: rmap
  !
  real    :: maxdiff
  integer :: errata = 0, i, np
  !
  siza = [ 40, 60, 40, 60, 50, 50 ]   ! sums up to n
  i = sum (siza)
  if (i .ne. n) stop " sum (siza) <> n "
  !
  np = number_of_processors ()
  !
  call random_number (rmap)
  map1 = rmap * np1 + 1
  !
  call random_number (rmap)
  map2 = rmap * np2 + 1
  !
  call lumm_arb (n, np, h, siza, map1, map2, maxdiff)
  !
  if (maxdiff .gt. eps) then
    errata = errata + 1
    print *, "--> : result is not exact enough for lumm_arb ", n
  end if
  if (errata .eq. 0) then
    print *, "lumm, np =",np," : all tests passed"
  else
    print *, "lumm, np =",np," : at least one test failed"
  end if
  !
end program
! -----------------------------------------------------------------
subroutine lumm_arb (n, np, h, siza, map1, map2, maxdiff)
  implicit none
  integer               , intent (in)  :: n
  integer               , intent (out) :: np
  integer               , intent (in)  :: h
  integer, dimension (h), intent (in)  :: siza, map1, map2
  real                  , intent (out) :: maxdiff
  real, parameter :: eps = 0.001
  real, dimension (n,n)  :: a, lu, b
  real, dimension (n)    :: col, row
  integer                :: i, j, k
  !hpf$ distribute a(arbitrary(h,siza,map1),arbitrary(h,siza,map2))
  !hpf$ align lu (i,j) with a (i,j)
  !hpf$ align  b (i,j) with a (i,j)
  !hpf$ align col (i)  with a (i,*)
  !hpf$ align row (j)  with a (*,j)
  real                :: x
  integer             :: h1
  real, dimension (5) :: time
  integer :: np1, np2
  !
  call get_shape (a, h, siza, map1, map2, np1, np2)
  print *, "proc shape = ", np1, " x ", np2
  return
  !
  call init (a, n)
  lu = a
  !
  print *,"LU decomposition "
  call system_clock (count = h1)   ! t1 = second ()
  !
  do k = 1, (n - 1)
     x = lu (k,k)
     if (abs (x) .lt. eps) then
       print *, "no pivoting, error at k = ", k, " x = ", x
       stop
     end if
     !hpf$ independent
     lu  (k+1:n,k)     = lu (k+1:n,k) / lu (k,k)
     col (k+1:n)       = lu (k+1:n,k)
     row (k+1:n)       = lu (k,k+1:n)
     lu  (k+1:n,k+1:n) = lu (k+1:n,k+1:n)      &
              - spread (col (k+1:n), 2, n-k)   &
              * spread (row (k+1:n), 1, n-k)
  end do ! k
  call  walltime ( h1, time (1) )
  !
  print *
  print *, "LU ready, now finds B such  A * B = I "
  call system_clock (count = h1)   ! t1 = second ()
  !
  b = 0.0
  forall (i=1:n) b (i,i) = 1.0
  do  j = 1, n
    col (j+1:n)   = lu (j+1:n,j)
    row (1:n)     =  b (j,1:n)
    b (j+1:n,1:n) =  b (j+1:n,1:n)      &
         - spread (col (j+1:n), 2, n  ) & 
         * spread (row (1:n)  , 1, n-j)
  end do
  call  walltime (h1, time (2) )
  !
  print *, "now finds x with r * x = I "
  call system_clock (count = h1)   ! t1 = second ()
  !
  do j = n, 1, -1
    col (1:n)   = lu (1:n,j)
    b   (j,1:n) =  b (j,1:n) / col (j)
    row (1:n)   =  b (j,1:n)
    b   (1:j-1,1:n) = b (1:j-1,1:n)   & 
          - spread (col (1:j-1), 2, n) &
          * spread (row (1:n), 1, j-1)
  end do
  call  walltime (h1, time (3) )
  !
  print *, "we have the inverse, now do LU = A * B "
  call system_clock (count = h1)   ! t1 = second ()
  lu = 0
  do  k = 1,n
    col = a (1:n,k)
    row = b (k,1:n)
    lu (1:n,1:n) = lu (1:n,1:n)  &
        + spread (col, 2, n)     &
        * spread (row, 1, n)
  end do
  call  walltime (h1, time (4) )
  !
  print *, "matmul correct, now check = I "
  call system_clock (count = h1)   ! t1 = second ()
  a = lu
  forall (i=1:n) a(i,i) = a(i,i) - 1.0
  a = abs (a)
  maxdiff = maxval (a)
  call  walltime (h1, time (5) )
  !
  print * , "maximal eps = ", maxdiff
  !
  print *, "ciclos    = ", h1
  print *, "LU decomposition ; time [s] = ", time (1)
  print *, "finds B          ; time [s] = ", time (2)
  print *, "finds x          ; time [s] = ", time (3)
  print *, "product A B      ; time [s] = ", time (4)
  print *, "check I          ; time [s] = ", time (5)
  !
end subroutine
! -----------------------------------------------------------------
! a will be set to a matrix with the following entries:
! (this matrix works fine to make a lu decomposition)
! 1.0  1.0  0.0  0.0  0.0
! 1.0  0.0  2.0  0.0  0.0
! 1.0  0.0  0.0  3.0  0.0
! 1.0  0.0  0.0  0.0  4.0
! 1.0  0.0  0.0  0.0  0.0
! -----------------------------------------------------------------
subroutine init (a, n)
  implicit none
  integer, intent (in)  :: n
  real, dimension (n,n) :: a
  !hpf$ distribute (block,block) :: a
  integer :: i
  a = 0.0
  forall (i=1:n-1) a(i,i+1) = real(i)
  a (:,1) = 1.0
end subroutine
! -----------------------------------------------------------------
subroutine get_shape (a, h, siza, map1, map2, np1, np2)
  use hpf_library
  implicit none
  real, dimension (:,:) , intent (in)  :: a
  integer               , intent (in)  :: h
  integer, dimension (h), intent (in)  :: siza, map1, map2
  integer               , intent (out) :: np1, np2
  !hpf$ distribute a(arbitrary(h,siza,map1),arbitrary(h,siza,map2))
  integer :: siza (2)
  call hpf_distribution (a, processors_shape=siza)
  np1 = siza (1)
  np2 = siza (2)
end subroutine
! -----------------------------------------------------------------
subroutine walltime (h1,time)
  implicit none
  real    :: time
  integer :: h1, h2, h3
  call system_clock (h2, h3)
  h1 = h2 - h1
  time = float (h1) / float (h3)
end subroutine
! =================================================================
