! =================================================================
! measure matrix-matrIX multiplication
!
! compilation with ADAPTOR and running with MPI:
!  adaptor -hpf -free -dm -o matmul.exe matmul.f90
!  mpirun -machinefile machi.dat -nolocal -np 8 matmul.exe
!
!  adaptor -hpf -free -c matmul.f90
!  adaptor -hpf -free -dm -o matmul.exe matmul.o
!  mpirun -machinefile machi.dat -nolocal -np 8 matmul.exe
!
! running details:
!  si n = 8 000 entonces m = n^2, ram = m * 8 /1e6 = 512 [Mbytes]
!  pero como aqui hay una copia de A se tiene ram = 1024 Mbytes
!
! output example:
! -----------------------------------------------------------------
program mxm
  implicit none
  integer :: n
  double precision, dimension (:,:), allocatable :: a, b, c
!hpf$ distribute (block,*) :: a
!hpf$ distribute (*,block) :: b
!hpf$ distribute (block,block) :: c
  integer :: tickstart, tickstop, tickrate, np
  real :: time, mflops
  !
  write (*,*)
  write (*,*)' matmul c = a * b '
  write (*,*)' n = ? '
  read (*,*) n
  !
  allocate ( a(n,n), b(n,n), c(n,n) )
  !
  a = 1.0
  b = 1.4
  c = 0.7
  !
  call system_clock (tickstart, tickrate)
  call matmulti (c,a,b,n)
  call system_clock (tickstop)
  !
  time = float (tickstop - tickstart) / float (tickrate)
  mflops = real (n) ** 3 / (1000000.0 * time)
  np = number_of_processors ()
  !
  write (*,*)
  write (*,*)' sum (c) = ', sum (c)
  write (*,*)
  !
  write (*,*)' time   = ', time, ' seconds'
  write (*,*)' mflops = ', mflops, ' on ', np, ' processors'
  write (*,*)' mflops = ', mflops/np, ' for one processor'
  write (*,*)
  !
  deallocate ( a, b, c )
  !
end program
!
subroutine matmulti (c,a,b,n)
  implicit none
  integer :: n
  double precision, dimension (n,n) :: a, b, c
!hpf$ distribute a (block,block)
  integer :: i, j, k
  c = 0.0d0
!hpf$ parallel, reduction (c)
  do j = 1, n
!hpf$ independent, on home (a(:,k))
    do k = 1, n
!hpf$ independent, on home (a(i,k))
      do i = 1, n
        c (i,j) = c(i,j) + a (i,k) * b (k,j)
      end do
    end do
  end do
end subroutine
! -----------------------------------------------------------------
