! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! Compilation with (ADAPTOR + MPICH):
!  adaptor -hpf -dm -free -o pi2.exe pi2.f90
!
! Running with MPICH:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 2 pi2.exe
!  mpdallexit
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program  calcula_pi
  implicit none
  ! parametros de precision usuales
  integer, parameter :: idp = kind (1.0d0)
  integer, parameter :: isp = kind (1.0e0)
  integer, parameter :: iin = kind (1)
  !
  real    (idp), dimension (:), allocatable :: area
  !hpf$ distribute (block) :: area
  real    (idp) :: f, x, h, pia
  integer (iin) :: p, e, n
  integer (iin) :: i, ier
  !para medir tiempos
  integer (iin) :: h1, h2, h3, h4
  real (idp)    :: time
  !
  p = number_of_processors ()
  e = 8
  n = 10 ** e      ! nro   de intervalos
  h = 1.0d0/n      ! ancho de los intervalos
  !
  !aloca
  allocate (area (n), stat = ier)
  if (ier .ne. 0) stop "error: aloca area"
  !
  !inicia cronometro;  
  call system_clock (count = h1)   ! t1 = second ()
  !
  ! tarea
  !hpf$ independent, new (f,x)
  do i = 1, n 
    x = h * (1.0*i - 0.5d0)
    f = 4.0d0 / (1.0d0 + x * x)
    area (i) = h * f               ! deberia ser f (x) pero no-anda
  end do
  pia = sum (area)
  !
  ! consulta cronometro
  call system_clock (count = h2, count_rate = h3)  ! t2 = second ()
  h4   = h2 - h1                       ! en ciclos
  time = real (h4) / real (h3)         ! en segundos
  !
  ! salida
  write (*,*)
  write (*,*) "exponente           ; e  = ", e
  write (*,*) "nro de intervalos   ; n  = ", n
  write (*,*) "pi aproximado       ; pi = ", pia
  write (*,*)
  write (*,*) "nro de procesadores ; p  = ", p
  write (*,*) "tiempo [ciclos]     ; c  = ", h4
  write (*,*) "tiempo [seg.]       ; t  = ", time
  write (*,*)
  !
  !dloca
  deallocate (area, stat = ier)
  if (ier .ne. 0) stop "error: dloca area"
  !
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
