! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! compilation with (adaptor + mpich):
!  adaptor -hpf -dm -free -o location.exe location.f90
!
! running with mpich:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 2 location.exe
!  mpdallexit
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program test_location
  implicit none
  integer :: errata = 0
  !
  print *
  print *, " Test MINLOC/MAXLOC intrinsics"
  print *, " ============================="
  print *
  !
  call test_minloc (errata)
  print *
  !
  call test_maxloc (errata)
  print *
  !
  if (errata .eq. 0) then
    print *, "Location: all tests PASSED"
  else
    print *, "Location: at least one test FAILED"
  end if
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine test_minloc (errata)
  implicit none
  integer, intent (inout) :: errata
  integer, dimension (5:8)       :: b
  integer, dimension (2:4,-2:1)  :: a
  !hpf$ distribute (block)       :: b
  !hpf$ distribute (block,block) :: a
  integer :: loc1 (1), loc2 (2)
  integer :: i, loc
  !
  b = [ 4, 3, 6, 5 ]
  loc1 = minloc (b)
  print 11, b, loc1
  11  format (" MINLOC (",4i5," ) = ",i5)
  call check_results (loc1, [ 2 ], 1, errata)
  !
  a (2,:) = [ 0, -5,  8, -3 ]
  a (3,:) = [ 3,  4, -1,  2 ]
  a (4,:) = [ 1,  5,  6, -4 ]
  !
  loc2 = minloc (a, mask = a > -4)
  do i =  2, 4
     print 12, a(i,:)
  end do
  print *, "MINLOC (a, mask = a > -4)  = ", loc2
  12  format (" a = ", 4i5)
  call check_results (loc2, [ 1, 4 ], 2, errata)
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine test_maxloc (errata)
!  implicit none
  integer, intent (inout) :: errata
  integer, dimension (5:8)       :: b
  integer, dimension (2:4,-2:1)  :: a
  !hpf$ distribute (block)       :: b
  !hpf$ distribute (block,block) :: a
  integer :: loc1 (1), loc2 (2)
  integer :: i, loc
  !
  b = [ 2, 6, 9, 3 ]
  loc1 = maxloc (b)
  print 11, b, loc1
  call check_results (loc1, [ 3 ], 1, errata)
  11  format (" MAXLOC (",4i5," ) = ",i5)
  !
  a (2,:) = [ 0, -5,  8, -3 ]
  a (3,:) = [ 3,  4, -1,  2 ]
  a (4,:) = [ 1,  5,  6, -4 ]
  !
  loc2 = maxloc (a, mask = a < 6)
  do i =  2, 4
     print 12, a (i,:)
  end do
  print *, "MAXLOC (A, mask = A < 6)  = ", loc2
  12  format (" A = ", 4i5)
  call check_results (loc2, [ 3, 2 ], 2, errata)
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine check_results (v1, v2, n, errata)
  implicit none
  integer, intent (in)    :: n
  integer, intent (inout) :: errata
  integer, dimension (n), intent (in) :: v1, v2
  if (any (v1 .ne. v2)) then
     errata = errata + 1
     print *, "--> ERROR, result: ", v1, ", shoud be ", v2
  end if
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
