! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! compilation with (adaptor + mpich):
!  adaptor -hpf -dm -free -o reduction.exe reduction.f90
!
! running with mpich:
!  mpdboot -n 21 -f ~/mpd.hosts
!  mpdtrace
!  mpiexec -machinefile ~/machi.dat -np 2 reduction.exe
!  mpdallexit
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program test_reductions
  implicit none
  integer :: errata = 0
  !
  print *
  print *, "Test Array Reductions"
  print *, "====================="
  print *
  !
  print *, "B  =  ( 1  3  5)   C = ( 0  3  5 )"
  print *, "      ( 2  4  6)       ( 7  4  8 )"
  !
  call test_all   (errata)
  call test_any   (errata)
  call test_count (errata)
  !
  print *, "B  =  ( 1  3  5)   MASK = ( F  F  T )"
  print *, "      ( 2  4  6)          ( T  T  T )"
  !
  call test_minval (errata)
  call test_maxval (errata)
  !
  print *
  if (errata .eq. 0) then
     print *, "Reductions: all tests PASSED"
  else
     print *, "Reductions: at least one test FAILED"
  end if
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine test_all (errata)
  implicit none
  integer, intent (inout) :: errata
  logical, parameter :: T = .true., F = .false.
  integer, dimension (2,3) :: b, c
  !hpf$ distribute (block,block) :: b, c
  logical :: l1, l2 (2), l3 (3)
  !
  b(1,:) = [ 1, 3, 5 ]
  b(2,:) = [ 2, 4, 6 ]
  !
  c(1,:) = [ 0, 3, 5 ]
  c(2,:) = [ 7, 4, 8 ]
  !
  l1 = all ([ T, F, F, T ])
  print *, "ALL ([ T, F, F, T ]) = ", l1
  call check_log_res (l1, F, errata)
  !
  l3 = all (mask = b .ne. c, dim=1)
  print *, "ALL (mask = b .ne. c, dim=1) = ", l3
  call check_log_results (l3, [ T, F, F ], 3, errata)
  !
  l2 = all (b .ne. c, dim=2)
  print *, "ALL (b .ne. c, dim=2) = ", l2
  call check_log_results (l2, [ F, F ], 2, errata)
  !
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine test_any (errata)
  implicit none
  integer, intent (inout) :: errata
  logical, parameter :: T = .true., F = .false.
  integer, dimension (2,3) :: b, c
  !hpf$ distribute (block,block) :: b, c
  logical :: l1, l2(2), l3 (3)
  !
  b(1,:) = [ 1, 3, 5 ]
  b(2,:) = [ 2, 4, 6 ]
  !
  c(1,:) = [ 0, 3, 5 ]
  c(2,:) = [ 7, 4, 8 ]
  !
  l1 = any ([ T, F, T ])
  print *, "ANY ([ T, F, T ]) = ", l1
  call check_log_res (l1, T, errata)
  !
  l3 = any (mask = b .ne. c, dim=1)
  print *, "ANY (mask = b .ne. c, dim=1) = ", l3
  call check_log_results (l3, [ T, F, T ], 3, errata)
  !
  l2 = any (b .ne. c, dim=2)
  print *, "ANY (b .ne. c, dim=2) = ", l2
  call check_log_results (l2, [ T, T ], 2, errata)
  !
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine test_count (errata)
  implicit none
  integer, intent (inout) :: errata
  integer, dimension (2,3) :: b, c
  logical, parameter :: T = .true., F = .false.
  integer :: i1, i2(2), I3(3)
  !
  b(1,:) = [ 1, 3, 5 ]
  b(2,:) = [ 2, 4, 6 ]
  !
  c(1,:) = [ 0, 3, 5 ]
  c(2,:) = [ 7, 4, 8 ]
  !
  i1 = count ([ T, F, T ])
  print *, "COUNT ([ T, F, T ]) = ", i1
  call check_int_res (i1, 2, errata)
  !
  I3 = count (mask = b .ne. c, dim=1)
  print *, "COUNT (mask = b .ne. c, dim=1) = ", I3
  call check_int_results (I3, [ 2, 0, 1 ], 3, errata)
  !
  i2 = count (b .ne. c, dim=2)
  print *, "COUNT (b .ne. c, dim=2) = ", i2
  call check_int_results (i2, [ 1, 2 ], 2, errata)
  !
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine test_minval (errata)
  implicit none
  integer, intent (inout) :: errata
  logical, parameter :: T = .true., F = .false.
  integer, dimension (2,3) :: b
  logical, dimension (2,3) :: maska
  !hpf$ distribute (block,block) :: b, maska
  integer i1, i2 (2), i3(3)
  !
  b(1,:) = [ 1, 3, 5 ]
  b(2,:) = [ 2, 4, 6 ]
  !
  maska(1,:) = [ F, F, T ]
  maska(2,:) = [ T, T, T  ]
  !
  i1 = minval ([ 1, 2, 3 ])
  print *, "MINVAL ([ 1, 2, 3 ]) = ",  i1
  call check_int_res (i1, 1, errata)
  !
  i1 = minval ([ 1, 2, 3 ], [ F, F, T])
  print *, "MINVAL ([ 1, 2, 3 ], [ F, F, T]) = ", i1
  call check_int_res (i1, 3, errata)
  !
  i3 = minval (array=b, dim=1)
  print *, "MINVAL (array=b, dim=1) = ", i3
  call check_int_results (i3, [ 1, 3, 5 ], 3, errata)
  !
  i2 = minval (array=b, dim=2)
  print *, "MINVAL (array=b, dim=2) = ", i2
  call check_int_results (i2, [ 1, 2 ], 2, errata)
  !
  i3 = minval (array=b, mask=maska, dim=1)
  print *, "MINVAL (array=b, mask=maska, dim=1) = ", i3
  call check_int_results (i3, [ 2, 4, 5 ], 3, errata)
  !
  i2 = minval (array=b, mask=maska, dim=2)
  print *, "MINVAL (array=b, mask=maska, dim=2) = ", i2
  call check_int_results (i2, [ 5, 2 ], 2, errata)
  !
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine test_maxval (errata)
  implicit none
  integer, intent (inout) :: errata
  logical, parameter :: T = .true., F = .false.
  integer, dimension (2,3) :: b
  logical, dimension (2,3) :: maska
  !hpf$ distribute (block,block) :: b, maska
  integer :: i1, i2(2), i3 (3)
  !
  b (1,:) = [ 1, 3, 5 ]
  b (2,:) = [ 2, 4, 6 ]
  !
  maska (1,:) = [ F, F, T ]
  maska (2,:) = [ T, T, T  ]
  !
  i1 = maxval ([ 1, 2, 3 ])
  print *, "MAXVAL ([ 1, 2, 3 ]) = ",  i1
  call check_int_res (i1, 3, errata)
  !
  i1 = maxval ([ 1, 2, 3 ], [ T, T, F])
  print *, "MAXVAL ([ 1, 2, 3 ], [ T, T, F]) = ", i1
  call check_int_res (i1, 2, errata)
  !
  i3 = maxval (array=b, dim=1)
  print *, "MAXVAL (array=b, dim=1) = ", i3
  call check_int_results (i3, [ 2, 4, 6 ], 3, errata)
  !
  i2 = maxval (array=b, dim=2)
  print *, "MAXVAL (array=b, dim=2) = ", i2
  call check_int_results (i2, [ 5, 6 ], 2, errata)
  !
  i3 = maxval (array=b, mask=maska, dim=1)
  print *, "MAXVAL (array=b, mask=maska, dim=1) = ", i3
  call check_int_results (i3, [ 2, 4, 6 ], 3, errata)
  !
  i2 = maxval (array=b, mask=maska, dim=2)
  print *, "MAXVAL (array=b, mask=maska, dim=2) = ", i2
  call check_int_results (i2, [ 5, 6 ], 2, errata)
  !
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine check_log_res (v1, v2, errata)
  implicit none
  integer, intent(inout) :: errata
  logical, intent(in)    :: v1, v2
  if (v1 .neqv. v2) then
     errata = errata + 1
     print *, "--> ERROR, result: ", v1, ", should be ", v2
  end if
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine check_int_res (v1, v2, errata)
  implicit none
  integer, intent(inout) :: errata
  integer, intent(in)    :: v1, v2
  if (v1 .ne. v2) then
     errata = errata + 1
     print *, "--> ERROR, result: ", v1, ", should be ", v2
  end if
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine check_log_results (v1, v2, n, errata)
  implicit none
  integer, intent(in)    :: n
  integer, intent(inout) :: errata
  logical, dimension (n), intent(in) :: v1, v2
  if (any (v1 .neqv. v2)) then
     errata = errata + 1
     print *, "--> ERROR, result: ", v1, ", should be ", v2
  end if
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
subroutine check_int_results (v1, v2, n, errata)
  implicit none
  integer, intent(in)    :: n
  integer, intent(inout) :: errata
  integer, dimension (n), intent(in) :: v1, v2
  if (any (v1 .ne. v2)) then
     errata = errata + 1
     print *, "--> ERROR, result: ", v1, ", should be ", v2
  end if
end subroutine
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----

