program lumm3
  implicit none
  ! parametros de precision usuales
  integer, parameter :: idp = kind (1.0d0)
  integer, parameter :: isp = kind (1.0e0)
  integer, parameter :: iin = kind (1)
  !
  real    (idp), parameter :: tol = 1e-9
  real    (idp), dimension (:,:), allocatable :: a, a0
  real    (idp), dimension (:,:), allocatable :: b, b0, r
  real    (idp), dimension   (:), allocatable :: s
  integer (iin) :: i, j, k, m, n, p
  !hpf$ distribute a (*,block)
  !hpf$ align (i,j) with a (i,j) :: a0
  !hpf$ align (i,j) with a (i,j) :: b0, b, r
  !hpf$ align (j)   with a (*,j) :: s
  !
  real    (idp) :: dt, ops, mflops
  real    (idp), dimension (4) :: time
  integer (iin), dimension (6) :: ier
  integer (iin) :: h1, h2, h3
  integer (iin) :: errors, errata
  !
  integer (iin) :: n_unknowns, n_loads
  namelist /lu_basic_data/ n_unknowns, n_loads
  !
  ! lee del disco
  errors = 0
  errata = 1
  open  (1, file = "lu_basic.dat", status = "old", err = 100)
  read  (1,nml = lu_basic_data)   ! notar que los dos namelist estan en el mismo
  write (*,nml = lu_basic_data)  
  close (1, status = "keep")
  !
  n = n_unknowns
  m = n_loads
  p = number_of_processors ()
  !
  print *
  print *, "matrix order         ; n = ",n
  print *, "loads number         ; m = ",m
  print *, "number of processors ; p = ",p
  !
  !aloca en orden 
  ier = 0
  allocate (a  (1:n,1:n), stat = ier (1) )
  allocate (a0 (1:n,1:n), stat = ier (2) )
  allocate (b  (1:n,1:m), stat = ier (3) )
  allocate (b0 (1:n,1:m), stat = ier (4) )
  allocate (r  (1:n,1:m), stat = ier (5) )
  allocate (s  (1:m)    , stat = ier (6) )
  if ( any (ier .ne.0) ) stop "allocation failed" 
  print *, "arrays are allocated ... "
  !   1.0  1.0  0.0  0.0  0.0
  !   1.0  0.0  2.0  0.0  0.0
  !   1.0  0.0  0.0  3.0  0.0
  !   1.0  0.0  0.0  0.0  4.0
  !   1.0  0.0  0.0  0.0  0.0
  a = 0.0
  forall (i=1:n-1) a (i,i+1) = dble (i)
  a (:,1) = 1.0
  forall (i=1:n) b (i,1:m) = 1.0
  a0 = a
  b0 = b
! print *, "b"
! do  i = 1, n  
!   print *, real (b0 (i,:))
! end do
! print *, "A"
! do  i = 1, n  
!   print *, real (a0 (i,:))
! end do
  !
  !inicia tiempos  
  time = 0.0
  !
  !para medir tiempos:
  !call cpu_time (t1)      ! nativa en F95 equivale a t1 = second ()
  !call walltime (t1)      ! de usuario
  !call system_clock (h1)  ! a veces es mejor
  !
  print *, "LU-factorization; A = LU ... "
  call system_clock (count = h1)
  do  k = 1, (n - 1)
    if ( abs (a (k,k) ) .lt. tol ) then
      print *, "no pivoting, error at k = ", k
      stop
    end if
    a (k+1:n,k:k)   = a (k+1:n,k:k) / a (k,k)
    a (k+1:n,k+1:n) = a (k+1:n,k+1:n) -              &
    spread (a (k+1:n,k), dim = 2, ncopies = n-k) *   &
    spread (a (k,k+1:n), dim = 1, ncopies = n-k)
  end do
  call system_clock (count = h2)
  time (1) = dble (h2 - h1)
  !
  print *, "forward substitution; solves  L Y = b ... "
  call system_clock (count = h1)
  do j = 1, n
      b (j+1:n,1:m) = b (j+1:n,1:m) -                &
      spread (a (j+1:n,j), dim=2, ncopies = m  ) *   &
      spread (b (j,1:m)  , dim=1, ncopies = n-j)
    end do
  call system_clock (count = h2)
  time (2) = dble (h2 - h1)
  ! print *, "y"
  ! print *,  real (b)
  !
  print *, "backward substitution; solves  U x = y ... "
  call system_clock (count = h1)
  do j = n, 1, -1
    b (j,1:m) = b (j,1:m) / a (j,j)
    b (1:j-1,1:m) = b (1:j-1,1:m) -                   &
    spread (a (1:j-1,j), dim = 2, ncopies =   m) *    &
    spread (b (j,1:n)  , dim = 1, ncopies = j-1)
  end do
  call system_clock (count = h2)
  time (3) = dble (h2 - h1)
  ! print *, "x"
  ! print *,  real (b)
  !
  print *, "verification; r = A x - b "
  call system_clock (count = h1)
  r = matmul (a0,b) - b0
  s = sqrt (sum (r * r, dim = 1))
  call system_clock (count = h2)
  time (4) = dble (h2 - h1)
  !
  !pasa a segundos
  call system_clock (count_rate = h3)  ! cte para pasar a seg
  if (h3 < 1) h3 = 1                   ! control
  time = time / dble (h3)              ! pasa de ciclos a seg
  !
  !estadistica  
  dt = sum (time (1:3))
  ops = (2.0/3.0) * dble (n) ** 3 + m * (2.0 * dble (n) ** 2)
  mflops = ops / (1.0e6 * dt)
  !
  print *, "l2-norm of the residual ; norm (r,2) = ", maxval (s)
  print 110, dt, mflops
  !
  open (1, file = "lu_sol.tim", status = "unknown")
  do 
    read (1,*,end=10) j
  end do
  10 backspace (1)
  write (1,130) p, n, time (1:3), mflops
  close (1, status = "keep")
  !
  !dloca en orden
  ier = 0
  deallocate (s , stat = ier (1) )
  deallocate (r , stat = ier (2) )
  deallocate (b0, stat = ier (3) )
  deallocate (b , stat = ier (4) )
  deallocate (a0, stat = ier (5) )
  deallocate (a , stat = ier (6) )
  if ( any (ier .ne.0) ) stop "allocation failed" 
  !
  print *
  print *, "fine ... "
  !
  errata = 0
  100 if (errata .ne. 0) stop "error: file not found"
  !
  110 format (" elapsed time = ",e24.16," s (",f8.2," mflops)")
  130 format (1x, i3, 1x, i8, 3 (1x,e16.8), 1x, f12.3 )
  contains
  !
  subroutine walltime (time)
    implicit none
    real    (idp), intent (out) :: time
    integer (iin) :: ticks, trate
    call system_clock (ticks, trate)
    time = dble (ticks) / dble (trate)
  end subroutine
end program
