program lusvecpivo
  implicit none
  ! parametros de precision usuales
  integer, parameter :: idp = kind (1.0d0)
  integer, parameter :: isp = kind (1.0e0)
  integer, parameter :: iin = kind (1)
  !
  real    (idp), parameter :: TINY = 1e-20
  real    (idp), parameter :: tol  = 1e-9
  real    (idp), dimension (:,:), allocatable :: a, a0
  real    (idp), dimension   (:), allocatable :: b, b0, d, r
  integer (iin), dimension   (:), allocatable :: indx
  integer (iin) :: i, j, k, l, m, n, p, kmax
  !hpf$ distribute a (*,block)
  !hpf$ align (i,j) with a (i,j) :: a0
  !hpf$ align (i)   with a (i,*) :: b0, b, d, r
  !hpf$ align (i)   with a (i,*) :: indx
  real    (idp) :: s, prima
  !
  real    (idp) :: dt, ops, mflops
  real    (idp), dimension (4) :: time
  integer (iin), dimension (7) :: ier
  integer (iin) :: h1, h2, h3
  integer (iin) :: errors, errata
  !
  integer (iin) :: n_unknowns, n_loads
  namelist /lu_basic_data/ n_unknowns, n_loads
  !
  ! lee del disco
  errors = 0
  errata = 1
  open  (1, file = "lu_basic.dat", status = "old", err = 100)
  read  (1,nml = lu_basic_data)   ! notar que los dos namelist estan en el mismo
  write (*,nml = lu_basic_data)  
  close (1, status = "keep")
  !
  n = n_unknowns
  m = n_loads
  p = number_of_processors ()
  !
  print *
  print *, "matrix order         ; n = ",n
  print *, "number of processors ; p = ",p
  !
  !aloca en orden 
  ier = 0
  allocate (a    (1:n,1:n), stat = ier (1) )
  allocate (a0   (1:n,1:n), stat = ier (2) )
  allocate (b    (1:n)    , stat = ier (3) )
  allocate (b0   (1:n)    , stat = ier (4) )
  allocate (r    (1:n)    , stat = ier (5) )
  allocate (d    (1:n)    , stat = ier (6) )
  allocate (indx (1:n)    , stat = ier (7) )
  if ( any (ier .ne.0) ) stop "allocation failed" 
  print *, "arrays are allocated ... "
  !   1.0  1.0  0.0  0.0  0.0
  !   1.0  0.0  2.0  0.0  0.0
  !   1.0  0.0  0.0  3.0  0.0
  !   1.0  0.0  0.0  0.0  4.0
  !   1.0  0.0  0.0  0.0  0.0
  a = 0.0
  forall (i=1:n-1) a (i,i+1) = dble (i)
  a (:,1) = 1.0
  forall (i=1:n) b (i) = 1.0
  a0 = a
  b0 = b
! print *, "b"
! print *,  real (b)
! print *, "A"
! do  i = 1, n  
!   print *, real (a0 (i,:))
! end do
  !
  !inicia tiempos  
  time = 0.0
  !
  !para medir tiempos:
  !call cpu_time (t1)      ! nativa en F95 equivale a t1 = second ()
  !call walltime (t1)      ! de usuario
  !call system_clock (h1)  ! a veces es mejor
  !
  print *, "LU-factorization; A = LU ... "
  call system_clock (count = h1)
  !
  !scaling
  indx = 0 
  d = maxval (abs (a), dim = 2) ! loop on rows 
  if (any (d == 0.0) ) stop" singular matrix ... "  
  d = 1.0 / d
  !
  do  k = 1, n
    !partial pivoting
    kmax = (k - 1) + imaxloc (d (k:n) * abs (a(k:n,k)))
    if (kmax .ne. k) then
      call swap (a (kmax,:), a (k,:))
      d (kmax) = d (k)
    end if
    indx (k) = kmax
    if  (a (k,k) == 0.0) a (k,k) = TINY
    !reduction with outer product
    a (k+1:n,k:k)   = a (k+1:n,k:k) / a (k,k)
    a (k+1:n,k+1:n) = a (k+1:n,k+1:n) -                &
                      spread (a (k+1:n,k), 2, n-k) *   &
                      spread (a (k,k+1:n), 1, n-k)
  end do
  call system_clock (count = h2)
  time (1) = dble (h2 - h1)
  !
  print *, "forward substitution; solves  L y = b ... "
  call system_clock (count = h1)
  k = 0
  do  i = 1, n
     l     = indx (i)
     prima = b (l)
     b (l) = b (i)
     if     (k .ne. 0) then
       s = dot_product (a (i,k:i-1), b (k:i-1))
     elseif (prima .ne. 0.0) then
       k = i
     end if
     b (i) = prima - s
  end do
  call system_clock (count = h2)
  time (2) = dble (h2 - h1)
  ! print *, "y"
  ! print *,  real (b)
  !
  print *, "backward substitution; solves  U x = y ... "
  call system_clock (count = h1)
  do  i = n,1,-1
    s = dot_product (a (i,i+1:n), b (i+1:n))
    b (i) = (b (i) - s) / a (i,i)
  end do
  call system_clock (count = h2)
  time (3) = dble (h2 - h1)
  ! print *, "x"
  ! print *,  real (b)
  !
  print *, "verification; r = A x - b "
  call system_clock (count = h1)
  r = matmul (a0,b) - b0
  s = sqrt (sum (r * r))
  call system_clock (count = h2)
  time (4) = dble (h2 - h1)
  !
  !pasa a segundos
  call system_clock (count_rate = h3)  ! cte para pasar a seg
  if (h3 < 1) h3 = 1                   ! control
  time = time / dble (h3)              ! pasa de ciclos a seg
  !
  !estadistica  
  dt = sum (time (1:3))
  ops = (2.0/3.0) * dble (n) ** 3 + 2.0 * dble (n) ** 2
  mflops = ops / (1.0e6 * dt)
  !
  print *, "l2-norm of the residual ; norm (r,2) = ", s
  print 110, dt, mflops
  !
  open (1, file = "lu_sol.tim", status = "unknown")
  do 
    read (1,*,end=10) j
  end do
  10 backspace (1)
  write (1,130) p, n, time (1:3), mflops
  close (1, status = "keep")
  !
  !dloca en orden
  ier = 0
  deallocate (indx, stat = ier (1) )
  deallocate (d   , stat = ier (2) )
  deallocate (r   , stat = ier (3) )
  deallocate (b0  , stat = ier (4) )
  deallocate (b   , stat = ier (5) )
  deallocate (a0  , stat = ier (6) )
  deallocate (a   , stat = ier (7) )
  if ( any (ier .ne.0) ) stop "allocation failed" 
  !
  print *
  print *, "fine ... "
  !
  errata = 0
  100 if (errata .ne. 0) stop "error: file not found"
  !
  110 format (" elapsed time = ",e24.16," s (",f8.2," mflops)")
  130 format (1x, i3, 1x, i8, 3 (1x,e16.8), 1x, f12.3 )
  contains
  !
  function imaxloc (a)
    real    (idp), dimension (:), intent (in) :: a
    integer (iin), dimension (1) :: nmax
    integer (iin) :: imaxloc
    nmax = maxloc (a(:))
    imaxloc = nmax (1)
  end function
  !
  subroutine swap (a,b)
    real (idp), dimension (:), intent (inout) :: a, b
    real (idp), dimension (size(a)) :: t
    t = a
    a = b
    b = t
  end subroutine
  !
  subroutine walltime (time)
    implicit none
    real    (idp), intent (out) :: time
    integer (iin) :: ticks, trate
    call system_clock (ticks, trate)
    time = dble (ticks) / dble (trate)
  end subroutine
end program
