!Fortran 90/95/2003 CourseFortran 90/95/2003 Course, May 2006
!
!Example program for timing matrix multiplications
!
!The region that is commented out calls the sgemm subroutine from the Basic 
!Linear Algebra Subprograms (BLAS)
!library. This will not work unless you are able to link with a BLAS library. Try 
!putting -llapack or -lblas at the end
!of your compilation command in linux.
!
! matrix size        n = 1000
!
! g95 -lblas -o matrixmul.exe matrixmul.f90
!   sgemm time           =   2.795575
!   matmul time          =   2.970548
!   triple jik-loop time =  40.814796
!   triple ijk-loop time =  36.44646
!   double loop time     =  23.242462
!
! g95 -lblas -O3 -o matrixmul.exe matrixmul.f90
!   sgemm time           =   2.5816069
!   matmul time          =   2.806573
!   triple jik-loop time =  10.371424
!   triple ijk-loop time =   9.842504
!   double loop time     =   9.790512
!
! g95 -lblas -O4 -o matrixmul.exe matrixmul.f90
!   sgemm time           =   2.50162
!   matmul time          =   2.776578
!   triple jik-loop time =  10.416416
!   triple ijk-loop time =   9.74052
!   double loop time     =   9.810511
!
! ifort -lblas -xP -O3 -ipo -no-prec-div -o matrixmul.exe matrixmul.f90
!   sgemm time           =    2.502620    
!   matmul time          =    1.016845    
!   triple jik-loop time =    1.010846    
!   triple ijk-loop time =    9.622538    
!   double loop time     =    9.663531    
!
! ifort -lblas -o matrixmul.exe matrixmul.f90
!   sgemm time           =    2.500620    
!   matmul time          =    2.654596    
!   triple jik-loop time =    9.686528    
!   triple ijk-loop time =    9.451562    
!   double loop time     =    9.703526   
!
! ifort -lblas -fast -o matrixmul.exe matrixmul.f90  # CRASH
! ifort -lblas -O3 -o matrixmul.exe matrixmul.f90    # very BAD
! ifort -lblas -O2 -o matrixmul.exe matrixmul.f90    # near without O? 
!
program matrixmult
  implicit none
  integer, parameter :: n = 1000
  real, dimension (n,n) :: a, b, c
  real    :: alpha, beta
  real    :: start, end
  integer :: i, j, k
  !
  print *
  print *, "timing matrix multiplications ... "
  print *
  print *, "matrix size        n = ", n
  !
  a = 1.0
  b = 1.0
  !
  ! Only uncomment this if you have an external subroutine
  ! called sgemm to link with. Try -llapack on linux.
  ! 
  alpha = 1.0
  beta  = 0.0
  !
  call cpu_time (start)
  call sgemm ('N','N',n, n, n, alpha, a, n, b, n, beta, c, n)
  call cpu_time (end)
  print *, "sgemm time           = ",end-start
  !  
  call cpu_time (start)
  c = matmul (a,b)
  call cpu_time (end)
  print *, "matmul time          = ",end-start
  !
  call cpu_time (start)
  do  j = 1, n
     do  i = 1, n
        c (i,j) = 0.0
        do  k = 1, n
           c (i,j) = c (i,j) + a (i,k) * b (k,j)
        enddo
     enddo
  enddo
  call cpu_time (end)
  print *, "triple jik-loop time = ",end-start
  !
  call cpu_time (start)
  do  i = 1, n
     do  j = 1, n
        c (i,j) = 0.0
        do  k = 1, n
           c (i,j) = c (i,j) + a (i,k) * b (k,j)
        enddo
     enddo
  enddo
  call cpu_time (end)
  print *, "triple ijk-loop time = ",end-start
  !
  call cpu_time (start)
  do  j = 1, n
     do  i = 1, n
        c (i,j) = dot_product (a(i,:),b(:,j))
     enddo
  enddo
  call cpu_time (end)
  print *, "double loop time     = ",end-start
  print *
  !
end program

