module m_realoca
implicit none
public
contains
  function realoca_i1 (p, n1)               ! reallocate
    integer, dimension (:), pointer :: p, realoca_i1
    integer, intent (in) :: n1
    integer :: n1old, ier
    allocate (realoca_i1 (1:n1), stat = ier)
    if (ier .ne. 0) stop "error: realoca_i1"
    if (.not. associated (p) ) return
    n1old = min (size (p), n1)
    realoca_i1 (1:n1old)    = p (1:n1old)
    realoca_i1 (n1old+1:n1) = 0
    deallocate (p, stat = ier)
    if (ier .ne. 0) stop "error: dloca p"
  end function
  function realoca_i2 (p, n1, n2)             ! reallocate
    integer, dimension (:,:), pointer :: p, realoca_i2
    integer, intent (in) :: n1, n2
    integer :: n1old, n2old, ier
    allocate (realoca_i2 (1:n1,1:n2), stat = ier)
    if (ier .ne. 0) stop "error: realoca_i2"
    if (.not. associated (p) ) return
    n1old = min (size (p,1), n1)
    n2old = min (size (p,2), n2)
    realoca_i2 (1:n1old,1:n2old) = p (1:n1old,1:n2old)
    realoca_i2 (n1old+1:n1,n2old+1:n2) = 0
    deallocate (p, stat = ier)
    if (ier .ne. 0) stop "error: dloca p"
  end function
end module
!
program realoca_test
  use m_realoca
  implicit none
  integer, dimension (:)  , pointer :: p => null() 
  integer, dimension (:,:), pointer :: q => null() 
  integer :: i, n1, nel = 4
  !
  allocate (p(1:nel))
  p (1) = 7
  !
  print *
  print *, "alocado inicial = ", size (p), " elementos"
  print *, "p (1) = ", p (1)
  !
  p => realoca_i1 (p, 6)    ! note pointer assignment
  print *
  print *, "alocado actual  = ", size (p), " elementos"
  print *, "p (1) = ", p (1)
  !
  p => realoca_i1 (p, 8)    ! note pointer assignment
  print *
  print *, "alocado actual  = ", size (p), " elementos"
  print *, "p (1) = ", p (1)
  !
  p => realoca_i1 (p, 3)    ! note pointer assignment
  print *
  print *, "alocado actual  = ", size (p), " elementos"
  print *, "p (1) = ", p (1)
  !
  !
  allocate (q(1:nel,1:nel))
  q (1,1) = 5
  q (1,2) = 4
  !
  print *
  print *, "alocado inicial = ", size (q,1), size (q,2), " elementos"
  n1 = size (q,1)
  do i = 1, n1
  print *, "q = ", q (i,:)
  end do
  !
  q => realoca_i2 (q, 6,6)    ! note pointer assignment
  print *
  print *, "alocado actual  = ", size (q,1), size (q,2), " elementos"
  n1 = size (q,1)
  do i = 1, n1
  print *, "q = ", q (i,:)
  end do
  !
  q => realoca_i2 (q, 8,8)    ! note pointer assignment
  print *
  print *, "alocado actual  = ", size (q,1), size (q,2), " elementos"
  n1 = size (q,1)
  do i = 1, n1
  print *, "q = ", q (i,:)
  end do
  !
  q => realoca_i2 (q, 3,3)    ! note pointer assignment
  print *
  print *, "alocado actual  = ", size (q,1), size (q,2), " elementos"
  n1 = size (q,1)
  do i = 1, n1
  print *, "q = ", q (i,:)
  end do
  !
  print *
end program

