Fortran 90/95/2003 CourseFortran 90/95/2003 Course, May 2006

Example program for one-dimensional SOR (Successive Over-Relaxation) 
program simpleSOR
  implicit none
  integer, parameter     :: nx=100
  real,    parameter     :: omega=1.9
  real,    dimension(nx) :: u,rho
  real                   :: Rmax,x,dx
  integer                :: i,its

  dx=1.0/real(nx-1)

  do i=1,nx
     x=real(i-1)*dx
     rho(i)=x*(1.0-x)
  enddo

  ! initial and boundary conditions

  u=0.0

  ! Output the maximum residual after every 10 iterations
  ! to the screen, and to a text file called fort.90

  do its=1,4000
     call relax(u,rho,Rmax,omega)
     if (mod(its,10).eq.0) then
        print *, its, Rmax
        write (90,*) its,Rmax
     endif
  enddo

  ! Following outputs the solution u, rho, and the approximate
  ! second derivative of u into a text file called fort.40

  do i=2,nx-1
     write (40,*) (i-1)*dx,u(i),rho(i),(u(i+1)-2.0*u(i)+u(i-1))/dx**2
  enddo

contains

  subroutine relax(u,rho,Rmax,omega)
    implicit none

    real, dimension(:), intent(inout) :: u
    real, dimension(:), intent(in)    :: rho
    real, intent(in)                  :: omega
    real, intent(out)                 :: Rmax

    real, dimension(size(u,1))        :: R ! automatic array of same shape as u

    real    :: dx
    integer :: nx,i

    nx = size(u,1)
    dx=1.0/real(nx-1)

    do i=2,nx-1
       u(i) = u(i) + (omega/2.0)*( ( u(i+1)+u(i-1)-2.0*u(i) ) - rho(i)*dx**2 )
    enddo

    do i=2,nx-1
       R(i) = ( u(i+1)+u(i-1)-2.0*u(i) )/dx**2-rho(i)
    enddo

    Rmax = maxval(abs(R(2:nx-1))) 

!   One may delete the above four statements, and the declaration of R as an 
automatic array,
!   and calculate Rmax directly in a single line using array sections

!  Rmax = maxval(abs( ( u(3:nx)+u(1:nx-2)-2.0*u(2:nx-1) )/dx**2 - rho(2:nx-1) ))

  end subroutine relax

end program simpleSOR

