! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! compilation with ADAPTOR and running with MPI:
!  adaptor -hpf -dm -free -o gauss1.exe gauss1.f90
!  mpirun -machinefile machi.dat -np 8 gauss1.exe
!  mpirun -machinefile machi.dat -nolocal -np 8 gauss1.exe
!
! running details:
!  si n = 10 000 entonces m = n^2, ram = m * 8 /1e6 = 800 [Mbytes]
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program gauss1
  implicit none
  integer, parameter :: n = 1000
  integer :: i, j, k, j1
  double precision :: a (n,n), b (n), x (n)
  !hpf$ distribute   (block) :: b
  !hpf$ distribute (*,block) :: a
  double precision :: q, diff
  double precision :: b1 (n), b0 (n), a0 (n,n)
  double precision :: ai (n)
  double precision :: bi
  integer :: tickstart, tickstop, tickrate
  integer :: np
  real :: time, mflops
  !
  !carteles
  write (*,*)
  write (*,*)" gauss solution of a SEL, with static memory"
  write (*,*)
  write (*,*)" number of unknowns ; n = ", n
  !genera un SEL regular
  !adp$ profile init begin
  b = [ (i, i=1,n) ]
  x = 1.0d0
  !hpf$ independent
  do  i = 1, n 
    !hpf$   independent
    do j = 1, n 
      if (i == j) then
          a (j,i) = 3.0d0
       else if (i == j-4 .or. j+4 == i)then
          a (j,i) = 18.0d0
         else
          a (j,i) = 0.0d0
       end if
     end do ! j
   end do ! i
  !adp$ end profile
  b0 = b
  a0 = a
  !
  !empieza la zona de resolucion del SEL
  call system_clock (tickstart, tickrate)
  !
  !triangulariza
  !adp$ profile gauss
  !hpf$ parallel
  do  i = 1, (n - 1)
    !hpf$ new (bi, ai) begin
    bi = b (i)
    ai (i:n) = a (i:n,i)
    q = 1.0d0 / ai (i)
    !hpf$ independent, on home (a(:,j))
    do  j = (i + 1), n
      b (j) = b (j) - bi * (ai (j) * q)
      do  k = (i + 1), n
        a (k,j) = a (k,j) - (ai (k) * q) * a (i,j)
      end do ! k
    end do ! j
    !hpf$ end new
  end do ! i
  !
  !retrosustitucion
  !adp$ profile backsub begin
  !hpf$ independent
  do  i = 1, n
    x (i) = b (i)
  end do
  x (n) = x (n) / a (n,n)
  do  i = (n - 1), 1, -1
    !hpf$ independent
    do  j = 1, i
      x (j) = x (j) - x (i + 1) * a(j, i+1)
    end do
    x (i) = x (i) / a (i,i)
  end do
  !adp$ end profile
  call system_clock (tickstop)
  !
  !verifica la solucion obtenida
  !adp$ profile verification begin
  b1 = 0.0d0
  do  j = 1, n
    do  i = 1, n
      b1 (i) = b1 (i) +  a0 (i,j) * x (j)
    end do
  end do
  diff = maxval (abs (b0 - b1))
  !adp$ end profile
  !
  !estadistica
  np = number_of_processors ()
  time = float (tickstop - tickstart) / float(tickrate)
  mflops = real (n) ** 3 / (1000000.0 * time)
  !
  ! carteles
  write (*,*)
  write (*,*)' max diff = ', diff 
  write (*,*)
  !
  write (*,*)' time     = ', time, ' seconds'
  write (*,*)' mflops   = ', mflops, ' on ', np, ' processors'
  write (*,*)' mflops   = ', mflops/np, ' for one processor'
  write (*,*)
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
