! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!   Gauss con (A, B), donde
!   A ; matriz de coeficientes
!   B ; matriz de cargas
! -----------------------------------------------------------------
module  m_gaus3       ! con Descriptive Mapping
  use   m_ctes
  use   m_temps
  use   m_tools
  implicit none
  private             ! por omision todo es privado
  public  gauss_dpi   ! gauss_zpi
  character (11), parameter :: c = "m_gauss3 > "
  contains
  !
  !----------------------------------------------------------------
  ! solucion por gauss Doble precision con PIvoteo
  subroutine gauss_dpi (a)
    integer (iin) :: i, j, k, p, q
    real (idp), dimension (:,:), intent (inout) :: a
    !hpf$ distribute * (*,block) :: a
    real (idp), dimension (:), allocatable :: t
    !hpf$ align (j) with a (*,j) :: t
    real    (idp) :: s
    integer (iin), dimension (1) :: ipivot
    integer (iin) :: n, m
    !
    !valia tamanios
    q = size (a,1)
    n = size (a,2)
    if (q < n) call errata (q,"q < n")
    m = q - n
    !
    !aloca
    ier = 0
    allocate (t (n), stat = ier (1) )
    if (any (ier .ne. 0)) call errata (ier(1), "gauss: aloca")
    !
    !inicia cronometro;  
    time = cero
    call system_clock (count = h1)   ! t1 = second ()
    !
    triangulariza: do  k = 1, (n - 1)
      !
      !select partial pivot in the k-column, so there is not loads swap
      ipivot = maxloc ( abs ( a (k:n,k) ) ) + (k - 1)
      p = ipivot (1)
      !swap between p-row and k-row of A
      if  (p .ne. k) then
        !hpf$  independent
        do  j = 1, n
          t   (j) = a (p,j)
          a (p,j) = a (k,j)
          a (k,j) = t (j)
        end do
      end if
      !
      !reduction
      !hpf$ new (t,s) begin
        t (k:n) = a (k:n,k)
        s = 1.0 / t (k)
        !hpf$ independent, new (i,p), on home ( a (:,j) )
        do  j = (k + 1), n
          do  i = (k + 1), n
            a (i,j) = a (i,j) - t (i) * a (k,j) * s
          end do ! i
          do  p = (n + 1), q
            a (p,j) = a (p,j) - t (j) * a (p,k) * s
          end do ! p
        end do ! j
      !hpf$ end new
    end do triangulariza
    !
    !delta de tiempo por triangularizar
    call system_clock (count = h2)     ! t2 = second ()
    time (1) = h2 - h1                 ! time (1) = t2 - t1
    !
    !retro-substitucion
    !adp$ profile retro begin
    pp: do  p = (n + 1), q
      a (p,n) = a (p,n) / a (n,n) 
      do  i = (n - 1), 1, -1
        !hpf$ independent
        do  j = 1, i
          a (p,j) = a (p,j) - a (p,i+1) * a (j,i+1)
        end do
        a (p,i) = a (p,i) / a (i,i)
      end do
    end do pp
    !adp$ end profile
    !
    !delta de tiempo por retrosustitucion
    call system_clock (count = h3)     ! t2 = second ()
    time (2) = h3 - h2                 ! time (2) = t3 - t2
    !
    !estadistica
    call timer (n,3,"gauss")
    !
    !dloca
    ier = 0
    deallocate (t, stat = ier (1) )
    if (any(ier.ne.0)) call errata (ier(1), "gauss: dloca")
  end subroutine
  !
  ! ---------------------------------------------------------------
  ! estadistica adaptada del Linpack Benchmark
  ! ---------------------------------------------------------------
  subroutine  mflops (n, m, time)
    real    (idp), parameter :: cray = 0.056d0
    real    (idp), parameter :: dos  = 2.0d0
    real    (idp), parameter :: tres = 3.0d0
    real    (idp), parameter :: mega = 1.0d6
    integer (iin),                intent (in)   :: n
    integer (iin),                intent (in)   :: m
    real    (idp), dimension (:), intent (inout):: time
    real    (idp) :: total, ops
    integer (iin) :: p=1, ciclos
    !
    call system_clock (count_rate = h4)  ! para pasar a segundos
    time (3) = time (1) + time (2)       ! ciclos total
    ciclos = floor (time (3))
    !
    time  = time / h4                    ! pasa de ciclos a seg
    total = time (3)                     ! total en seg
    !
    ops = (dos / tres) * dfloat (n)**3 + dos * dfloat (n)**2
    p = size (time)
    if (p < 7) call errata ("error (mflops): p < 7")
    time (3) = total                 ! tiempo [seg]
    time (4) = ops / (mega * total)  ! Mflops
    time (5) = dos / time (4)        ! unit
    time (6) = total / cray          ! ratio
    !
    ! solo en HPF
    p = number_of_processors ()
    !
    write (*,*)
    write (*,*) "nro de procesadores ;      p  = ", p
    write (*,*) "tamanio de la matriz;      n  = ", n
    write (*,*) "nro de cargas       ;      m  = ", m
    write (*,*) "nro de ciclos       ;  cicles = ", ciclos
    write (*,*) "tiempo total        ; seconds = ", real (time(3))
    write (*,*) "                       MFlops = ", real (time(4))
    write (*,*)
    write (*,*) "numero de operaciones ; o  = ", real (ops)
    write (*,*) "tpo de factoriz. [seg]; t1 = ", real (time(1))
    write (*,*) "tpo de restrosu. [seg]; t2 = ", real (time(2))
    write (*,*) "unit                       = ", real (time(5))
    write (*,*) "ratio                      = ", real (time(6))
    write (*,*)
    write (*,*) " p  A_size  B_loads  cicles     seconds      Mflops"
    !
    ! resumen a disco
    open (1, file = 'gauss.tpo', &
             status = 'unknown', &
             position = 'append')
    write (*,110)  p, n, m, ciclos, total, time (4)
    write (1,110)  p, n, m, ciclos, total, time (4)
    close (1, status = 'keep')
    write (*,*)
    !
    ! formatos
    100 format (a)
    110 format (1x, i2, i8, 1x, i8, 1x, i7, 1x, e11.4, 1x, e11.4)
  end subroutine
end module
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
