! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! GNU:   
! g95 -w -c constan_m.f90 --free-form
! g95 -w -c globals_m.f90 --free-form
! g95 -w -c herrami_m.f90 --free-form
! g95 -w -c gaussol_m.f90 --free-form
! g95 -w -c gauss_sol.f90 --free-form
! g95 -w -o gauss_sol.exe constan_m.o globals_m.o herrami_m.o 
!   gaussol_m.o gauss_sol.o
! gauss_sol.exe
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! PGI, F90:   
! pgf90 -V -Minfo -fast -c constan_m.f90
! pgf90 -V -Minfo -fast -c globals_m.f90
! pgf90 -V -Minfo -fast -c herrami_m.f90
! pgf90 -V -Minfo -fast -c gaussol_m.f90
! pgf90 -V -Minfo -fast -c gauss_sol.f90
! pgf90 -V -fast -o gauss_sol.exe constan_m.o globals_m.o herrami_m.o 
!   gaussol_m.o gauss_sol.o
! gauss.exe
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! PGI (HPF + MPI):   
! pghpf -V -Minfo -Mmpi -fast -c constan_m.f90
! pghpf -V -Minfo -Mmpi -fast -c globals_m.f90
! pghpf -V -Minfo -Mmpi -fast -c herrami_m.f90
! pghpf -V -Minfo -Mmpi -fast -c gaussol_m.f90
! pghpf -V -Minfo -Mmpi -fast -c gauss_sol.f90
! pghpf -V -Mmpi -fast -o gauss.exe constan_m.o globals_m.o herrami_m.o 
!  gaussol_m.o gauss_sol.o
! mpirun -machinefile machi.dat -np 1 gauss.exe
! mpirun -machinefile machi.dat -nolocal -np 1 gauss.exe
! mpirun -machinefile machi.dat -nolocal -np 2 gauss.exe
! mpirun -machinefile machi.dat -nolocal -np 4 gauss.exe
! bw "ps -aef | grep "gauss.exe""
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! PGI (HPF):
! pghpf -V -Minfo -fast -c constan_m.f90
! pghpf -V -Minfo -fast -c globals_m.f90
! pghpf -V -Minfo -fast -c herrami_m.f90
! pghpf -V -Minfo -fast -c gaussol_m.f90
! pghpf -V -Minfo -fast -c gauss_sol.f90
! pghpf -V -fast -o gauss.exe constan_m.o globals_m.o herrami_m.o 
!  gaussol_m.o gauss_sol.o
! gauss.exe -pghpf -machinefile machi.dat -np 4
! bw "ps -aef | grep "gauss.exe"
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! ADAPTOR for Distributed Memory Machines using MPI:
! adaptor -hpf -dm -free -c constan_m.f90
! adaptor -hpf -dm -free -c globals_m.f90
! adaptor -hpf -dm -free -c herrami_m.f90
! adaptor -hpf -dm -free -c gaussol_m.f90
! adaptor -hpf -dm -free -c gauss_sol.f90
! adaptor -hpf -dm -o gauss.exe constan_m.o globals_m.o herrami_m.o 
!  gaussol_m.o gauss_sol.o
! mpirun -machinefile machi.dat -np 1 gauss.exe
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Nota 1: en un cluster heterogeneo (e.g. Pentium III/IV) y con
!         PGI, compilar para el procesador "mas viejo" para que 
!         asi el ejecutable paralelo pueda funcionar con 
!         hardware mixto.
! Nota 2: verificar antes la coherencia de "machi.dat" con la
!         configuracion instantanea del cluster y del uso o no
!         de "nolocal" 
! Nota 3: en caso de usar el sistema de colas PBS la misma
!         observacion anterior: revisar la coherencia del
!         "script" usado !!
! Nota 4: para no tener problemas con la licencia, la fecha y hora
!         debe ser, mas o menos, la misma en todos los nodos
! -----------------------------------------------------------------
program gauss_sol
  use constan_m
  use globals_m
  use herrami_m
  use gaussol_m, only: gauss
  implicit none
  integer (iin), parameter :: nax = 500000
  real    (idp), dimension (:,:), allocatable :: a, b
  real    (idp) :: a_ij, b_i
  character (11), parameter :: f = "gauss_sol > "
  !
  !hpf$ template t (nax,nax)
  !hpf$ distribute t (*,cyclic)
  !hpf$ align  a (:,:) with t (1:nax,1:nax)
  !hpf$ align  b (:,:) with t (1:nax,1:nax)
  !
  integer (iin) :: n, m, i, j, l
  !
  ! lee en disco un tamanio
  call predefine_sea (n, m)
  !
  ! aloca sistema
  ier = 0
  allocate ( a (n,n), stat = ier (1) )
  allocate ( b (m,n), stat = ier (2) )
  if ( any (ier .ne. 0) ) call errata (ier, "aloca" )
  !
  write (*,*)
  write (*,*)"lee matriz A ... "
  open (1,file = "a.dat", status = "old")
  l = 0
  do 
    read (1,*,end=50) i, j, a_ij
    if (i < 1 .or. i > n) stop "i < 1 .or. i > n"
    if (j < 1 .or. j > n) stop "j < 1 .or. j > n"
    a (i,j) = a_ij
    l = l + 1
    write (*,*)"leidos ; l = ", l
  end do
  50 close (1,status = "keep")
  !
  write (*,*)
  write (*,*)"lee vector b ... "
  open (2,file = "b.dat", status = "old")
  i = 0
  do 
    read (2,*,end=60) b_i
    i = i + 1
    if (i > n) stop "i > n"
    b (1,i) = b_i
    write (*,*)"faltan ; l = ", n - i
  end do
  60 close (2,status = "keep")
  write (*,*)
  write (*,*)"Gauss ..."
  !
  ! resuelve por Gauss con pivotaje parcial
  call  gauss (a, b)
  !
  write (*,*)
  write (*,*)"imprime solucion x ... "
  open (3,file = "x.sol", status = "unknown")
  do  i = 1, n
    write (3,400)  b (1,i)
  end do
  close (3,status = "keep")
  !
  ! dloca 
  ier = 0
  deallocate ( b , stat = ier (4) )
  deallocate ( a , stat = ier (5) )
  if (any (ier .ne. 0))  call errata (ier, 'dloca' )
  !
  write (*,*)
  write (*,*)"finalizado ... "
  write (*,*)
  !
  ! formatos
  100 format (a)
  110 format (1x, i4, 1x, i20, 1x, e24.12)
  400 format (1x,e24.16) 
 !
 contains
 !
 ! predefine SEAL
 subroutine predefine_sea (n, m)
  integer (iin), intent (out) :: n
  integer (iin), intent (out) :: m
  integer (iin) :: nro_de_incognitas
  integer (iin) :: nro_de_cargas
  integer (iin) :: nro_max_iteraciones
  real    (idp) :: tolerancia_residuo
  integer (iin) :: nit
  real    (idp) :: rum
  logical :: alto
  namelist  /seal_data/ nro_de_incognitas,   &
                        nro_de_cargas,       &
                        nro_max_iteraciones, & 
                        tolerancia_residuo
  write (*,100) f
  write (*,100) f // "Sistema de Ecuaciones Algebraicas (SEA) "
  write (*,100) f // "Resolucion directa por Gauss "
  write (*,100) f
  write (*,100) f // "lee parametros de la corrida en el disco ... "
  alto = .true.
  open  (1, file = "seal_siza.dat", status = "old", err = 300)
  read  (1, nml = seal_data, err = 310)
  close (1, status = 'keep')
  !
  n   = nro_de_incognitas
  m   = nro_de_cargas
  nit = nro_max_iteraciones
  rum = tolerancia_residuo
  !
  write (*,100) f
  write (*,200) f // "nro de incognitas ; n = ", n
  write (*,200) f // "nro de cargas     ; m = ", m
  !
  ! control
  if (n > nax) call errata (nax, "nax insuficiente")
  if (m > nax) call errata (nax, "nax insuficiente")
  !
  ! salida
  alto = .false.
  300 if (alto)  call  errata ("apertura *.dat")
  310 if (alto)  call  errata ("lectura  *.dat")
  100 format (a)
  200 format (a,i4)
 end subroutine
end program
! *****************************************************************
