! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Compilation with INTEL:
!  ifort -w -c m_ctes.f90
!  ifort -w -c m_globals.f90
!  ifort -w -c m_tools.f90
!  ifort -w -c m_jordan.f90
!  ifort -w -c gauss_rr.f90
!  ifort -w -o gauss.exe2 *.o
!
! Compilation with GNU:
!  g95 -w -o gauss.exe2 gauss_rr.f90
!
! Compilation with (ADAPTOR + MPICH):
!  adaptor -hpf -dm -free -c m_ctes.f90
!  adaptor -hpf -dm -free -c m_globals.f90
!  adaptor -hpf -dm -free -c m_tools.f90
!  adaptor -hpf -dm -free -c m_jordan.f90
!  adaptor -hpf -dm -free -c gauss_rr.f90
!  adaptor -hpf -dm -free -o gauss.exe2 *.o
!
! Running with MPICH:
!  mpdboot -n 10 -f ~/mpd.hosts
!  mpiexec -machinefile ~/machi.dat -np 2 gauss.exe2
!  mpiexec -machinefile ~/machi.dat -1 -np 4 gauss.exe2
!
! The same running but with MPI (old):
!  mpirun -machinefile ~/machi.dat -np 2 gauss.exe2
!  mpirun -machinefile ~/machi.dat -nolocal -np 4 gauss.exe2
!
! Some running details:
!  si n = 8 000 entonces m = n^2, RAM = m * 8 /1e6 =  512 [Mbytes]
!  pero como aqui hay una copia de A se tiene ram = 1,024 [Gbytes]
!
! output example:
!  gauss 
!  number of unknowns ; n = ? 
!  max diff =  5.83895598538220E-10
!  time   =  576.0575  seconds
!  mflops =  374.9626  on  8  processors
!  mflops =  46.87032  for one processor
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program  gauss_rr
  use  m_ctes
  use  m_tools
  use  m_globals    
  use  m_jordan, only : jordan
  implicit none
  integer, parameter :: nax = 90000
  logical, parameter :: ideba = .false.
  !................................................................
  real (idp), dimension (4,4), parameter ::   &
    aa0 = reshape ( [ 11,  1, -21,  4,        &
                      13,  9, -12, 31,        &
                      -4, -5,   5,  7,        &
                       8, -3,  -1,  3 ],  [4, 4] )
  real (idp), dimension (2,4), parameter ::   & 
    bb0 = reshape ( [ -4,  9,   8,  1,        &
                      -8,  7,  17,  2], [2, 4] )
  ! ...............................................................
  real (idp), dimension (:,:), allocatable :: a0, a1, a2
  real (idp), dimension (:,:), allocatable :: b0, b1, b2
  real (idp), dimension (:,:), allocatable :: c, d, r
  ! ...............................................................
  !hpf$   processors, dimension (2,2) :: qq
  !hpf$   template    t (nax,nax)
  !hpf$   distribute  t (*,cyclic)  ! onto qq
  !................................................................
  !hpf$   align    a0 (*,:) with t (*,1:nax)
  !hpf$   align    a1 (*,:) with t (*,1:nax)
  !hpf$   align    a2 (*,:) with t (*,1:nax)
  !................................................................
  !hpf$   align    b0 (*,:) with t (*,1:nax)
  !hpf$   align    b1 (*,:) with t (*,1:nax)
  !hpf$   align    b2 (*,:) with t (*,1:nax)
  !................................................................
  !hpf$   align     c (*,:) with t (*,1:nax)
  !hpf$   align     d (*,:) with t (*,1:nax)
  !hpf$   align     r (*,:) with t (*,1:nax)
  !................................................................
  real    (idp) :: fx, fy, s
  integer (iin) :: i, j, k, p, n, m
  !----------------------------------------------------------------
  ! define un tamanio
  ! ---------------------------------------------------------------
! if  (ideba) then
!   n = size (bb0,1)
!   m = size (bb0,2)
! else
    n = 1000
    m =   10
! end if
  ! ---------------------------------------------------------------
  ! control
  !----------------------------------------------------------------
   if (n > nax) call errata (nax, "nax insuficiente")
   if (m > nax) call errata (nax, "nax insuficiente")
  ! ---------------------------------------------------------------
  ! aloca sistema
  ! ---------------------------------------------------------------
  ier = 0
  allocate ( a0 (n,n), stat = ier (1) )
  allocate ( a1 (n,n), stat = ier (2) )
  allocate ( a2 (n,n), stat = ier (3) )
  allocate ( b0 (m,n), stat = ier (4) )
  allocate ( b1 (m,n), stat = ier (5) )
  allocate ( b2 (m,n), stat = ier (6) )
  allocate (  c (n,n), stat = ier (7) )
  allocate (  d (m,n), stat = ier (8) )
  allocate (  r (m,n), stat = ier (9) )
  if ( any (ier .ne. 0) ) call errata (ier, "aloca" )
  ! ---------------------------------------------------------------
  ! define un SEAL A X = B donde A es la matriz del sistema,
  ! X, B matrices solucion y carga, respectivamente. 
  ! ---------------------------------------------------------------
  if  (ideba) then
    a0 = aa0
    b0 = bb0
  else
    do  i = 1, n
    call  random_number ( b0 (:,i) )
    call  random_number ( a0 (i,:) )
    end do ! i
    forall (i = 1 : n)  a0 (i,i) = a0 (i,i) + n
  end if
  ! ---------------------------------------------------------------
  ! Solucion por Gauss
  !----------------------------------------------------------------
  a1 = a0
  b1 = b0
! call  system_clock (count = h1)       ! t1 = second ()
! call  gauss (a1, b1)
! call  zyztem_clock (h1, 'cronos.cig') ! t2 = second ()
  ! ---------------------------------------------------------------
  ! Solucion por Gauss-Jordan puro serial estandard
  ! ---------------------------------------------------------------
  a1 = a0
  b1 = b0
! call  system_clock (count = h1)       ! t1 = second ()
! call  jordan (a0, b0)
! call  zyztem_clock (h1, 'cronos.cig') ! t2 = second ()
  ! ---------------------------------------------------------------
  ! Solucion por Gauss-Jordan puro recursivo estandard
  ! ---------------------------------------------------------------
  a1 = a0
  b1 = b0
  a2 = a0
  b2 = b0
  call  system_clock (count = h1)       ! t1 = second ()
!  call  jordan (a1, a2, b1, b2, 1, n)
  call  zyztem_clock (h1, 'cronos.cig') ! t2 = second ()
  ! ---------------------------------------------------------------
  ! Verifica matriz solucion X con R = B - A X
  ! ---------------------------------------------------------------
  do  j = 1, m
  r (j,:) = matmul ( a0, b1 (j,:) ) - b0 (j,:)
  s = maxval ( abs ( r (j,:) ) )
  write (*,*)' j max_abs_res = ', j, real (s)
  end do ! j
  ! ---------------------------------------------------------------
  ! dloca 
  !----------------------------------------------------------------
  ier = 0
  deallocate ( r , stat = ier (1) )
  deallocate ( c , stat = ier (2) )
  deallocate ( d , stat = ier (3) )
  deallocate ( b2, stat = ier (4) )
  deallocate ( b1, stat = ier (5) )
  deallocate ( b0, stat = ier (6) )
  deallocate ( a2, stat = ier (7) )
  deallocate ( a1, stat = ier (8) )
  deallocate ( a0, stat = ier (9) )
  if (any (ier .ne. 0))  call errata (ier, "dloca" )
  ! ---------------------------------------------------------------
  ! formatos
  ! ---------------------------------------------------------------
  100   format (a)
  110   format (1x, i4, 1x, i20, 1x, e24.12)
  ! ...............................................................
end program
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
