! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!   Gauss con (A, B), donde
!   A ; matriz de coeficientes
!   B ; matriz de cargas
! -----------------------------------------------------------------
module m_jordan
  use  m_ctes
  use  m_globals
  use  m_tools
  implicit none
  private                           ! por omision todo es privado
  !
  interface  jordan 
    module procedure jordan1_d, jordan2_d
  end interface
  public jordan
  !
  character (10), parameter :: c = "m_jordan> "
  contains
  !
  !----------------------------------------------------------------
  !   Triangularizacion Gaussiana con pivotaje: variable real
  subroutine gauss_d (m, n, a, b)
    integer (iin), intent (in) :: m, n
    real (idp), dimension (n,n), intent (inout) :: a
    real (idp), dimension (m,n), intent (inout) :: b
    real (idp), dimension   (:), allocatable    :: u, v
    real (idp) :: s
    !hpf$  inherit :: a, b
    integer (iin), dimension (1) :: ipivot
    integer (iin) :: i, j, k, p
    !
    ! aloca
    ier = 0
    allocate ( u (n), stat = ier (1) )
    allocate ( v (m), stat = ier (2) )
    if ( any (ier .ne. 0) ) call errata (ier, "gauss: aloca")
    !
    ! inicia cronometro;  
    time = cero
    call system_clock (count = h1)   ! t1 = second ()
    !
    ! lazo Gauss con pivotaje
    gauss: do  k = 1, (n - 1)
      ! selecciona pivote
      ipivot = maxloc ( abs (a(k:n,k)) ) + (k - 1)
      p = ipivot (1)
      ! permuta filas: a (p,:) <--> a (k,:)
      !                b (p)   <--> b (k)
      if  ( p .ne. k ) then
        !hpf$ independent
        forall (j = k : n)  
          u (j)   = a (p,j)
          a (p,j) = a (k,j)
          a (k,j) = u (j)
        end forall
        forall (j = 1 : m)  
          v (j)   = b (j,p)
          b (j,p) = b (j,k)
          b (j,k) = v (j)
        end forall
      end if
      ! triangulariza matriz
      !hpf$ independent
      forall (i = k + 1 : n)
        a (i,k) = a (i,k) / a (k,k)
        !hpf$  independent
        forall (j = k + 1 : n)
          a (i,j) = a (i,j) - a (i,k) * a (k,j)
        end forall
        forall (p = 1 : m)
          b (p,i) = b (p,i) - a (i,k) * b (p,k)
        end forall
      end forall
    end do gauss
    !
    ! delta de tiempo por triangularizar
    call system_clock (count = h2)     ! t2 = second ()
    time (1) = h2 - h1                 ! time (1) = t2 - t1
    !
    ! retro-substitucion
    pp: do  p = 1, m
      ii: do  i = n, 1, -1
        s = cero
        !hpf$ independent, reduction (s)
        jj : do  j = (i + 1), n
          s = s + a (i,j) * b (p,j)
        end do jj
        b (p,i) = ( b (p,i) - s ) / a (i,i)
      end do ii
    end do pp
    !
    ! delta de tiempo por retrosustitucion
    call system_clock (count = h3)     ! t2 = second ()
    time (2) = h3 - h2                 ! time (2) = t3 - t2
    !
    ! estima MFlops
!    call mflops (n, m, time)
    !
    ! dloca
    ier = 0
    deallocate ( u, stat = ier (1) )
    deallocate ( v, stat = ier (2) )
    if ( any(ier.ne.0) ) call errata (ier, "gauss: dloca")
  end subroutine
  !
  !----------------------------------------------------------------
  ! Solucion por Gauss-Jordan serial estandar
  subroutine  jordan1_d (a, b)
    real    (idp), dimension (:,:), intent (inout) :: a
    real    (idp), dimension (:,:), intent (inout) :: b
    !hpf$   inherit :: a, b
    !!hpf$   inherit, distribute (*,cyclic) :: a
    !!hpf$   align    b (:,:)  with  a (:,:)
    real    (idp), dimension (:), allocatable :: u, v
    integer (iin), dimension (1) :: imax
    real    (idp) :: smax
    integer (iin) :: n, m
    integer (iin) :: i, j, k, p
    logical       :: regular
    ! tamanios y constantes
    n = siza (size (a,1), size (b,2), "a, b" )
    m = size (b,1)
    ! aloca
    ier = 0
    allocate (u (n), stat = ier (1))
    allocate (v (m), stat = ier (2))
    if ( any (ier .ne. 0) ) call errata (ier, "jordan1: aloca")
    ! inicia lazo secuencial k: reduccion
    regular = .true.
    reduccion: do   k = 1, n
      !  eventual pivoteo de filas
      imax = maxloc ( abs ( a (k:n,k) ) )
      smax = maxval ( abs ( a (k:n,k) ) )
      if  (smax < tol) then 
        regular = .false.
        exit
      end if
      p = imax (1) + (k - 1)
      if  (p .ne. k) then
        !hpf$ independent, new (u)
        forall (j = 1 : n)  
          u (j)   = a (p,j)
          a (p,j) = a (k,j)
          a (k,j) = u (j)
        end forall
        !hpf$ independent, new (v)
        forall (j = 1 : m)  
          v (j)   = b (j,p)
          b (j,p) = b (j,k)
          b (j,k) = v (j)
        end forall
      end if
      ! triangulariza hacia abajo y hacia adelante
      !hpf$ independent
      forall (i = k + 1 : n)
        a (i,k) = a (i,k) / a (k,k)            ! pivote
        !hpf$ independent
        forall (j = k + 1 : n)
        a (i,j) = a (i,j) - a (i,k) * a (k,j)  ! reduce
        end forall
        !hpf$ independent
        forall (p = 1 : m)
          b (p,i) = b (p,i) - a (i,k) * b (p,k)
        end forall
      end forall
      ! triangulariza hacia arriba y hacia adelante
      !hpf$   independent
      forall (i = 1 : k - 1)
        a (i,k) = a (i,k) / a (k,k)            ! pivote
        !hpf$ independent
        forall (j = k + 1 : n)
          a (i,j) = a (i,j) - a (i,k) * a (k,j)  ! reduce
        end forall
        !hpf$ independent
        forall (p = 1 : m)
          b (p,i) = b (p,i) - a (i,k) * b (p,k)
        end forall
      end forall
    end do reduccion
    ! solucion
    if  (regular) then
      !hpf$  independent
      forall (i = 1:m, j = 1:n)
        b (i,j) = b (i,j) / a (j,j)
      end forall
    else
      write (*,*)" matriz SINGULAR: impone solucion nula "
      b = cero
    end if
    ! dloca
    ier = 0
    deallocate ( u, stat = ier (1) )
    deallocate ( v, stat = ier (2) )
    if ( any (ier .ne. 0) ) call errata (ier, "jordan1: dloca")
  end subroutine
  !
  !----------------------------------------------------------------
  ! Solucion por Gauss-Jordan recursivo estandar
  recursive subroutine jordan2_d (a1, a2, b1, b2, n1, n2)
    real    (idp), dimension (:,:), intent (inout) :: a1, a2
    real    (idp), dimension (:,:), intent (inout) :: b1, b2
    integer (iin)                 , intent (in)    :: n1, n2
    !!!hpf$  inherit, distribute (*,cyclic) :: a1
    !!hpf$  align    a2 (*,:)  with  a1 (*,:)
    !!hpf$  align    b1 (*,:)  with  a1 (*,:)
    !!hpf$  align    b2 (*,:)  with  a1 (*,:)
    !hpf$  inherit :: a1, a2, b1, b2
    real (idp), dimension (:), allocatable :: u, v
    integer (iin), dimension (1) :: imax
    real    (idp) :: t
    integer (iin) :: n, m, z
    integer (iin) :: i, j, k, p
    integer (iin) :: i1, i2, i3, i4        
    ! tamanios y constantes
    n = siza (size (a1,1), size (b1,2), "a, b" )
    m = size (b1,1)
    z = abs (n2 - n1) + 1
    !
    ! paso base de la recursion
    if (z .eq. 1) then
      do  j = 1, m
        b1 (j,n1) = b1 (j,n1) / a1 (n1,n1)
        b2 (j,n1) = b1 (j,n1)
      end do ! j
      return
    end if
    !
    ! paso inductivo de la recursion
    !
    ! aloca
    ier = 0
    allocate (u (n), stat = ier (1))
    allocate (v (m), stat = ier (2))
    if (any (ier .ne. 0)) call errata (ier, "jordan2: aloca")
    i1 = n1
    i2 = (z / 2) + (n1 - 1)
    i3 = i2 + 1
    i4 = n2
    !
    ! inicia lazo k: triangulariza hacia adelante
    hadelante: do   k = i1, i2
      ! eventual pivoteo
        imax = maxloc ( abs ( a1 (k:i4,k) ) )
        p = imax (1) + (k - 1)
        if  (p .ne. k) then
          !hpf$  independent, new (u)
          forall (j = i1 : i4)  
            u (j)    = a1 (p,j)
            a1 (p,j) = a1 (k,j)
            a1 (k,j) = u (j)
          end forall
          forall (j = 1 : m)  
            v (j)    = b1 (j,p)
            b1 (j,p) = b1 (j,k)
            b1 (j,k) = v (j)
          end forall
        end if
        ! triangulariza hacia adelante ppamente dicho
        !hpf$  independent
        forall (i =  k + 1 : i4)
          a1 (i,k) = a1 (i,k) / a1 (k,k)             ! pivote
          !hpf$  independent
          forall (j =  k + 1 : i4)
          a1 (i,j) = a1 (i,j) - a1 (i,k) * a1 (k,j)  ! reduce
          end forall
          forall (p = 1 : m)
          b1 (p,i) = b1 (p,i) - a1 (i,k) * b1 (p,k)
          end forall
        end forall
      end do hadelante
      !
      ! inicia lazo k: triangulariza hacia atras
      hatras:  do   k = i4, i3, -1
        ! eventual pivoteo
        imax = maxloc (abs (a2 (i1:k,k)))
        p = imax (1) + (i1 - 1)
        if  (p .ne. k) then
          !hpf$  independent, new (u)
          forall (j = i1 : i4)  
            u (j)    = a2 (p,j)
            a2 (p,j) = a2 (k,j)
            a2 (k,j) = u (j)
          end forall
          forall (j = 1 : m)  
            v (j)    = b2 (j,p)
            b2 (j,p) = b2 (j,k)
            b2 (j,k) = v (j)
          end forall
        end if
        !hpf$ independent
        forall (i = k - 1 : i1 : -1)
          a2 (i,k) = a2 (i,k) / a2 (k,k)             ! pivote
          !hpf$  independent
          forall (j =  k - 1 : i1 : -1)
            a2 (i,j) = a2 (i,j) - a2 (i,k) * a2 (k,j)  ! reduce
          end forall
          forall (p = 1 : m)
            b2 (p,i) = b2 (p,i) - a2 (i,k) * b2 (p,k)
          end forall
        end forall
      end do hatras
      ! permuta A1/A2 y B1/B2
      !hpf$ independent
      forall (i = i1 : i2)
        !hpf$ independent
        forall (j = i1 : i2)  a1 (i,j) = a2 (i,j)
        !hpf$ independent
        forall (j = 1 :   m)  b1 (j,i) = b2 (j,i)
      end forall
      !hpf$ independent
      forall (i = i3 : i4)
        !hpf$ independent
        forall (j = i3 : i4)  a2 (i,j) = a1 (i,j)
        !hpf$   independent
        forall (j =  1 :  m)  b2 (j,i) = b1 (j,i)
      end forall
      ! dloca
      ier = 0
      deallocate (u, stat = ier (1))
      deallocate (v, stat = ier (2))
      if (any (ier .ne. 0)) call errata (ier, "jordan2: dloca")
      !
      ! llamadas recursivas
      call jordan2_d (a1, a2, b1, b2, i1, i2)
      call jordan2_d (a1, a2, b1, b2, i3, i4)
   end subroutine
end module
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
