! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!   Gauss con (A, B), donde
!   A ; matriz de coeficientes
!   B ; matriz de cargas
! -----------------------------------------------------------------
module  pde_sor
  use ctes
  use globals
  use tools
  implicit none
  private                           ! por omision todo es privado
! public sor1
  public sor2
  character (10), parameter :: c = "pde_sor > "
  contains
  !
  !----------------------------------------------------------------
  subroutine sor2 (a, b, c, d, e,f, u, rjac)
    real (idp), dimension (:,:), target, intent (in)   :: a, b, c
    real (idp), dimension (:,:), target, intent (in)   :: d, e, f
    real (idp), dimension (:,:), target, intent (inout):: u
    real (idp)                         , intent (in)   :: rjac
    !
    real    (idp), dimension (:,:), allocatable :: resid
    logical      , dimension (:,:), allocatable :: rojo
    !
    real    (idp), dimension (:,:), pointer :: u_int
    real    (idp), dimension (:,:), pointer :: u_down, u_up
    real    (idp), dimension (:,:), pointer :: u_left, u_right
    !
    real    (idp), dimension (:,:), pointer :: a_int, b_int
    real    (idp), dimension (:,:), pointer :: c_int, d_int
    real    (idp), dimension (:,:), pointer :: e_int, f_int
    !
    integer (iin), dimension (14) :: ii
    !
    real    (idp) :: fnorm, rnorm, omega
    real    (idp) :: alfa, beta, chi
    integer (iin) :: jmx, jm1, jm2, jm3
    integer (iin) :: n, m
    integer (iin) :: i, j, k, p
    !
    ii  (1) = size (a,1)
    ii  (2) = size (a,2)
    ii  (3) = size (b,1)
    ii  (4) = size (b,2)
    ii  (5) = size (c,1)
    ii  (6) = size (c,2)
    ii  (7) = size (d,1)
    ii  (8) = size (d,2)
    ii  (9) = size (e,1)
    ii (10) = size (e,2)
    ii (11) = size (f,1)
    ii (12) = size (f,2)
    ii (13) = size (u,1)
    ii (14) = size (u,2)
    jmx = siza ( ii, "sor2: jmx" )
    jm1 = jmx - 1
    jm2 = jmx - 2
    jm3 = jmx - 3
    !
    ! aloca para el interior: A (jmx-2, jmx-2)
    allocate ( resid (jm2, jm2), rojo (jm2, jm2) )
    !
    rojo = .false.
    rojo (1:jm2:2, 1:jm2:2) = .true.
    rojo (2:jm3:2, 2:jm3:2) = .true.
    !
    u_int   => u (2:jm1,2:jm1)
    u_down  => u (3:jmx,2:jm1)
    u_up    => u (1:jm2,2:jm1)
    u_left  => u (2:jm1,1:jm2)
    u_right => u (2:jm1,3:jmx)
    !
    a_int => a (2,jm1,2:jm1)
    b_int => b (2,jm1,2:jm1)
    c_int => c (2,jm1,2:jm1)
    d_int => d (2,jm1,2:jm1)
    e_int => e (2,jm1,2:jm1)
    f_int => f (2,jm1,2:jm1)
    !
    fnorm = sum (abs(f_int))
    omega = 1.0
    chi = rjac * rjac
    itera: do n = 1, max_ite
      where (rojo)
        resid = a_int * u_down + b_int * u_up  + c_int * u_right &
              + d_int * u_left + e_int * u_int - f_int
        u_int = u_int - omega * resid / e_int
      end where
      alfa = 1.0 / (1.0 - 0.50 * chi)
      beta = 1.0 / (1.0 - 0.25 * chi * omega)
      omega = merge (alfa,beta,n==1)
      where (.not. rojo)
        resid = a_int * u_down + b_int * u_up  + c_int * u_right &
              + d_int * u_left + e_int * u_int - f_int
        u_int = u_int - omega * resid / e_int
      end where
      omega = 1.0 / (1.0 - 0.25 * chi * omega)
      rnorm = sum (abs (resid))
      if (rnorm < eps_ite * fnorm) exit
    end do itera
    deallocate (resid, rojo)
    if (n > max_ite) call errata (max_ite, "supera max_ite")
  end subroutine
  !
  !----------------------------------------------------------------
end module
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
