! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Matrices ralas: formato CSC (Compressed Sparse Column)
! Producto y=Ax con scatter HPF
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! GNU:   
! ***
!
! PGI (HPF):   
! pghpf -V -Minfo -Mmpi -fast -c rala_csc.f90
! pghpf -V -Minfo -Mmpi -fast -o rala_csc.exe rala_csc.o
! mpirun -machinefile machi.dat -np 1 rala_csc.exe
!
! ADAPTOR (HPF)
! compilation for Distributed Memory Machines using MPI:
! adaptor -hpf -dm -free -c rala_csc.f90
! adaptor -hpf -dm -o rala_csc.exe rala_csc.o
! mpirun -machinefile machi.dat -np 1 rala_csc.exe
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program rala_csc
  use hpf_library
  implicit none
  integer, parameter :: n = 7, p = 17
  integer, dimension (n)   :: x, y
  integer, dimension (n+1) :: j
  integer, dimension (p)   :: i
  integer, dimension (p)   :: a, b, c, d
  logical, dimension (p)   :: s
  !hpf$ distribute (block)   :: a
  !hpf$ align (:) with a (:) :: b, c, d
  !hpf$ align (:) with a (:) :: i, s
  !hpf$ align (:) with a (:) :: x, y
  integer :: k, l
  logical :: tira
  !
  write (*,*)
  write (*,*) "Matrices ralas: formato CSC (Compressed Sparse Column)"
  write (*,*) "producto y=Ax con scatter HPF"
  !
  j = (/ 1, 4, 6, 9, 10, 13, 16, 18 /)
  i = (/ 1, 4, 6, 2, 3, 1, 3, 5, 4, 2, 5, 7, 3, 6, 7, 5, 7 /)
  a = (/ 1, 2, 3, 4, 5, 6, 7, 8, 9, 1, 2, 3, 4, 5, 6, 7, 8 /) 
  x = (/ 1, 2, 3, 4, 5, 6, 7 /)
  !
  ! construye segmentos booleanos para identificar cada columna
  tira = .true.
  do  k = 1, n
   do  l = j (k), j (k + 1) - 1 
     s (l) = tira
   end do
   tira = .not. tira
  end do
  write (*,*)
  write (*,100)" i          : ", i
  write (*,100)" j          : ", j
  write (*,100)" a          : ", a
  write (*,110)" s          : ", s
  !
  b (j (1:n)) = x                   
  c = copy_prefix (b, segment = s)
  d = a * c
  y = 0
  y (1:n) = sum_scatter (d (1:p), y (1:n), i (1:p) ) 
  write (*,*)
  write (*,100)" b(j(1:n))  : ", b
  write (*,100)" copy_prefix: ", c
  write (*,100)" a * b      : ", d
  write (*,100)" x          : ", x
  write (*,100)" y          : ", y
  write (*,*)
  100 format (a, 20 (1x,i4))
  110 format (a, 20 (1x,l4))
end program 
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
