! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
! Matrices ralas: formato CSR (Compressed Sparse Row)
! Producto y=Ax con scatter HPF
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! GNU:   
! ***
!
! PGI (HPF):   
! pghpf -V -Minfo -Mmpi -fast -c rala_csr.f90
! pghpf -V -Minfo -Mmpi -fast -o rala_csr.exe rala_csr.o
! mpirun -machinefile machi.dat -np 1 rala_csr.exe
!
! ADAPTOR (HPF)
! compilation for Distributed Memory Machines using MPI:
! adaptor -hpf -dm -free -c rala_csr.f90
! adaptor -hpf -dm -o rala_csr.exe rala_csr.o
! mpirun -machinefile machi.dat -np 1 rala_csr.exe
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
program rala_csr
  use hpf_library
  implicit none
  integer, parameter :: n = 7, p = 17
  integer, dimension (n)   :: x, y
  integer, dimension (n+1) :: i
  integer, dimension (p)   :: j
  integer, dimension (p)   :: a, b, c, d
  logical, dimension (p)   :: s
  !hpf$ distribute (block)   :: a
  !hpf$ align (:) with a (:) :: b, c, d
  !hpf$ align (:) with a (:) :: i, s
  !hpf$ align (:) with a (:) :: x, y
  integer :: k, l
  logical :: tira
  !
  write (*,*)
  write (*,*) "Matrices ralas: formato CSR (Compressed Sparse Row)"
  write (*,*) "producto y=Ax con scatter HPF"
  !
  i = (/ 1, 3, 5,  8, 10, 13, 15, 18 /)
  j = (/ 1, 3, 2, 5, 2, 3, 6, 1, 4, 3, 5, 7, 1, 6, 5, 6, 7 /)
  a = (/ 1, 6, 4, 1, 5, 7, 4, 2, 9, 8, 2, 7, 3, 5, 3, 6, 8 /) 
  x = (/ 1, 2, 3, 4, 5, 6, 7 /)
  !
  ! construye segmentos booleanos para identificar cada columna
  tira = .true.
  do  k = 1, n
   do  l = i (k), i (k + 1) - 1 
     s (l) = tira
   end do
   tira = .not. tira
  end do
  write (*,*)
  write (*,100)" i          : ", i
  write (*,100)" j          : ", j
  write (*,100)" a          : ", a
  write (*,110)" s          : ", s
  !
  b = x (j (1:p))                   
  c = a * b
  d (1:p) = sum_suffix (c (1:p), segment = s) 
  y = d (i(1:n))
  write (*,*)
  write (*,100)" x(j(1:p))  : ", b
  write (*,100)" a * b      : ", c
  write (*,100)" sum_suffix : ", d
  write (*,100)" x          : ", x
  write (*,100)" y          : ", y
  write (*,*)
  100 format (a, 20 (1x,i4))
  110 format (a, 20 (1x,l4))
end program 
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
