! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
!
! "clase" simple para un
!  Sistema de Ecuaciones Algebraicas Rala (SEAR)
!
!  Resolucion iterativa: por Conjugate Gradient Squared (CGS)
!
!  Paralelismo opcional: HPF unicamente via compilador PGI
!
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
module sea_rala1
 use hpf_library
 use ctes
 use temps
 use globals
 use tools1
 implicit none
 private         ! por omision todo es privado
 public ralo_t
 public random_sea
 public tridia_sea
 public prelee_rala1
 public prelee_rala2
 public lect_rala1
 public lect_rala2
 public eco_toda
 public eco_diag
 public sax1_rala
 public sax2_rala
 public traspuesta_rala
 public diagonal_rala
 public cgs_rala
 !
 ! parametros globales del modulo
 integer (iin), parameter :: nmax = 100
 logical (iin), parameter :: adisco = .false.
 character(11), parameter :: f = 'sea_rala:> '
 !
 ! este tipo de formato ralo tiene menos estructura que el formato
 ! "row - indexed sparse storage mode"
 type ralo_t
  integer (iin) :: n   ! dimension de la matriz
  integer (iin) :: m   ! nro de cargas en el miembro derecho del SEA
  integer (iin) :: p   ! nro de elementos en la matriz rala
  integer (iin) :: nit ! nro max de iteraciones
  real    (idp) :: rum ! tolerancia
  character (16):: raiz
 end type
 !
 contains
 !
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 ! genera en forma aleatoria un SEAR cuya matriz de coeficientes es
 ! rala pero dominante en su diagonal principal
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 subroutine random_sea (sa, ifila, jcolu, valor, b)
  type (ralo_t)                 , intent (in)  :: sa 
  integer (iin), dimension   (:), intent (out) :: ifila
  integer (iin), dimension   (:), intent (out) :: jcolu
  real    (idp), dimension   (:), intent (out) :: valor
  real    (idp), dimension (:,:), intent (out) :: b
  logical      , dimension (size(ifila)) :: mask
  real    (idp), dimension (size(ifila)) :: a
  !hpf$ inherit, distribute (cyclic) :: ifila
  !hpf$ align jcolu (:) with ifila (:)
  !hpf$ align valor (:) with ifila (:)
  character (len(sa % raiz)+4) :: arch1, arch2
  character (len(sa % raiz))   :: raiz
  integer (iin), dimension (2) :: ij
  real    (idp) :: rr
  integer (iin) :: n, m, p, h
  integer (iin) :: ii, jj, ll
  integer (iin) :: i, j, k
  !
  n = sa % n
  p = sa % p
  m = sa % m
  raiz = sa % raiz
  !
  write (*,100) f
  write (*,100) f // "genera matriz SEA rala aleatoria ... "
  write (*,100) f
  write (*,110) f // "nro de incognitas     ; n   = ", n
  write (*,110) f // "nro de cargas         ; m   = ", m
  write (*,110) f // "nro de elementos ralos; p   = ", p
  !
  ! indices
  ifila (1:n) = barajar (1,n)
  jcolu (1:n) = ifila (1:n)
  do  k = (n + 1), p
    ij = irandom_ne (1, n+1)
    ifila (k) = ij (1)
    jcolu (k) = ij (2)
  end do
  !
  ! coeficientes
  call  random_number (valor)
  rr = sqrt ( real (n) )
  !hpf$ independent
  do  k = 1, n
   valor (k) = valor (k) + rr
  end do
  !
  ! arreglo de carga
  call  random_number (b)
  !
  ! eventual retorno
  if (.not. adisco) return

  ! impresion a disco
  ll = len_trim (raiz)
  arch1 = raiz (1:ll) // "_a.dat"
  arch2 = raiz (1:ll) // "_b.dat"

  write (*,100) f
  write (*,100) f // "graba en disco TODA la matriz rala ... "
  write (*,100) f
  write (*,100) f // "archivo matriz: " // arch1
  write (*,100) f // "        carga : " // arch2
  !
  open (1, file = arch1, status = "unknown")
  do  i = 1, p
   ii = ifila (i)
   jj = jcolu (i)
   rr = valor (i)
   write (1,300) ii, jj, rr
  end do
  close (1, status = "keep")
  !
  open (2, file = arch2, status = "unknown")
  do jj = 1, m
   do ii = 1, n
    rr = b (ii,jj)
    write (2,300) ii, jj, rr
   end do
  end do
  close (2, status = "keep")
  !
  ! formatos
  100 format (a)
  110 format (a, 2 (i8), e24.12)
  120 format (a,         e24.12)
  300 format ( 2 (1x, i8), 1x, e24.16 )
  !
 end subroutine
 !
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 ! genera SEA tridiagonal
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 subroutine tridia_sea (sa, ifila, jcolu, valor, b)
  type (ralo_t)                 , intent (in)  :: sa 
  integer (iin), dimension   (:), intent (out) :: ifila
  integer (iin), dimension   (:), intent (out) :: jcolu
  real    (idp), dimension   (:), intent (out) :: valor
  real    (idp), dimension (:,:), intent (out) :: b
  logical      , dimension (size(ifila)) :: mask
  real    (idp), dimension (size(ifila)) :: a
  real    (idp), dimension (3)           :: coef
  !hpf$ inherit, distribute (cyclic) :: ifila
  !hpf$ align jcolu (:) with ifila (:)
  !hpf$ align valor (:) with ifila (:)
  character (len(sa % raiz)+4) :: arch1, arch2
  character (len(sa % raiz))   :: raiz
  integer (iin), dimension (2) :: ij
  real    (idp) :: rr
  integer (iin) :: n, m, p, h
  integer (iin) :: ii, jj, ll
  integer (iin) :: i, j, k
  integer (iin) :: j1, j2
  !
  n = sa % n
  p = sa % p
  m = sa % m
  raiz = sa % raiz
  !
  write (*,100) f
  write (*,100) f // "genera un SEA triadiagonal ... "
  write (*,100) f
  write (*,110) f // "nro de incognitas     ; n   = ", n
  write (*,110) f // "nro de cargas         ; m   = ", m
  write (*,110) f // "nro de elementos ralos; p   = ", p
  !
  ! indices
  coef = (/ -1.0, log(dble(n)), -1.0 /)
  k = 0
  do  i = 1, n
   j1 = i - 1
   j2 = i + 1
   if (i .eq. 1)  j1 = i
   if (i .eq. n)  j2 = i
   h = 0
   do  j = j1, j2
     k = k + 1
     h = h + 1
     if (k > p) call errata (k, "k > p")
     ifila (k) = i
     jcolu (k) = j
     valor (k) = coef (h)
   end do ! j
  end do ! i
  !
  ! arreglo de carga
  do  k = 1, m
  do  j = 1, n
    b (k,j) = dble ((k - 1) * m + j)
  end do
  end do
  !
  if (.not. adisco) return
  !
  ! impresion a disco
  ll = len_trim (raiz)
  arch1 = raiz (1:ll) // "_a.dat"
  arch2 = raiz (1:ll) // "_b.dat"
  !
  write (*,100) f
  write (*,100) f // "graba en disco TODA la matriz rala ... "
  write (*,100) f
  write (*,100) f // "archivo matriz: " // arch1
  write (*,100) f // "        carga : " // arch2
  !
  open (1, file = arch1, status = "unknown")
  do  i = 1, p
   ii = ifila (i)
   jj = jcolu (i)
   rr = valor (i)
   write (1,300) ii, jj, rr
  end do
  close (1, status = "keep")
  !
  open (2, file = arch2, status = "unknown")
  do jj = 1, m
   do ii = 1, n
    rr = b (ii,jj)
    write (2,300) ii, jj, rr
   end do
  end do
  close (2, status = "keep")
  !
  ! formatos
  100 format (a)
  110 format (a, 2 (i8), e24.12)
  120 format (a,         e24.12)
  300 format ( 2 (1x, i8), 1x, e24.16 )
  !
 end subroutine
 !
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 ! prelectura SEAR asumiendo que en disco se tiene TODA la matriz
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 subroutine prelee_rala1 (raiz, sa)
  character (*), intent (in)  :: raiz
  type (ralo_t), intent (out) :: sa 
  !
  character (len(raiz)+7) :: arch1, arch2
  integer (iin) :: n, m, p
  integer (iin) :: k1, k2
  integer (iin) :: l1, l2
  integer (iin) :: n1, n2, ll
  integer (iin) :: ii, jj, kk
  integer (iin) :: nlineas
  real    (idp) :: rr
  !
  ll = len_trim (raiz)
  arch1 = raiz (1:ll) // "_a.dat"
  arch2 = raiz (1:ll) // "_b.dat"
  !
  write (*,100) f
  write (*,100) f // "prelectura asumiendo que en disco se tiene "
  write (*,100) f // "TODA la matriz rala ... "
  write (*,100) f
  write (*,100) f // "archivo matriz: " // arch1
  write (*,100) f // "        carga : " // arch2
  !
  open (1, file = arch1, status = "unknown")
  l1 = 0
  n1  = 0
  itera1: do
   read (1,*,end=10) ii, jj, rr
   if (ii > n1)  n1 = ii
   if (jj > n1)  n1 = jj
   l1 = l1 + 1
  end do itera1
  10 rewind (1)
  !
  open (2, file = arch2, status = "unknown")
  l2 = 0
  n2 = 0
  k2 = 0
  itera2: do
   read (2,*,end=20) ii, kk, rr
   if (ii > n2) n2 = ii
   if (kk > k2) k2 = kk
   l2 = l2 + 1
  end do itera2
  20 rewind (2)
  !
  n = siza (n1,n2, "prelee_sea")
  p = l1               ! los elementos que deben quedar
  m = k2
  if ( (n * m) .ne. l2 ) call errata ("no sincroniza")
  !
  sa % n = n
  sa % p = p
  sa % m = m
  !
  ! formatos
  100 format (a)
  110 format (a, 2 (i8), e24.12)
  120 format (a,         e24.12)
  300 format ( 2 (1x, i8), 1x, e24.16 )
  !
 end subroutine
 !
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 ! prelectura SEAR simetrico asumiendo que en disco solo se tiene 
 ! la parte triangular superior mas su diagonal principal
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 subroutine prelee_rala2 (raiz, sa)
  character (*), intent (in)  :: raiz
  type (ralo_t), intent (out) :: sa 
  !
  character (len(raiz)+7) :: arch1, arch2
  integer (iin) :: n, m, p
  integer (iin) :: k1, k2
  integer (iin) :: l1, l2
  integer (iin) :: n1, n2, ll
  integer (iin) :: ii, jj, kk
  integer (iin) :: nlineas
  real    (idp) :: rr
  !
  ll = len_trim (raiz)
  arch1 = raiz (1:ll) // "_a.dat"
  arch2 = raiz (1:ll) // "_b.dat"
  write (*,*)
  write (*,*) "prelectura SEAR simetrico asumiendo que en disco "
  write (*,*) "se tiene solo la parte TRIA SUP + DIAG PPAL"
  write (*,*)
  write (*,*) "archivo matriz: " // arch1
  write (*,*) "        carga : " // arch2
  !
  open (1, file = arch1, status = "unknown")
  l1 = 0
  n1  = 0
  itera1: do
   read (1,*,end=10) ii, jj, rr
   if (ii > n1)  n1 = ii
   if (jj > n1)  n1 = jj
   l1 = l1 + 1
  end do itera1
  10 rewind (1)
  !
  open (2, file = arch2, status = "unknown")
  l2 = 0
  n2 = 0
  k2 = 0
  itera2: do
   read (2,*,end=20) ii, kk, rr
   if (ii > n2) n2 = ii
   if (kk > k2) k2 = kk
   l2 = l2 + 1
  end do itera2
  20 rewind (2)
  !
  n = siza (n1,n2, "prelee_sea")
  p = 2 * (l1 - n) + n ! los elementos que deben quedar
  m = k2
  if ( (n * m) .ne. l2 ) call errata ("no sincroniza")
  !
  sa % n = n
  sa % p = p
  sa % m = m
  !
  !write (*,*)
  !write (*,*)'nro de lineas       ; l1  = ', l1
  !write (*,*)'nro de cargas       ; m   = ', m
  !write (*,*)'nro de incognitas   ; n   = ', n
  !write (*,*)'nro de elementos    ; p   = ', p
  !
 end subroutine
 !
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 ! lectura SEAR asumiendo que en disco se tiene TODA la matriz
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 subroutine  lect_rala1 (sa, ifila, jcolu, valor, b)
  type (ralo_t)           , intent (in)  :: sa 
  integer (iin), dimension   (:), intent (out) :: ifila
  integer (iin), dimension   (:), intent (out) :: jcolu
  real    (idp), dimension   (:), intent (out) :: valor
  real    (idp), dimension (:,:), intent (out) :: b
  !hpf$ inherit, distribute (cyclic) :: ifila
  !hpf$ align jcolu (:) with ifila (:)
  !hpf$ align valor (:) with ifila (:)
  !
  integer (iin) :: n, m, p, k, l
  integer (iin) :: k1, k2
  integer (iin) :: l1, l2
  integer (iin) :: n1, n2, ll
  integer (iin) :: ii, jj, kk
  integer (iin) :: nlineas
  real    (idp) :: rr
  !
  ! tamanios
  n = sa % n
  p = sa % p
  m = sa % m
  !
  write (*,100) f
  write (*,100) f // "lectura asumiendo que en disco se tiene "
  write (*,100) f // "TODA la matriz rala ... "
  write (*,100) f
  write (*,110) f // "nro de incognitas        ;    n = ", sa % n
  write (*,110) f // "nro de cargas            ;    m = ", sa % m
  write (*,110) f // "nro de elementos ralos   ;    p = ", sa % p
  write (*,110) f // "nro maximo de iteracione ;  nit = ", sa % nit
  write (*,120) f // "tolerancia               ;  rum = ", sa % rum
  write (*,100) f // "raiz archivos            ; raiz = " // sa % raiz
  write (*,100) f
  write (*,100) f // "matriz del sistema ... "
  write (*,100) f
  do  k = 1, p
   !write (*,110) f // "k = ", k
   read (1,*) ii, jj, rr
   if (n < nmax)  write (*,110) f, ii, jj, rr
   ifila (k) = ii
   jcolu (k) = jj
   valor (k) = rr
  end do
  !
  write (*,100) f
  write (*,100) f // "matriz de cargas ... "
  nlineas = m * n
  do  l = 1, nlineas
   !write (*,110) f // "l = ", l
   read  (2,*) ii, kk, rr
   if (n < nmax) write (*,110) f, ii, kk, rr
   b (kk,ii) = rr
  end do
  !
  !
  write (*,100) f
  write (*,100) f // "fin lectura ... "
  !
  close (1,status = "keep")
  close (2,status = "keep")
  !
  ! formatos
  100 format (a)
  110 format (a, 2 (i8), e24.12)
  120 format (a,         e24.12)
  300 format ( 2 (1x, i8), 1x, e24.16 )
  !
 end subroutine
 !
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 ! lectura SEAR simetrico asumiendo que en disco solo se tiene 
 ! la parte triangular superior mas su diagonal principal
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 subroutine  lect_rala2 (sa, ifila, jcolu, valor, b)
  type (ralo_t)           , intent (in)  :: sa 
  integer (iin), dimension   (:), intent (out) :: ifila
  integer (iin), dimension   (:), intent (out) :: jcolu
  real    (idp), dimension   (:), intent (out) :: valor
  real    (idp), dimension (:,:), intent (out) :: b
  !hpf$ inherit, distribute (cyclic) :: ifila
  !hpf$ align jcolu (:) with ifila (:)
  !hpf$ align valor (:) with ifila (:)
  !
  integer (iin) :: n, m, p, k, l
  integer (iin) :: k1, k2
  integer (iin) :: l1, l2
  integer (iin) :: n1, n2, ll
  integer (iin) :: ii, jj, kk
  integer (iin) :: nlineas
  integer (iin) :: pmitad
  real    (idp) :: rr
  !
  n = sa % n
  p = sa % p
  m = sa % m
  !
  write (*,*)
  write (*,*) "lectura SEAR simetrico asumiendo que en disco "
  write (*,*) "se tiene solo la parte TRIA SUP + DIAG PPAL"
  write (*,*)
  write (*,*) "lee matriz rala completando la parte TRIA INF"
  k = 0
  pmitad = (p - n) / 2 + n
  do  l = 1, pmitad
   read  (1,*) ii, jj, rr
   write (*,*) ii, jj, rr
   k = k + 1
   ifila (k) = ii
   jcolu (k) = jj
   valor (k) = rr
   if (ii .eq. jj) cycle
   k = k + 1
   if (k > p) call errata ("p > p")
   write (*,*) jj, ii, rr
   ifila (k) = jj
   jcolu (k) = ii
   valor (k) = rr
  end do
  write (*,*)
  write (*,*) "nro de incognitas   ;      n = ", n
  write (*,*) "nro de cargas       ;      m = ", m
  write (*,*) "nro de coeficientes ;      p = ", p
  write (*,*) "nro de lineas leidas; pmitad = ", pmitad
  write (*,*) "ultimo coefic asigna;      k = ", k
  !
  write (*,*)
  write (*,*) "matriz de cargas"
  nlineas = m * n
  do  ll = 1, nlineas
   read  (2,*) ii, kk, rr
   write (*,*) ii, kk, rr
   b (kk,ii) = rr
  end do
  close (1,status = "keep")
  close (2,status = "keep")
  !
 end subroutine
 !
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 ! imprime toda la matriz rala a disco
 subroutine eco_toda (sa, ifila, jcolu, valor)
  type (ralo_t)         , intent (in) :: sa 
  integer (iin), dimension (:), intent (in) :: ifila
  integer (iin), dimension (:), intent (in) :: jcolu
  real    (idp), dimension (:), intent (in) :: valor
  !hpf$ inherit, distribute (cyclic) :: ifila
  !hpf$ align jcolu (:) with ifila (:)
  !hpf$ align valor (:) with ifila (:)
  integer (iin) :: ii, jj, k
  integer (iin) :: n, p
  real    (idp) :: rr
  n = sa % n
  p = sa % p
  write (*,*)
  write (*,*) f // "imprime toda la matriz rala a disco ... "
  write (*,*) f // "n = ", n
  write (*,*) f // "p = ", p
  do  k = 1, p
   ii = ifila (k)
   jj = jcolu (k)
   rr = valor (k)
   write (10,100) ii, jj, rr
  end do
  100 format (2 (1x, i8), 1 (1x, e24.16) )
  !
 end subroutine
 !
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 ! imprime la diagonal principal del SEAR
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 subroutine eco_diag (sa, ifila, jcolu, valor)
  type (ralo_t)         , intent (in) :: sa 
  integer (iin), dimension (:), intent (in) :: ifila
  integer (iin), dimension (:), intent (in) :: jcolu
  real    (idp), dimension (:), intent (in) :: valor
  !hpf$ inherit, distribute (cyclic) :: ifila
  !hpf$ align jcolu (:) with ifila (:)
  !hpf$ align valor (:) with ifila (:)
  real    (idp), dimension (:), allocatable :: u
  integer (iin) :: k
  integer (iin) :: n, p
  n = sa % n
  p = sa % p
  write (*,*)
  write (*,*) f // "imprime diagonal principal en disco ... "
  write (*,*) f // "n = ", n
  write (*,*) f // "p = ", p
  allocate ( u (n) )
  call diagonal_rala (sa,ifila,jcolu,valor,u)
  do  k = 1, n
   write (11,100)  k, k, u (k)
  end do
  deallocate (u)
  100 format ( 2 (1x,i8), 1 (1x,e24.16) )
  !
 end subroutine
 !
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 ! producto matriz rala por vector: y = A * x ;
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 subroutine vaxqy (sa, ifila, jcolu, valor, x, y)
  type (ralo_t)           , intent (in) :: sa 
  integer (iin), dimension (:)  , intent (in) :: ifila
  integer (iin), dimension (:)  , intent (in) :: jcolu
  real    (idp), dimension (:)  , intent (in) :: valor
  real    (idp), dimension (:,:), intent (in) :: x
  real    (idp), dimension (:,:), intent (out):: y
  !hpf$ inherit, distribute (cyclic) :: ifila
  !hpf$ align jcolu (:) with ifila (:)
  !hpf$ align valor (:) with ifila (:)
  integer (iin) :: n, m, k
  n = siza (size (x,2), size (y,2), "vaxqy, n")
  m = siza (size (x,1), size (y,1), "vaxqy, m")
  do  k = 1, m
  call sax1_rala (sa, ifila, jcolu, valor, x (k,:), y (k,:))
  end do 
 end subroutine
 !
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 ! producto matriz traspuesta rala por vector: y = A' * x ;
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 subroutine waxqy (sa, ifila, jcolu, valor, x, y)
  type (ralo_t)                 , intent (in) :: sa 
  integer (iin), dimension (:)  , intent (in) :: ifila
  integer (iin), dimension (:)  , intent (in) :: jcolu
  real    (idp), dimension (:)  , intent (in) :: valor
  real    (idp), dimension (:,:), intent (in) :: x
  real    (idp), dimension (:,:), intent (out):: y
  !hpf$ inherit, distribute (cyclic) :: ifila
  !hpf$ align jcolu (:) with ifila (:)
  !hpf$ align valor (:) with ifila (:)
  integer (iin) :: n, m, k
  n = siza (size (x,2), size (y,2), "waxqy, n")
  m = siza (size (x,1), size (y,1), "waxqy, m")
  do  k = 1, m
  call sax2_rala (sa, ifila, jcolu, valor, x (k,:), y (k,:))
  end do
 end subroutine
 !
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 ! precondicionador de Jacobi
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 subroutine qdiag_prec (qii, x, y)
  real    (idp), dimension (:)  , intent (in) :: qii
  real    (idp), dimension (:,:), intent (in) :: x
  real    (idp), dimension (:,:), intent (out):: y
  integer (iin) :: k, m, n
  n = siza (size (x,2), size (y,2), "qdiag_prec")
  m = siza (size (x,1), size (y,1), "qdiag_prec")
  do  k = 1, m
   y (k,:) = qii (:) * x (k,:)
  end do
 end subroutine
 !
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 ! Multiplica la matriz "sa" en formato ralo por un vector "x" dando  
 ! el vector "y". Cada entrada de la matriz rala "sa" suma un termino 
 ! a alguna componente de "b"
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 subroutine sax1_rala (sa, ifila, jcolu, valor, x, y)
  type (ralo_t)               , intent (in) :: sa 
  integer (iin), dimension (:), intent (in) :: ifila
  integer (iin), dimension (:), intent (in) :: jcolu
  real    (idp), dimension (:), intent (in) :: valor
  real    (idp), dimension (:), intent (in) :: x
  real    (idp), dimension (:), intent (out):: y
  !hpf$ inherit, distribute (cyclic) :: ifila
  !hpf$ align jcolu (:) with ifila (:)
  !hpf$ align valor (:) with ifila (:)
  integer (iin) :: n, p
  n = siza (sa % n, size (x), size (y), "sax1_rala")
  p = siza (sa % p, size (valor)      , "sax1_rala")
  y = 0.0
  ! call scatter_add (y, valor * x (jcolu), ifila)
  y (1:n) = sum_scatter (valor * x (jcolu), y (1:n), ifila (1:p))
 end subroutine
 !
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 ! Multiplica la traspuesta de la matriz "sa" en formato ralo por un 
 ! vector "x" dando el vector "b". Cada entrada de la matriz rala "sa" 
 ! suma un termino a alguna componente de "b"
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 subroutine sax2_rala (sa, ifila, jcolu, valor, x, y)
  type (ralo_t)               , intent (in) :: sa 
  integer (iin), dimension (:), intent (in) :: ifila
  integer (iin), dimension (:), intent (in) :: jcolu
  real    (idp), dimension (:), intent (in) :: valor
  real    (idp), dimension (:), intent (in) :: x
  real    (idp), dimension (:), intent (out):: y
  !hpf$ inherit, distribute (cyclic) :: ifila
  !hpf$ align jcolu (:) with ifila (:)
  !hpf$ align valor (:) with ifila (:)
  integer (iin) :: n, p
  n = siza (sa % n, size (x), size (y), "sax2_rala, n")
  p = siza (sa % p, size (valor)      , "sax2_rala, p")
  y = 0.0
  ! call scatter_add (y, valor * x (ifila), jcolu)
  y (1:n) = sum_scatter (valor * x (ifila), y (1:n), jcolu (1:p))
 end subroutine
 !
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 ! Reemplaza la matriz "sa" en formato ralo por su traspuesta
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 subroutine traspuesta_rala (ifila, jcolu)
  integer (iin), dimension (:), intent (inout) :: ifila
  integer (iin), dimension (:), intent (inout) :: jcolu
  integer (iin), dimension (size(ifila)) :: t
  !hpf$ inherit, distribute (cyclic) :: ifila
  !hpf$ align jcolu (:) with ifila (:)
  !hpf$ align     t (:) with ifila (:)
  t = ifila
  ifila = jcolu
  jcolu = t
 end subroutine
 !
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 ! extrae la diagonal principal de una matriz rala
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 subroutine diagonal_rala (sa, ifila, jcolu, valor, u)
  type (ralo_t)               , intent (in) :: sa 
  integer (iin), dimension (:), intent (in) :: ifila
  integer (iin), dimension (:), intent (in) :: jcolu
  real    (idp), dimension (:), intent (in) :: valor
  real    (idp), dimension (:), intent (out):: u
  real    (idp), dimension (size(u)) :: v
  integer (iin), dimension (size(u)) :: i
  logical      , dimension (:), allocatable :: mask
  !hpf$ inherit, distribute (cyclic) :: ifila
  !hpf$ align jcolu (:) with ifila (:)
  !hpf$ align valor (:) with ifila (:)
  !hpf$ align mask  (:) with ifila (:)
  integer (iin) :: k, p, n, h
  n = siza (sa % n, size (u), "diag_rala")
  p = siza (size (ifila), size (jcolu), size (valor), "diag_rala")
  allocate (mask (p))
  mask = ( ifila (1:p) == jcolu (1:p) )
  k = count (mask)
  !call copia_arreglo (pack(valor(1:p),mask), valor, k, h)
  if (k < 1) call errata (k, "k < 1")
  if (k > n) call errata (k, "k > n")
  v (1:k) = pack (valor (1:p), mask)
  i (1:k) = pack (ifila (1:p), mask)
  u = 0.0
  u (i(1:k)) = v (1:k)
  deallocate (mask, stat=ier(1))
 end subroutine
 !
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 ! resolucion iterativa del SEAL:  H S = B
 ! por Conjugate Gradient Squared (CGS), donde
 !   H ; matriz del sistema
 !   S ; matriz solucion
 !   B ; matriz fuente
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 subroutine cgs_rala (sa, ifila, jcolu, valor, b, nit, rum)
  type (ralo_t)               , intent (in) :: sa 
  integer (iin), dimension (:), intent (in) :: ifila
  integer (iin), dimension (:), intent (in) :: jcolu
  real    (idp), dimension (:), intent (in) :: valor
  !hpf$ inherit, distribute (cyclic) :: ifila
  !hpf$ align jcolu (:) with ifila (:)
  !hpf$ align valor (:) with ifila (:)
  !
  real    (idp), dimension (:,:), intent (inout) :: b
  integer (iin)                 , intent (in)    :: nit
  real    (idp)                 , intent (in)    :: rum
  !
  real    (idp), dimension (size(b,2)) :: qii
  real    (idp), allocatable, dimension (:,:) ::  w, s
  real    (idp), allocatable, dimension (:,:) :: t1, t2, t3
  real    (idp), allocatable, dimension (:,:) :: r1, r2, r3
  real    (idp), allocatable, dimension (:,:) :: p1, p2, p3
  real    (idp), allocatable, dimension (:)   :: alfa, beta
  real    (idp), allocatable, dimension (:)   :: gama, delt
  real    (idp), allocatable, dimension (:)   :: omic, taug
  real    (idp), allocatable, dimension (:)   :: zeta, rhog
  logical      , allocatable, dimension (:)   :: otro
  !
  character (60):: pala
  real    (idp) :: mflops, elas
  integer (iin) :: i, j, k, l
  integer (iin) :: h1, h2, h3, h4, dh
  integer (iin) :: nope1, nope2, noper
  integer (iin) :: n, p
  integer (iin) :: nil, nek
  logical :: sale
  !
  nil = size (b,2)  ! incognitas
  nek = size (b,1)  ! cargas
  !
  write (*,200) f
  write (*,200) f // 'Conjugate Gradients Squared  '
  write (*,200) f // '(CGS) Ribersek + barrido ... ' 
  write (*,200) f
  write (*,300) f // 'nro de incognitas ;  nil = ', nil
  write (*,300) f // 'nro de cargas     ;  nek = ', nek
  write (*,300) f // 'techo iteraciones ;  nit = ', nit 
  write (*,400) f // 'tolerancia        ;  rum = ', real (rum)
  write (*,200) f
  !
  ! aloca memoria
  ier = 0
  allocate (    s (nek,nil), stat = ier  (1) )
  allocate (    w (nek,nil), stat = ier  (2) )
  allocate (   t1 (nek,nil), stat = ier  (3) )
  allocate (   t2 (nek,nil), stat = ier  (4) )
  allocate (   t3 (nek,nil), stat = ier  (5) )
  allocate (   p1 (nek,nil), stat = ier  (6) )
  allocate (   p2 (nek,nil), stat = ier  (7) )
  allocate (   p3 (nek,nil), stat = ier  (8) )
  allocate (   r1 (nek,nil), stat = ier  (9) )
  allocate (   r2 (nek,nil), stat = ier (10) )
  allocate (   r3 (nek,nil), stat = ier (11) )
  allocate ( alfa (nek)    , stat = ier (12) )
  allocate ( beta (nek)    , stat = ier (13) )
  allocate ( gama (nek)    , stat = ier (14) )
  allocate ( delt (nek)    , stat = ier (15) )
  allocate ( omic (nek)    , stat = ier (16) )
  allocate ( taug (nek)    , stat = ier (17) )
  allocate ( zeta (nek)    , stat = ier (18) )
  allocate ( rhog (nek)    , stat = ier (19) )
  allocate ( otro (nek)    , stat = ier (20) )
  if ( any (ier .ne. 0) ) then 
    call errata (ier,'error (cgs_rala): aloca' )
  end if
  !
  ! re-incia cronometro
  call system_clock (count = h1) ! t1 = second ()
  dh = 0                         ! para luego descontar por escribir
  !
  ! precondicionador diagonal
  call diagonal_rala (sa, ifila, jcolu, valor, qii)
  where (abs (qii) < tol) qii = 1.0
  qii = uno / qii
  !
  ! inicializa
  s    = 0.0 ;    w = 0.0
  t1   = 0.0 ;   t2 = 0.0 ;   t3 = 0.0
  p1   = 0.0 ;   p2 = 0.0 ;   p3 = 0.0
  r1   = 0.0 ;   r2 = 0.0 ;   r3 = 0.0
  alfa = 0.0 ; beta = 0.0 ; gama = 0.0
  delt = 0.0 ; omic = 0.0 ; taug = 0.0
  zeta = 0.0 ; rhog = 0.0 ; rhog = 0.0
  !
  ! lazo 0
  s    = 0.0
  otro = .false. ! para forzar el calculo en cada carga
  !
  ! t1 = H * s
  call vaxqy (sa, ifila, jcolu, valor, x = s, y = t1)
  !  
  ! escalar: alfa = | b |_2
  do  l = 1, nek
   alfa (l) = sqrt (dot_product(b (l,:),b(l,:)))
   if ( alfa (l) .lt. tol )  alfa (l) = 1.0
  end do ! l
  !
  ! diferencia: t2 = b - t1
  t2 = b - t1
  !
  ! r2 = Q * t2
  call qdiag_prec (qii, x = t2, y = r2)
  !
  ! inicializa:
  p3 = 0.0
  r3 = 0.0
  r1 = r2
  !
  ! escalar: delt = (r2, r2)
  do  l = 1, nek
   delt (l) = dot_product (r2 (l,:) , r2 (l,:) )
  end do ! l
  !
  ! escalar: rhog = | delt |_2 / alfa ; beta = 0, gama = delt
  rhog = sqrt (delt) / alfa
  beta = 0.0
  gama = delt
  !
  ! lazo iterativo
  kiter: do  k = 1, nit
   !
   ! control
   sale = salida (sa, s, rhog, k, otro, nit, rum, dh, pala)
   if (sale) exit 
   !
   carga1: do  l = 1, nek
    if ( otro (l) ) cycle
    p2 (l,:) = r2 (l,:) + beta (l) * r3 (l,:)
    p1 (l,:) = r3 (l,:) + beta (l) * p3 (l,:)
    p3 (l,:) = p2 (l,:) + beta (l) * p1 (l,:)
   end do carga1
   !
   ! arreglo: t1 = H * p3
   call vaxqy (sa, ifila, jcolu, valor, x = p3, y = t1)
   !
   carga2: do  l = 1, nek
    if ( otro (l) ) cycle
    !
    ! arreglo: t2 = Q * t1
    call qdiag_prec (qii, x = t1, y = t2)
    !
    ! escalar: omic = (r1, t2)
    omic (l) = dot_product ( r1 (l,:) , t2 (l,:) )
    !
    ! escalar: taug = gama / omic
    zeta (l) = abs ( omic (l) )
    if  (zeta (l) < tol)  then
     !write (*,300) f // "final (itecgs): z_kl nulo, l = ", l
     taug (l) = 0.0
     otro (l) = .true.
     if (l .eq. nek) sale = .true.
    else
     taug (l) = gama (l) / omic (l)
    end if
   end do carga2
   if (sale) exit
   !
   carga3: do  l = 1, nek
    if ( otro (l) ) cycle
    r3 (l,:) = p2 (l,:) - taug (l) * t2 (l,:)
    p1 (l,:) = p2 (l,:) +            r3 (l,:)
    s  (l,:) = s  (l,:) + taug (l) * p1 (l,:)
   end do carga3
   !
   ! vector: t1 = H * p1
   call vaxqy (sa, ifila, jcolu, valor, x = p1, y = t1)
   !
   carga4: do  l = 1, nek
    if ( otro (l) ) cycle
    !
    ! vector: t2 = Q * t1
    call qdiag_prec (qii, x = t1, y = t2)
    !
    r2 (l,:) = r2 (l,:) - taug (l) * t2 (l,:)
    delt (l) = dot_product (r2 (l,:) , r2 (l,:) )
    rhog (l) = sqrt ( delt (l) ) / alfa (l)
    delt (l) = dot_product (r1 (l,:) , r2 (l,:) )
    beta (l) = delt (l) / gama (l)
    gama (l) = delt (l)
   end do carga4
   !
  end do kiter
  b = s
  !
  ! consulta cronometro
  call system_clock (count = h2)   ! t2 = second ()
  call timer (h2, nil, "cgs_rala")
  !
  ! dloca memoria
  ier = 0
  deallocate (    s, stat = ier  (1) )
  deallocate (    w, stat = ier  (2) )
  deallocate (   t1, stat = ier  (3) )
  deallocate (   t2, stat = ier  (4) )
  deallocate (   t3, stat = ier  (5) )
  deallocate (   r1, stat = ier  (6) )
  deallocate (   r2, stat = ier  (7) )
  deallocate (   r3, stat = ier  (8) )
  deallocate (   p1, stat = ier  (9) )
  deallocate (   p2, stat = ier (10) )
  deallocate (   p3, stat = ier (11) )
  deallocate ( alfa, stat = ier (12) )
  deallocate ( beta, stat = ier (13) )
  deallocate ( gama, stat = ier (14) )
  deallocate ( delt, stat = ier (15) )
  deallocate ( omic, stat = ier (16) )
  deallocate ( taug, stat = ier (17) )
  deallocate ( zeta, stat = ier (18) )
  deallocate ( rhog, stat = ier (19) )
  deallocate ( otro, stat = ier (20) )
  if ( any (ier .ne. 0) ) then 
    call errata (ier,'error (cgs_rala): dloca' )
  end if
  !
  ! consulta cronometro
  call system_clock (count = h2, count_rate = h4)  ! t2 = second () 
  h3 = h2 - dh                      ! delta tiempo en ciclos
  elas = h3 / h4                    ! pasa de ciclos a seg
  !
  ! estima flops; O2 = (2 p + 6 n) + k (4 p + 21 n) ; p = siza (A)
  n = sa % n              ! nro de incognitas
  p = sa % p              ! nro de coeficientes en la matriz rala
  nope1  = 2 * p +  9 * n 
  nope2  = 4 * p + 21 * n
  noper  = nope1 + nope2 * k
  mflops = noper / (1.0e6 * elas )
  !
  ! impresion
  write (*,200) f
  write (*,200) f // pala
  write (*,200) f
  write (*,300) f // "nro iterac. ;      k = ", k
  write (*,400) f // "mega flops  ; mflops = ", real (mflops) 
  write (*,400) f // "tiempo      ;    seg = ", real (elas)
  write (*,300) f // "            ;     h2 = ", h2
  write (*,300) f // "            ;     dh = ", dh
  write (*,300) f // "            ; ciclos = ", h3
  write (*,200) f // "CGS finalizado"
  write (*,200) f
  !
  ! formatos
  200 format (a)
  300 format (a, i20)
  400 format (a, 2 (1x, e24.12) )
 end subroutine
 !
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 ! control de convergencia
 ! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
 logical function salida (sa, s, ek, k, otro, nit, rum, dh, pala) 
  type (ralo_t)                 , intent (in)    :: sa 
  real    (idp), dimension (:,:), intent (in)    :: s
  real    (idp), dimension (:)  , intent (in)    :: ek
  integer (iin)                 , intent (in)    :: k
  logical      , dimension (:)  , intent (inout) :: otro
  integer (iin)                 , intent (in)    :: nit
  real    (idp)                 , intent (in)    :: rum
  integer (iin)                 , intent (inout) :: dh
  character (*)                 , intent (out)   :: pala
  character (len(sa % raiz)+4) :: arch1, arch2
  !
  real    (idp) :: emax, delt
  real    (idp) :: elas
  integer (iin) :: i1, i2, i3, i4, di
  integer (iin) :: nil, nek, i, j, l, lfila
  !
  nil = size (s,2)
  nek = siza ( size (s,1), size (ek), "salida" )
  !
  ! escribe en disco solucion POR ESTADO DE CARGA
  if (adisco) then
   !
   ! consulta cronometro para luego descontar por imprimir
   call system_clock (count = i1)  
   !
   ! archivo
   l = len_trim (sa % raiz)
   arch1 = sa % raiz (1:l) // ".sol"
   arch2 = sa % raiz (1:l) // ".his"
   !
   open (8, file = arch1, status = 'unknown') ! *.sol
   do  l = 1, nek
   do  i = 1, nil
    write (8,400)  s (l,i)
    end do ! i
   end do ! l
   close (8, status = 'keep')
  end if
  !
  ! control por convergencia individual
  where (ek .lt. rum)  otro = .true.
  !
  ! control de iteraciones por cargas multiples
  salida = .false.
  if ( all (otro) )  salida = .true.
  !
  ! control de iteraciones por convergencia
  emax = maxval (ek)
  delt = abs (emax)
  if     ( delt .lt. rum .and. k .le. nit )  then
    pala = "converge para (tol,max_iter) prefijados ..."
  elseif (  k   .ge. nit  )  then
    pala = "ATENCION: supera max iter sin converger "
  elseif ( delt .gt. rmax )  then
    pala = "ATENCION: explota la solucion   "
  elseif (salida) then
    pala = "ATENCION: sale por corte ... "
  else
    pala = "situacion desconocida ... "
  end if
  !
  ! estadistica
  if (adisco) then
    !
    ! consulta cronometro para descontar por imprimir
    call system_clock (count = i2, count_rate = i4)  
    di = i2 - i1                     ! actual    por escribir
    dh = dh + di                     ! acumulado por escribir
    i3 = i2 - dh                     ! delta tiempo desde el inicio
    elas = i3 / i4                   ! pasa de ciclos a seg
    !
    ! (re)abre archivo historico y apendiza
    open (10,file     =     arch2, &
             status   = "unknown", &
             position = "append")     !*.his
    do  l = 1, nek
     write (10,300)  k, l, ek (l), elas
    end do ! l
    close (10, status = 'keep')
    !
    ! impresion
    do  l = 1, nek
     write (*,112) f // "klor =", k, l, otro (k), real ( ek (l) )
    end do
  end if
  !
  ! formatos
  100 format (a)
  110 format (a, 2 (i8), e24.12)
  112 format (a, 2 (i8), 1x, l1, 1x, e24.12)
  120 format (a,         e24.12)
  300 format ( 2 (1x, i4), 2 (1x, e24.16), 1x, e12.4 )
  400 format ( 3 (1x, e24.16) )
 end function salida
 !
 !-----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
end module
! -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
