/******************************************************************************/
/*Auslesen einer Datei mit MPI-IO
/*Autor: H. J. Pflug, RWTH Aachen
/******************************************************************************/

#include <iostream>
using namespace std;
#include "mpi++.h"

#define BLOCK_X 2            /*Groesse des Datenblocks in X-Richtung*/
#define BLOCK_Y 2            /*Groesse des Datenblocks in Y-Richtung*/
#define BLOCK_SIZE BLOCK_X*BLOCK_Y  /*Gesamtgroesse des Datenblocks*/

/********************************MPI - Variablen ******************************/

MPI::Datatype type_file;      /*Ausschnitt in Datei*/
MPI::Datatype type_memory;    /*Ausschnitt im Speicher*/

MPI::File inputHandle;        /*Handle zur Eingabedatei*/
MPI::Cartcomm comm_cart;      /*Kommunikator fuer kartesische Topologie*/
MPI::Status status;

/********************************Andere Variablen******************************/

int size;                    /*Anzahl der Prozesse*/                   
int rank;                    /*Prozessnummer im kartesischen Gitter*/
int dims[2];                 /*Ausdehnung des Gitters in x/y-Richtung*/
bool periods[2] = {false, false};    /*Periodizitaet des Gitters (keine)*/
int myCoords[2];             /*Gitterkoordinaten des Prozesses*/
char data[BLOCK_SIZE+1];     /*Ausgelesene Daten*/

/**************************Ausfuehrbare Anweisungen****************************/

int main(int args, char** argv) {
  
  MPI::Init(args, argv);
  MPI::COMM_WORLD.Set_errhandler(MPI::ERRORS_THROW_EXCEPTIONS);
  MPI::FILE_NULL.Set_errhandler(MPI::ERRORS_THROW_EXCEPTIONS);
  
  /*Initialisierung der Topologie*/
  size = MPI::COMM_WORLD.Get_size();
  MPI::Compute_dims(size, 2, dims);        /*Prozesse zweidimensional anordnen*/
  comm_cart = MPI::COMM_WORLD.Create_cart(2, dims, periods, 1);
                                           /*Kartesische Topologie erzeugen*/
  rank = comm_cart.Get_rank();             /*Prozesskoordinaten bestimmen*/
  comm_cart.Get_coords(rank, 2, myCoords); /*            "         */

  /*Subarray-Typen initialisieren*/
  /*Beide Typen muessen nicht identisch sein, aber zueinander passen*/

    /*Dieser Typ soll aus der Datei ausgelesen werden, 13=Zeilenlaenge*/
  type_file = MPI::CHAR.Create_vector(BLOCK_Y, BLOCK_X, 13);   
    /*Dieser Typ soll in Speicher eingesetzt werden*/
  type_memory = MPI::CHAR.Create_contiguous(BLOCK_SIZE);           

  type_file.Commit();
  type_memory.Commit();

  try
  {
    inputHandle = MPI::File::Open(comm_cart, "ioinput", MPI::MODE_RDONLY, MPI::INFO_NULL);
  
    /*Sicht des Prozesses auf die Datei bestimmen. Der Prozess soll nur
      den Ausschnitt sehen, den er auch laden soll.*/
    inputHandle.Set_view(myCoords[1]*BLOCK_Y*13+myCoords[0]*BLOCK_X,
                       MPI::CHAR,
                       type_file,
                       "native",
                       MPI::INFO_NULL);
		    
    /*Eingabedaten einlesen*/
    inputHandle.Read_all(data, 1, type_memory, status);
		       
    /*Daten ausgeben*/
    data[BLOCK_SIZE]=(char) 0;
    cout << "Rank: " << myCoords[0] << "/" << myCoords[1] << ", Data: " << data << endl;
    		       
    inputHandle.Close();
  }
  catch(MPI::Exception e)
  {
    cout << "Error: number= " << e.Get_error_code() << ", description= " << e.Get_error_string() << endl;
  }
  
  MPI::Finalize();
}
