// calculating PI using one-sided communication with ..
// ! .. MPI_Win_create, MPI_Win_fence, MPI_Put
// program onesided

#include<iostream>
using namespace std;
#include"mpi++.h"

// function to integerate
#define f(a) 4.0/(1.0 + a*a)

int main(int argc, char** argv)
{
     int          myrank, nprocs, n, i, islave, master=0, count;
     MPI::Win     win;
     MPI_Status   status;
     int          ierr, resultlen, tag, assert;
     MPI::Aint    address;
     char         hostname[MPI_MAX_PROCESSOR_NAME];
     double       t0, t1, t2, fh;
     double	  *fi;
     double       PI25DP = 3.141592653589793238462643;
     double       mypi;
     double       a, h, x, pi, sum;

     // Initialize MPI
     MPI::Init(argc, argv);

     // Get my rank id and number of MPI processes
     myrank = MPI::COMM_WORLD.Get_rank();
     nprocs = MPI::COMM_WORLD.Get_size();

     // Get hostname
     MPI::Get_processor_name(hostname, resultlen);
     cout << "Rank " << myrank << " out of " << nprocs << " processes running on " << hostname << endl;

     // Synchronization point (not really necessary here)
     t0 = MPI::Wtime();
     MPI::COMM_WORLD.Barrier();
     t1 = MPI::Wtime();

  //!-----------------------------------------------------------
  //! Do work here ...

  while(true)
  {
    // The master reads an integer number form standard input and sends it to all slaves
     if (myrank == master)
     { // Master part
       cout << "Number of intervals? (0 quits)" << endl;
       cout << "->: ";
       cin >> n;
       fi = (double *)calloc(n, sizeof(double)) ;
       win = win.Create(fi, 8*n, 8, 0, MPI::COMM_WORLD);
     }
     else
       win = win.Create(fi, 0, 8, 0, MPI::COMM_WORLD);

     MPI::COMM_WORLD.Bcast(&n, 1, MPI::INT, master);
     if ((n <= 0) || (n > 10000000))
       break;

     h = 1.0/(double)n ;   // even step size h as a function of partitions

     assert = 0;
     win.Fence(assert);
     for (i = myrank+1 ;i <= n ; i += nprocs)
     { // simple approach of work sharing
        x = h * ((double)i - 0.5);
        fh = f(x);
	win.Put(&fh, 1, MPI::DOUBLE, master,i-1, 1, MPI::DOUBLE);
     }

     win.Fence(assert);

     if (myrank == master)
     {
        sum = 0.0;
	for(i=0 ;i<n ;i++)
	   sum += fi[i];
	pi = h * sum;
	cout << "pi is approximately: " << pi << " Error is " << fabs(pi-PI25DP) << endl;
        free(fi);
     }
     win.Free();
  }

  //-----------------------------------------------------------

  //Synchronization point (not really necessary here)
  MPI::COMM_WORLD.Barrier();
  t2 = MPI::Wtime();
  cout << "process " << myrank << " : " << t2-t1 << " seconds" << endl;

  // Shut Down MPI
  MPI::Finalize();

//end program onesided
}
