// simple point-to-point communication with MPI::Comm::Send and MPI::Comm::Recv

#include <iostream>
using namespace std;
#include"mpi.h"

//program ex2f

int main(int argc, char* argv[])
{
     int          myrank, nprocs, n, i, islave, master, count;
     MPI::Status  status;
     int          ierr, resultlen, tag;
     char         hostname[MPI_MAX_PROCESSOR_NAME];
     double       t0, t1, t2;

     // Initialize MPI
     MPI::Init(argc, argv);

     // Enable C++ Exceptions in MPI
      

     // Get my rank id and number of MPI processes
     myrank = MPI::COMM_WORLD.Get_rank();
     nprocs = MPI::COMM_WORLD.Get_size();

     //Get hostname
     MPI::Get_processor_name(hostname, resultlen);

     //  Synchonization point (not really necessary here)
     t0 = MPI::Wtime();
     MPI::COMM_WORLD.Barrier();
     t1 = MPI::Wtime();

     //-----------------------------------------------------------
     // Do work here ...
     // The master reads an integer number form standard input and sends it to all slaves
     if ( myrank == 0 )
     {
       // ------------------ Master part -------------------------
       cout << "input n:";
       cin >> n;

       // Send n to all workers
       tag = 10; // any message tag is fine

       for(islave=1; islave < nprocs; islave++)
       {
         try
	 {
	   MPI::COMM_WORLD.Send(&n, 1, MPI::INTEGER, islave, tag);
	 }
	 catch(MPI::Exception e)
	 {
	   cout << "master sending to slave " << islave << " causes error " << e.Get_error_code() << "(" << e.Get_error_string() << ")" << endl;
	   MPI::COMM_WORLD.Abort(99);
	 }
	 cout << "master successfully sent message to slave " << islave << endl;
       }
     }
     else
     {
       // ----------------- Slave part ---------------------------
       master = 0;
       tag = 10;
       try
       {
	 MPI::COMM_WORLD.Recv(&n, 1, MPI::INTEGER, master, tag, status);
       }
       catch(MPI::Exception e)
       {
	 cout << "master sending to slave " << islave << "causes error " << e.Get_error_code() << "(" << e.Get_error_string() << ")" << endl;
	  MPI::COMM_WORLD.Abort(99);
       }
       cout << "slave " << myrank << " successfully received a message from master" << endl;
     }


     // The slave add their rank id to this number and returns the result to the master
     if ( myrank != 0 )
     {
       // ----------------- Slave part ---------------------------
       n = n + myrank;
       master = 0;
       tag = 10 + myrank;
       try
       {
	 MPI::COMM_WORLD.Send(&n, 1, MPI::INT, master, tag);
       }
       catch(MPI::Exception e)
       {
	 cout << "slave " << islave << " sending to master causes error " << e.Get_error_code() << "(" << e.Get_error_string() << ")" << endl;
	 MPI::COMM_WORLD.Abort(99);
       }
       cout << "slave " << myrank << "successfully sent a message to master" << endl;
     }
     else
     {
       //------------------ Master part --------------------------
       for(i = 1; i < nprocs ; i++)
       {
         try
	 {
	   MPI::COMM_WORLD.Recv(&n, 1, MPI::INT, MPI::ANY_SOURCE, MPI::ANY_TAG, status);
	 }
	 catch(MPI::Exception e)
	 {
	   cout << "master receiving a message from any slave causes error " << e.Get_error_code() << "(" << e.Get_error_string() << ")" << endl;
	   MPI::COMM_WORLD.Abort(99);
	 }
	 cout << "master successfully recieved a message from any slave: " << n << endl;
	 count = status.Get_count(MPI::INT);
         cout << "source= " << status.Get_source() << ", tag= " << status.Get_tag() << ", error= " << status.Get_error() << ", count= " << count << endl;
       }
     }

     //-----------------------------------------------------------

     // Synchronization point (not really necessary here)
     MPI::COMM_WORLD.Barrier();
     t2 = MPI::Wtime();
     cout << "process " << myrank << " : " << t2-t1 << " seconds" << endl;

     // Shut Down MPI
     MPI::Finalize();
     return 0;
}
