/*
!! a simple server/client example with 
!!  MPI_Lookup_name, MPI_Comm_connect
!!  both server and client executables must be build, see server
!!  Note: run server and client on different xterms 

*/
#include <stdio.h>
#include <unistd.h>
#include <netdb.h>
#include <string.h>
#include "mpi.h"

#define DISCONNECT 101

int main(int argc, char **argv)
{
	MPI_Comm newcomm;		   	/* new (inter)communicator between server and client */
	char port_name[MPI_MAX_PORT_NAME];	/* port add = host:port */	
	char *service_name = "test_server";     /* published service name */
	int procnum, rank;
	char message[256] = "";
	char hostname[MAXHOSTNAMELEN];

	gethostname(hostname, MAXHOSTNAMELEN);

	MPI_Init(&argc, &argv);
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
        MPI_Comm_size(MPI_COMM_WORLD, &procnum);	
	printf("\nClient %d of %d: initializing MPI\n", rank+1, procnum);
	
	printf("Client %d of %d: service name -> %s\n", rank+1, procnum, service_name);
	printf("Client %d of %d: looking up port name ... ");
						 /* find port associated with a service name */
	MPI_Lookup_name(service_name, MPI_INFO_NULL, port_name);
	printf(" OK port name -> %s\n", port_name);

	printf("Client %d of %d: trying to connect server @%s ... ", rank+1, procnum);
						  /* establishes communication with server */
	MPI_Comm_connect(port_name, MPI_INFO_NULL, 0, MPI_COMM_WORLD, &newcomm);
	printf(" OK communication established\n");
						   /* send a greeting message */
	printf("Client %d of %d: sending message ...", rank+1 , procnum);
	sprintf(message, "Hello Server, message from:\nMPI proc (rank): %d\nPID: %d\n@%s\n", rank, getpid(), hostname);
	MPI_Send(message, strlen(message)+1,  MPI_CHAR, 0, 100, newcomm);
	printf(" OK message sent\n");	

	MPI_Barrier(MPI_COMM_WORLD);			/* wait for all to finish comm. */
							/* disconnect from server */
	if ( rank == 0)
	{
		sprintf(message, "Disconnect");
		printf("Client (Master) : sending disconnection request ...");
		MPI_Send(message, strlen(message)+1, MPI_CHAR, 0, DISCONNECT, newcomm);
	        printf(" OK message sent\n");
	}
	
	MPI_Comm_disconnect(&newcomm);
        MPI_Finalize();
        return 0;
}       

