// sending counterrotating messages in a ring ..
//.. MPI_Send, MPI_Recv
//program counterrotating

#include<stdio.h>
#include<stdlib.h>
#include"mpi.h"
#define TRUE 1

int	main(int  argc,char** argv)
{
     int          myrank, nprocs, n, i, islave, master=0, count;
     MPI_Status   status;
     int          ierr, resultlen, ltag, rtag;
     int          left, right;
     char         hostname[MPI_MAX_PROCESSOR_NAME];
     double       t0, t1, t2;
     char	  *lbytes, *rbytes, *bytes;
     // character (LEN=1), allocatable, dimension(:) :: rsbytes, lsbytes, rrbytes, lrbytes, lbytes, rbytes

     // Initialize MPI
     MPI_Init(&argc, &argv);

     // Get my rank id and number of MPI processes
     MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
     MPI_Comm_size(MPI_COMM_WORLD, &nprocs);

     t0 = MPI_Wtime() ;

  //!-----------------------------------------------------------
  //! Do work here ...

  left  = (nprocs + myrank - 1) % nprocs;
  right =  (myrank + 1) % nprocs;
  rtag = 23;
  ltag = 24;
  printf("myrank= %d left neighbor= %d rigth neighbor= %d\n", myrank,left,right);
  MPI_Barrier ( MPI_COMM_WORLD );

  while(TRUE)
  {
  // The master reads an integer number form standard input and sends it to all slaves
    if ( myrank == master )
    { //----------------------- Master part-----------------------------------
      printf("Number of bytes ? (0 quits, max %d = 128MB\n",1024*1024*128);
      scanf("%d",&n);
    }

    MPI_Bcast ( &n, 1, MPI_INT, master, MPI_COMM_WORLD);
    if ((n <= 0) || (n > 1024*1024*128 ))
	break;

    if (myrank == master )
    { //-------------- Master part -------------------------------------------

      lbytes = (char *) calloc(n,sizeof(char)) ;
      rbytes = (char *) calloc(n,sizeof(char)) ;
      bytes = (char *) calloc(n,sizeof(char)) ;

      for(i=0;i<n;i++)
      {
	lbytes[i] = 'L';
        rbytes[i] = 'R';
	bytes[i] = ' ';
      }


      t1 = MPI_Wtime();
      MPI_Send ( lbytes, n, MPI_BYTE, left, ltag, MPI_COMM_WORLD);
      MPI_Send ( rbytes, n, MPI_BYTE, right, rtag, MPI_COMM_WORLD);

      MPI_Recv ( bytes, n, MPI_BYTE, MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &status);
      if ( status.MPI_TAG == rtag )
      {
        //print *, count(rbytes .eq. "R")
        for(i=0;i<n;i++)
	    bytes[i] = ' ';
	MPI_Recv ( lbytes, n, MPI_BYTE, right, ltag, MPI_COMM_WORLD, &status);
	//print *, count(lbytes .eq. "L")
      }
      else if ( status.MPI_TAG == ltag )
      {
        //print *, count(lbytes .eq. "L")
       	for(i=0;i<n;i++)
	    bytes[i] = ' ';
	MPI_Recv ( rbytes, n, MPI_BYTE, left, rtag, MPI_COMM_WORLD, &status);
	//print *, count(rbytes .eq. "R")
      }
      else
      	MPI_Abort ( MPI_COMM_WORLD, 99);

      t2 = MPI_Wtime();
      printf(" message length %d bytes transfer rate %lf  MB/s\n", n,1e-6 * (double)(2 * n * nprocs) / (t2-t1));

      free(lbytes);free(rbytes);free(bytes);
    }
    else
    {//--------------------------------------- Slave part------------------------------------------

      bytes = (char *) calloc(n,sizeof(char)) ;

      MPI_Recv ( bytes, n, MPI_BYTE, MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &status);
      if ( status.MPI_TAG == rtag )
      {
        MPI_Send ( bytes, n, MPI_BYTE, right, rtag, MPI_COMM_WORLD);
        MPI_Recv ( bytes, n, MPI_BYTE, right, ltag, MPI_COMM_WORLD, &status);
        MPI_Send ( bytes, n, MPI_BYTE, left, ltag, MPI_COMM_WORLD);
      }
      else if ( status.MPI_TAG == ltag )
      {
        MPI_Send ( bytes, n, MPI_BYTE, left, ltag, MPI_COMM_WORLD);
        MPI_Recv ( bytes, n, MPI_BYTE, left, rtag, MPI_COMM_WORLD, &status);
        MPI_Send ( bytes, n, MPI_BYTE, right, rtag, MPI_COMM_WORLD);
      }
      else
      	MPI_Abort ( MPI_COMM_WORLD, 99);


      free(bytes);

    }

  }


  //-----------------------------------------------------------

  //Shut Down MPI
  MPI_Finalize();

//end program counterrotating
}
