/*in general the optimizing Fortran compiler may cause problems ..
!! .. when data which is supposed to be received by asynchronous transfers
!! .. is kept in registers.
!! .. A common trick is to call MPI_Get_address before the received value
!! .. is used.*/
//program fortranproblem

#include<stdio.h>
#include<math.h>
#include"mpi.h"
#define TRUE 1

int	main(int  argc,char** argv)
{
     int          myrank, nprocs, n, i, islave, master=0, count;
     MPI_Win      win;
     MPI_Status   status;
     int          ierr, resultlen, tag;
     MPI_Aint     address;
     char         hostname[MPI_MAX_PROCESSOR_NAME];
     double       t0, t1, t2, fh;
     double	  fi;
     /* why do we need this ?
     double       PI25DP = 3.141592653589793238462643;
     double       mypi;
     double       f, a, h, x, pi, sum;*/
   
     // Initialize MPI
     MPI_Init(&argc, &argv);

     // Get my rank id and number of MPI processes
     MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
     MPI_Comm_size(MPI_COMM_WORLD, &nprocs);

     // Get hostname
     MPI_Get_processor_name(hostname, &resultlen);
     printf("Rank %d out of %d processes running on %s\n",myrank, nprocs, hostname);

     // Synchronization point (not really necessary here)
     t0 = MPI_Wtime();
     MPI_Barrier (MPI_COMM_WORLD);
     t1 = MPI_Wtime();

  //!-----------------------------------------------------------
  //! Do work here ...

  while(TRUE)
  {
    if(myrank == master)
    {
      MPI_Win_create ( &fi, 8, 8, 0, MPI_COMM_WORLD, &win);
      fi = 999.0;
      MPI_Win_fence ( 0, win);
      MPI_Win_fence ( 0, win);
      printf("%lf\n",fi);
      if( fi != 777.0 )
         MPI_Abort ( MPI_COMM_WORLD, 99);
    }
    else
    {
      fh = 777.0;
      MPI_Win_create ( &fi, 0, 8, 0, MPI_COMM_WORLD, &win);
      MPI_Win_fence ( 0, win);
      MPI_Put ( &fh, 1, MPI_DOUBLE, master, 0, 1, MPI_DOUBLE, win);
      MPI_Win_fence ( 0, win);
    }
    MPI_Win_free ( &win );

    if ( myrank == master )
    { // Master part
      MPI_Win_create ( &fi, 8, 8, 0, MPI_COMM_WORLD, &win);
      fi = 999.0;
      MPI_Win_fence ( 0, win);
      MPI_Win_fence ( 0, win);
      MPI_Get_address ( &fi, &address);
      printf("%lf\n",fi);
    }
    else
    {
      fh = 777.0;
      MPI_Win_create ( &fi, 0, 8, 0, MPI_COMM_WORLD, &win);
      MPI_Win_fence ( 0, win);
      MPI_Put ( &fh, 1, MPI_DOUBLE, master, 0, 1, MPI_DOUBLE, win);
      MPI_Win_fence ( 0, win);
    }
    MPI_Win_free ( &win);
  }
  //-------------------------------------------------------------------------------------

  // Synchronization point (not really necessary here)
  MPI_Barrier (MPI_COMM_WORLD);
  t2 = MPI_Wtime();
  printf("process %d : %lf seconds",myrank, t2-t1);

  // Shut Down MPI
  MPI_Finalize();

//end program fortranproblem
}
