/******************************************************************************/
/*Auslesen einer Datei mit MPI-IO
/*Autor: H. J. Pflug, RWTH Aachen
/******************************************************************************/

#include <stdio.h>
#include "mpi.h"

#define BLOCK_X 2            /*Groesse des Datenblocks in X-Richtung*/
#define BLOCK_Y 2            /*Groesse des Datenblocks in Y-Richtung*/
#define BLOCK_SIZE BLOCK_X*BLOCK_Y  /*Gesamtgroesse des Datenblocks*/

/********************************MPI - Variablen ******************************/

MPI_Datatype type_file;      /*Ausschnitt in Datei*/
MPI_Datatype type_memory;    /*Ausschnitt im Speicher*/

MPI_File inputHandle;        /*Handle zur Eingabedatei*/
MPI_Comm comm_cart;          /*Kommunikator fuer kartesische Topologie*/
MPI_Status status;

/********************************Andere Variablen******************************/

int size;                    /*Anzahl der Prozesse*/                   
int rank;                    /*Prozessnummer im kartesischen Gitter*/
int dims[2];                 /*Ausdehnung des Gitters in x/y-Richtung*/
int periods[2] = {(char)0, (char)0};    /*Periodizitaet des Gitters (keine)*/
int myCoords[2];             /*Gitterkoordinaten des Prozesses*/
char data[BLOCK_SIZE+1];     /*Ausgelesene Daten*/

/**************************Ausfuehrbare Anweisungen****************************/

int main(int args, char** argv) {
  
  MPI_Init(&args, &argv);
  
  /*Initialisierung der Topologie*/
  MPI_Comm_size(MPI_COMM_WORLD, &size);
  MPI_Dims_create(size, 2, dims);          /*Prozesse zweidimensional anordnen*/
  MPI_Cart_create(MPI_COMM_WORLD, 2, dims, periods, 1, &comm_cart);
                                           /*Kartesische Topologie erzeugen*/
  MPI_Comm_rank(comm_cart, &rank);         /*Prozesskoordinaten bestimmen*/
  MPI_Cart_coords(comm_cart, rank, 2, myCoords); /*            "         */

  /*Subarray-Typen initialisieren*/
  /*Beide Typen muessen nicht identisch sein, aber zueinander passen*/

    /*Dieser Typ soll aus der Datei ausgelesen werden, 13=Zeilenlaenge*/
  MPI_Type_vector(BLOCK_Y, BLOCK_X, 13, MPI_CHAR, &type_file);   
    /*Dieser Typ soll in Speicher eingesetzt werden*/
  MPI_Type_contiguous(BLOCK_SIZE, MPI_CHAR, &type_memory);           

  MPI_Type_commit(&type_file);
  MPI_Type_commit(&type_memory);

  MPI_File_open(comm_cart, "ioinput", MPI_MODE_RDONLY, MPI_INFO_NULL,
                &inputHandle);
  
  /*Sicht des Prozesses auf die Datei bestimmen. Der Prozess soll nur
  den Ausschnitt sehen, den er auch laden soll.*/
  MPI_File_set_view(inputHandle,
                    myCoords[1]*BLOCK_Y*13+myCoords[0]*BLOCK_X,
                    MPI_CHAR,
                    type_file,
                    "native",
                    MPI_INFO_NULL);
		    
  /*Eingabedaten einlesen*/
  MPI_File_read_all(inputHandle, data, 1, type_memory, &status);
		       
  /*Daten ausgeben*/
  data[BLOCK_SIZE]=(char) 0;
  printf("Rank: %d/%d, Data: %s\n",myCoords[0],myCoords[1], data);
    		       
  MPI_File_close(&inputHandle);
  MPI_Finalize();
}
