// calculating PI using one-sided communication with ..
//! .. MPI_Win_create, MPI_Win_fence, MPI_Put
//program onesided

#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include"mpi.h"
// function to integerate
#define f(a) 4.0/(1.0 + a*a)
#define TRUE 1

int	main(int  argc,char** argv)
{
     int          myrank, nprocs, n, i, islave, master=0, count;
     MPI_Win      win;
     MPI_Status   status;
     int          ierr, resultlen, tag, assert;
     MPI_Aint     address;
     char         hostname[MPI_MAX_PROCESSOR_NAME];
     double       t0, t1, t2, fh;
     double	  *fi;
     double       PI25DP = 3.141592653589793238462643;
     double       mypi;
     double       a, h, x, pi, sum;

     // Initialize MPI
     MPI_Init(&argc, &argv);

     // Get my rank id and number of MPI processes
     MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
     MPI_Comm_size(MPI_COMM_WORLD, &nprocs);

     // Get hostname
     MPI_Get_processor_name(hostname, &resultlen);
     printf("Rank %d out of %d processes running on %s\n",myrank, nprocs, hostname);

     // Synchronization point (not really necessary here)
     t0 = MPI_Wtime();
     MPI_Barrier (MPI_COMM_WORLD);
     t1 = MPI_Wtime();

  //!-----------------------------------------------------------
  //! Do work here ...

  while(TRUE)
  {
    // The master reads an integer number form standard input and sends it to all slaves
     if ( myrank == master )
     { // Master part
       printf("Number of intervals ? (0 quits): ");
       scanf("%d",&n);
       fi= (double *) calloc(n,sizeof(double)) ;
       MPI_Win_create ( fi, 8*n, 8, 0, MPI_COMM_WORLD, &win);
     }
     else
       MPI_Win_create ( fi, 0, 8, 0, MPI_COMM_WORLD, &win);

     MPI_Bcast ( &n, 1, MPI_INT, master, MPI_COMM_WORLD);
     if( (n <= 0) || (n > 10000000) )
       break;

     h = 1.0/(double)n ;   // even step size h as a function of partitions

     assert = 0;
     MPI_Win_fence ( assert, win);
     for(i = myrank+1 ;i <= n ; i += nprocs)
     {// simple approach of work sharing
        x = h * ((double)i - 0.5);
        fh = f(x);
	MPI_Put ( &fh, 1, MPI_DOUBLE, master,i-1, 1, MPI_DOUBLE, win);
     }

     MPI_Win_fence ( assert, win);

     if ( myrank == master )
     {
        sum = 0.0;
	for(i=0 ;i<n ;i++)
	   sum += fi[i];
	pi = h * sum;
	printf("pi is approximately: %18.16f Error is %18.16f\n",pi,fabs(pi-PI25DP));
        free(fi);
     }
     MPI_Win_free ( &win );

  }

  //-----------------------------------------------------------

  //Synchronization point (not really necessary here)
  MPI_Barrier (MPI_COMM_WORLD);
  t2 = MPI_Wtime();
  printf("process %d : %lf seconds\n",myrank, t2-t1);

  // Shut Down MPI
  MPI_Finalize();

//end program onesided
}
