/*
!! a simple server/client example with 
!!  MPI_Open_port, MPI_Publish_name, MPI_Comm_accept, MPI_Comm_disconnect
!!  both server and client executables must be build, see client
!!  Note: run server and client on different xterms 
*/
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <stdlib.h>
#include "mpi.h"

#define DISCONNECT 101

MPI_Comm newcomm;                       /* new communicator between server and client */
char port_name[MPI_MAX_PORT_NAME];      /* addres = host:prot */
char *service_name = "test_server";     /* published service name */


void quit(int sig)   /* signal hanler, see line 54 */
{
	if (newcomm)
		MPI_Comm_disconnect(&newcomm);

	MPI_Unpublish_name(service_name, MPI_INFO_NULL, port_name);
	MPI_Finalize();
	exit (0);
}
	

int main(int argc, char **argv)
{
	MPI_Status status;
	
	int procnum, rank;
	char message[256] ="";
	

	printf("\nSever: a simple server example\n");
	printf("Server: ^C to end\n");
	printf("Server: initializing MPI\n");
	MPI_Init(&argc, &argv);
	MPI_Comm_rank(MPI_COMM_WORLD, &rank);
	MPI_Comm_size(MPI_COMM_WORLD, &procnum);	

	MPI_Open_port(MPI_INFO_NULL, port_name);	/* establishes a network address to
							   accept connections from clients */

	printf("Server: port name -> %s\n", port_name);	
		
							/* publish the service name thats 
							   associated with the port */
	MPI_Publish_name(service_name, MPI_INFO_NULL, port_name);
	printf("Server: service name published -> %s\n", service_name);
	
						/* signal handler : ^C to terminate server:
					   	unpublische the service name 
					   	then MPI_Finalize()  */
	signal(SIGINT, quit);

						/*establishes communication with a client,
						  return an intercommunicator that allows comm. 
						  with the clients */
	while (1)
	{
		printf("\nServer: accepting connections ... \n"); 
		MPI_Comm_accept(port_name, MPI_INFO_NULL, 0, MPI_COMM_WORLD, &newcomm);
	
		while (1)
		{	
			MPI_Recv(message, 256, MPI_CHAR, MPI_ANY_SOURCE, MPI_ANY_TAG, newcomm, &status);
			printf("\nServer: message received ->\n%s", message);

							/* check whether clinet wants
								disconnect */
		
			if ( status.MPI_TAG == DISCONNECT )	 	
			{
				printf("\nServer: disconnection request from clinet ->");
				MPI_Comm_disconnect(&newcomm);
				printf(" disconnected\n");
				break;
			}
		}
	}
	
}	
