// simple point-to-point communication with MPI_Send and MPI_Recv //
// program ex2f

#include<stdio.h>
#include"mpi.h"

int	main(int  argc,char** argv)
{
     int          myrank, nprocs, n, i, islave, master, count;
     MPI_Status   status;
     int          ierr, resultlen, tag;
     char         hostname[MPI_MAX_PROCESSOR_NAME];
     double       t0, t1, t2;

     /* Initialize MPI  */
     MPI_Init(&argc, &argv);

     /* Get my rank id and number of MPI processes */
     MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
     MPI_Comm_size(MPI_COMM_WORLD, &nprocs);

     //Get hostname
     MPI_Get_processor_name(hostname, &resultlen);

     //  Synchonization point (not really necessary here)
     t0 = MPI_Wtime();
     MPI_Barrier (MPI_COMM_WORLD);
     t1 = MPI_Wtime();

/*-----------------------------------------------------------
     Do work here ... */

  // The master reads an integer number form standard input and sends it to all slaves
  if ( myrank == 0 )
  {
    // ------------------ Master part -----------------------

    printf("input n:");
    scanf("%d",&n);

    // Send n to all workers
    tag = 10; // any message tag is fine

    for(islave=1 ; islave<nprocs ; islave++)
    {
           ierr = MPI_Send(&n, 1, MPI_INT, islave, tag, MPI_COMM_WORLD);

	   if(ierr == 0)
	   	printf("master successfully sent message to slave %d\n",islave);
	   else
	   {
	   	printf("master sending to slave %d causes error %d\n", islave,ierr);
                MPI_Abort ( MPI_COMM_WORLD, 99 );
	   }
    }

  }
  else
  {
    // ----------------- Slave part ------------------------------------------------------
    master = 0;
    tag = 10;
    ierr = MPI_Recv (&n, 1, MPI_INT, master, tag, MPI_COMM_WORLD, MPI_STATUS_IGNORE );  // MPI2

    if(ierr == 0)
    	printf("slave %d successfully received a message from master\n",myrank);
    else
    {
    	printf("master sending to slave %d causes error %d\n ", islave,ierr);
	MPI_Abort ( MPI_COMM_WORLD, 99 );
    }
  }


  // The slave add their rank id to this number and returns the result to the master

  if ( myrank != 0 )
  {
    // ----------------- Slave part ------------------------------------------------------
    n = n + myrank;
    master = 0;
    tag = 10 + myrank;
    ierr = MPI_Send (&n, 1, MPI_INT, master, tag, MPI_COMM_WORLD);

    if(ierr == 0)
    	printf("slave %d successfully sent a message to master\n",myrank);
    else
    {
    	printf("slave %d sending to master causes error %d", islave,ierr);
	MPI_Abort ( MPI_COMM_WORLD, 99 );
    }
  }
  else
  {
    //------------------ Master part -------------------------------------------------------
    for(i = 1; i < nprocs ; i++)
    {
    	ierr = MPI_Recv (&n, 1, MPI_INT, MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, &status);

	if(ierr == 0)
	{
	   printf("master successfully recieved a message from any slave: %d\n",n);
	   MPI_Get_count(&status, MPI_INT, &count);
	   printf("source= %d , tag= %d , error= %d ,count= %d\n",status.MPI_SOURCE,status.MPI_TAG,status.MPI_ERROR,count);
	}
	else
	{
	   printf("master receiving a message from any slave causes error %d\n", ierr);
	   MPI_Abort ( MPI_COMM_WORLD, 99 );
	}
    }
  }

  /*----------------------------------------------------------- */

  /* Synchronization point (not really necessary here) */
  MPI_Barrier (MPI_COMM_WORLD);
  t2 = MPI_Wtime();
  printf("process %d : %lf seconds\n", myrank,t2-t1);

  // Shut Down MPI
  MPI_Finalize();

//end program ex2f
}
