/*
!! calculating PI using collective communication with ..
!! .. more sophiticated work sharing
*/

//program worksharing
#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include"mpi.h"
// function to integerate
#define f(a) 4.0/(1.0 + a*a)
#define TRUE 1

int	main(int  argc,char** argv)
{
     int          myrank, nprocs, n, i, islave, master=0, count;
     int          chunk, rest, is, ie, ilen;
     MPI_Status   status;
     int          ierr, resultlen, tag;
     char         hostname[MPI_MAX_PROCESSOR_NAME];
     double       t0, t1, t2;
     double       PI25DP = 3.141592653589793238462643;
     double       mypi;
     double       a, h, s, x, pi, sum;

     // Initialize MPI
     MPI_Init(&argc, &argv);

     // Get my rank id and number of MPI processes
     MPI_Comm_rank(MPI_COMM_WORLD, &myrank);
     MPI_Comm_size(MPI_COMM_WORLD, &nprocs);

     // Get hostname
     MPI_Get_processor_name(hostname, &resultlen);
     printf("Rank %d out of %d processes running on %s\n",myrank, nprocs, hostname);

     // Synchronization point (not really necessary here)
     t0 = MPI_Wtime();
     MPI_Barrier(MPI_COMM_WORLD);
     t1 = MPI_Wtime();

  //!-----------------------------------------------------------
  //! Do work here ...

  while(TRUE)
  {
    // The master reads an integer number form standard input and sends it to all slaves
    if ( myrank == master )
    {   // Master part
    	   printf("Number of intervals ? (0 quits): \n");
           scanf("%d",&n);
    }

    MPI_Bcast (&n, 1, MPI_INT, master, MPI_COMM_WORLD);
    if ( (n <= 0) || (n > 10000000) )
    	break;

     h   = 1.0/ (double) n;   // even step size h as a function of partions
     sum = 0.0;              // zeroise sum

     // improved approach of work sharing
     rest = n % nprocs ;
     chunk = ( n - rest ) / nprocs ;
     if ( myrank < rest )
     {
       is = myrank * ( chunk + 1 ) ;
       ie = is + chunk ;
     }
     else
     {
       is = myrank * chunk + rest ;
       ie = is + chunk - 1 ;
     }
     ilen = ie - is + 1 ;

     printf("nprocs= %d myrank = %d is = %d ie = %d ilen = %d\n",nprocs,myrank,is,ie,ilen);

     for(i=is ;i<=ie; i++)
     {
        x = h * ((double)i - 0.5) ;
        sum +=  f(x) ;
     }
     mypi = h * sum ;

     MPI_Reduce(&mypi,&pi,1,MPI_DOUBLE,MPI_SUM,0,MPI_COMM_WORLD); // collect all the sums

     if ( myrank == 0 )
	printf("pi is approximately: %18.16f Error is %18.16f\n",pi,fabs(pi-PI25DP));
  }

  //-------------------------------------------------------------------------------------

  //Synchronization point (not really necessary here)
  MPI_Barrier (MPI_COMM_WORLD);
  t2 = MPI_Wtime();
  printf("process %d : %lf seconds\n", myrank,t2-t1);

  //Shut Down MPI
  MPI_Finalize();

  //end program worksharing
}
