!! sending counterrotating messages in a ring using asynchronous transfers..
!! .. MPI_Isend, MPI_Irecv
program asynchronous

  implicit none

  include 'mpif.h'

  integer   ::         myrank, nprocs, n, i, islave, master, count
  integer   ::         status(MPI_STATUS_SIZE,4)
  integer   ::         ierr, resultlen, ltag, rtag, reqid(4), ix
  integer   ::         left, right
  character (LEN=MPI_MAX_PROCESSOR_NAME) :: hostname
  real*8    ::         t0, t1, t2
  character (LEN=1), allocatable, dimension(:) :: rsbytes, lsbytes, rrbytes, lrbytes, lbytes, rbytes
  
  ! Initialize MPI
  Call MPI_Init(Ierr)

  ! Get my rank id and number of MPI processes
  call MPI_Comm_Rank(MPI_COMM_WORLD, myrank, ierr)
  call MPI_Comm_Size(MPI_COMM_WORLD, nprocs, ierr)
  
  t0 = MPI_Wtime()

  !-----------------------------------------------------------
  ! Do work here ...


  left = mod ( nprocs + myrank - 1, nprocs )
  right = mod ( myrank + 1, nprocs )
  rtag = 23
  ltag = 24
  write (*,*) "myrank=", myrank, " left neighbor=", left,  " rigth neighbor=", right
  call MPI_Barrier ( MPI_COMM_WORLD, ierr )

  do
  
    ! The master reads an integer number form standard input and sends it to all slaves
    if ( myrank .eq. master ) then ! Master part
      write(*,*) 'Number of bytes ? (0 quits, max ', 1024*1024*128, '=128 MB): '
      read (*,*) n
    end if
    call MPI_Bcast ( n, 1, MPI_INTEGER, master, MPI_COMM_WORLD, ierr )
    if ( n .le. 0 .or. n > 1024*1024*128 ) exit

    if ( myrank .eq. 0 ) then ! Master part

      allocate ( lsbytes(1:n), rsbytes(1:n), lrbytes(1:n), rrbytes(1:n) )
      lsbytes="L"
      rsbytes="R"
      lrbytes= " "
      rrbytes= " "

      t1 = MPI_Wtime()
      call MPI_Isend ( lsbytes, n, MPI_BYTE, left, ltag, MPI_COMM_WORLD, reqid(1), ierr )
      call MPI_Isend ( rsbytes, n, MPI_BYTE, right, rtag, MPI_COMM_WORLD, reqid(2), ierr )
      call MPI_Irecv ( lrbytes, n, MPI_BYTE, right, ltag, MPI_COMM_WORLD, reqid(3), ierr ) 
      call MPI_Irecv ( rrbytes, n, MPI_BYTE, left, rtag, MPI_COMM_WORLD, reqid(4), ierr ) 
      call MPI_Waitall ( 4, reqid, status, ierr ) 
      
      t2 = MPI_Wtime()
      print *, count(rrbytes .eq. "R"), count(lrbytes .eq. "L")
      write (*,*) "message length ", n, " bytes   transfer rate ", 1e-6 * 2 * n * nprocs / (t2-t1), " MB/s"
      
      deallocate ( lsbytes, rsbytes, lrbytes, rrbytes )

    else ! Slave part
    
      allocate ( lbytes(1:n), rbytes(1:n) )
      lbytes= " "
      rbytes= " "
      call MPI_Irecv ( lbytes, n, MPI_BYTE, right, ltag, MPI_COMM_WORLD, reqid(1), ierr ) 
      call MPI_Irecv ( rbytes, n, MPI_BYTE, left, rtag, MPI_COMM_WORLD, reqid(2), ierr ) 
      call MPI_Waitany ( 2, reqid, ix, status, ierr )
      
      if ( ix .eq. 1 ) then
        call MPI_Isend ( lbytes, n, MPI_BYTE, left, ltag, MPI_COMM_WORLD, reqid(3), ierr )
	call MPI_Wait ( reqid(2), status, ierr )
        call MPI_Isend ( rbytes, n, MPI_BYTE, right, rtag, MPI_COMM_WORLD, reqid(4), ierr )
	call MPI_Wait ( reqid(4), status, ierr )
      else if ( ix .eq. 2 ) then
        call MPI_Isend ( rbytes, n, MPI_BYTE, right, rtag, MPI_COMM_WORLD, reqid(4), ierr )
	call MPI_Wait ( reqid(1), status, ierr )
        call MPI_Isend ( lbytes, n, MPI_BYTE, left, ltag, MPI_COMM_WORLD, reqid(3), ierr )
	call MPI_Wait ( reqid(3), status, ierr )
      else 
      	call MPI_Abort ( MPI_COMM_WORLD, 99, ierr )      
      end if
    
      deallocate ( rbytes, lbytes )

    end if

  end do

  !-----------------------------------------------------------

  ! Shut Down MPI
  call MPI_Finalize(ierr )

end program asynchronous
