!! in general the optimizing Fortran compiler may cause problems ..
!! .. when data which is supposed to be received by asynchronous transfers 
!! .. is kept in registers.
!! .. A common trick is to call MPI_Get_address before the received value
!! .. is used.
program fortranproblem

  implicit none

  include 'mpif.h'

  integer   ::         myrank, nprocs, n, i, islave, master=0, count, win
  integer   ::         status(MPI_STATUS_SIZE)
  integer   ::         ierr, resultlen, tag, address
  character (LEN=MPI_MAX_PROCESSOR_NAME) :: hostname
  real*8    ::         t0, t1, t2, fh
  real*8    ::         fi
  real*8, parameter :: PI25DP = 3.141592653589793238462643d0
  ! integer chunk, ierr, ilen, rest, is, ie
  real*8 :: mypi  
  double precision f, a, h, sum, x, pi

  f(a) = 4.d0 / (1.d0 + a*a) ! function to integrate
  
  ! Initialize MPI
  Call MPI_Init(Ierr)

  ! Get my rank id and number of MPI processes
  call MPI_Comm_Rank(MPI_COMM_WORLD, myrank, ierr)
  call MPI_Comm_Size(MPI_COMM_WORLD, nprocs, ierr)
  
  ! Get hostname
  call MPI_Get_processor_name(hostname, resultlen, ierr)
  write (*,*) "Rank ", myrank, " out of ", nprocs,  &
    " processes running on ", hostname(1:index(hostname,".")-1)

  
  ! Synchonization point (not really necessary here)
  t0 = MPI_Wtime()
  call MPI_Barrier (MPI_COMM_WORLD, ierr)
  t1 = MPI_Wtime()

  !-----------------------------------------------------------
  ! Do work here ...

  do
    if ( myrank .eq. master ) then ! Master part
      call MPI_Win_create ( fi, 8, 8, 0, MPI_COMM_WORLD, win, ierr )
      fi = 999
      call MPI_Win_fence ( 0, win, ierr )
      call MPI_Win_fence ( 0, win, ierr )
      write (*,*) fi
      if ( fi .ne. 777 ) then
         call MPI_Abort ( MPI_COMM_WORLD, 99, ierr )
      end if
    else
      fh = 777
      call MPI_Win_create ( fi, 0, 8, 0, MPI_COMM_WORLD, win, ierr )
      call MPI_Win_fence ( 0, win, ierr )
      call MPI_Put ( fh, 1, MPI_DOUBLE_PRECISION, master, 0, 1, MPI_DOUBLE_PRECISION, win, ierr)
      call MPI_Win_fence ( 0, win, ierr )
    end if
    call MPI_Win_free ( win, ierr )
     
    if ( myrank .eq. master ) then ! Master part
      call MPI_Win_create ( fi, 8, 8, 0, MPI_COMM_WORLD, win, ierr )
      fi = 999
      call MPI_Win_fence ( 0, win, ierr )
      call MPI_Win_fence ( 0, win, ierr )
      call MPI_Get_address ( fi, address, ierr )
      write (*,*) fi
    else
      fh = 777
      call MPI_Win_create ( fi, 0, 8, 0, MPI_COMM_WORLD, win, ierr )
      call MPI_Win_fence ( 0, win, ierr )
      call MPI_Put ( fh, 1, MPI_DOUBLE_PRECISION, master, 0, 1, MPI_DOUBLE_PRECISION, win, ierr)
      call MPI_Win_fence ( 0, win, ierr )
    end if
    call MPI_Win_free ( win, ierr )
  end do
  !-----------------------------------------------------------

  ! Synchronization point (not really necessary here)
  call MPI_Barrier (MPI_COMM_WORLD, ierr)
  t2 = MPI_Wtime()
  write (*,*) "process ", myrank, ": ", t2-t1, " seconds"

  ! Shut Down MPI
  call MPI_Finalize(ierr )

end program fortranproblem
