!! calculating PI using one-sided communication with .. 
!! .. MPI_Win_create, MPI_Win_fence, MPI_Put
program onesided

  implicit none

  include 'mpif.h'

  integer   ::         myrank, nprocs, n, i, islave, master=0, count, win
  integer   ::         status(MPI_STATUS_SIZE)
  integer   ::         ierr, resultlen, tag, assert
  character (LEN=MPI_MAX_PROCESSOR_NAME) :: hostname
  real*8    ::         t0, t1, t2, fh
  real*8, dimension(:), allocatable :: fi
  real*8, parameter :: PI25DP = 3.141592653589793238462643d0
  ! integer chunk, ierr, ilen, rest, is, ie
  real*8 :: mypi  
  double precision f, a, h, sum, x, pi

  f(a) = 4.d0 / (1.d0 + a*a) ! function to integrate
  
  ! Initialize MPI
  Call MPI_Init(Ierr)

  ! Get my rank id and number of MPI processes
  call MPI_Comm_Rank(MPI_COMM_WORLD, myrank, ierr)
  call MPI_Comm_Size(MPI_COMM_WORLD, nprocs, ierr)
  
  ! Get hostname
  call MPI_Get_processor_name(hostname, resultlen, ierr)
  write (*,*) "Rank ", myrank, " out of ", nprocs,  &
    " processes running on ", hostname(1:index(hostname,".")-1)

  
  ! Synchonization point (not really necessary here)
  t0 = MPI_Wtime()
  call MPI_Barrier (MPI_COMM_WORLD, ierr)
  t1 = MPI_Wtime()

  !-----------------------------------------------------------
  ! Do work here ...

  do
  
    ! The master reads an integer number form standard input and sends it to all slaves
    if ( myrank .eq. master ) then ! Master part
      write(*,*) 'Number of intervals ? (0 quits): '
      read (*,*) n
      allocate ( fi(n) )
      call MPI_Win_create ( fi, 8*n, 8, 0, MPI_COMM_WORLD, win, ierr )
    else
      call MPI_Win_create ( fi, 0, 8, 0, MPI_COMM_WORLD, win, ierr )
    end if
    call MPI_Bcast ( n, 1, MPI_INTEGER, master, MPI_COMM_WORLD, ierr )
    if ( n .le. 0 .or. n > 10000000 ) exit
          
     h = 1.0d0/dble(n)   ! even step size h as a function of partitions

     assert = 0
     call MPI_Win_fence ( assert, win, ierr )

     do i = myrank+1, n, nprocs  ! simple approach of work sharing
        x = h * (dble(i) - 0.5d0)
        fh = f(x)
        call MPI_Put ( fh, 1, MPI_DOUBLE_PRECISION, master, i-1, 1, MPI_DOUBLE_PRECISION, win, ierr)
     end do

     call MPI_Win_fence ( assert, win, ierr )
 
     if ( myrank .eq. 0 ) then
        sum = 0.0d0
	do i = 1, n
	   sum = sum + fi(i)
	end do
	pi = h * sum
        write(*,'(a,F18.16,a,F18.16)') 'pi is approximately: ',pi, &
             ' Error is: ', abs(pi - PI25DP)
	deallocate ( fi )
     end if
     call MPI_Win_free ( win, ierr )
    
  end do
     
  !-----------------------------------------------------------

  ! Synchronization point (not really necessary here)
  call MPI_Barrier (MPI_COMM_WORLD, ierr)
  t2 = MPI_Wtime()
  write (*,*) "process ", myrank, ": ", t2-t1, " seconds"

  ! Shut Down MPI
  call MPI_Finalize(ierr )

end program onesided
