!! measuring the bandwidth when sending messages in a ring ..
!! .. MPI_Send, MPI_Recv
program ring

  implicit none

  include 'mpif.h'

  integer   ::         myrank, nprocs, n, i, islave, master, count
  integer   ::         status(MPI_STATUS_SIZE)
  integer   ::         ierr, resultlen, tag
  integer   ::         left, right
  character (LEN=MPI_MAX_PROCESSOR_NAME) :: hostname
  real*8    ::         t0, t1, t2
  character (LEN=1), allocatable, dimension(:) :: bytes
  
  ! Initialize MPI
  Call MPI_Init(Ierr)

  ! Get my rank id and number of MPI processes
  call MPI_Comm_Rank(MPI_COMM_WORLD, myrank, ierr)
  call MPI_Comm_Size(MPI_COMM_WORLD, nprocs, ierr)
  
  t0 = MPI_Wtime()

  !-----------------------------------------------------------
  ! Do work here ...


  left = mod ( nprocs + myrank - 1, nprocs )
  right = mod ( myrank + 1, nprocs )
  tag = 23
  write (*,*) "myrank=", myrank, " left neighbor=", left,  " rigth neighbor=", right
  call MPI_Barrier ( MPI_COMM_WORLD, ierr )

  do
  
    ! The master reads an integer number form standard input and sends it to all slaves
    if ( myrank .eq. master ) then ! Master part
      write(*,*) 'Number of bytes ? (0 quits, max ', 1024*1024*128, '=128 MB): '
      read (*,*) n
    end if
    call MPI_Bcast ( n, 1, MPI_INTEGER, master, MPI_COMM_WORLD, ierr )
    if ( n .le. 0 .or. n > 1024*1024*128 ) exit

    allocate ( bytes(1:n) )
    bytes="X"

    if ( myrank .eq. 0 ) then ! Master part
      t1 = MPI_Wtime()
      call MPI_Send ( bytes, n, MPI_BYTE, right, tag, MPI_COMM_WORLD, ierr )
      call MPI_Recv ( bytes, n, MPI_BYTE, left, tag, MPI_COMM_WORLD, MPI_STATUS_IGNORE, ierr ) 
      t2 = MPI_Wtime()
      write (*,*) "message length ", n, " bytes   transfer rate ", 1e-6 * n * nprocs / (t2-t1), " MB/s"

    else ! Slave part
      call MPI_Recv ( bytes, n, MPI_BYTE, left, tag, MPI_COMM_WORLD, MPI_STATUS_IGNORE, ierr )
      call MPI_Send ( bytes, n, MPI_BYTE, right, tag, MPI_COMM_WORLD, ierr )
    end if

    deallocate ( bytes )

  end do
     

  !-----------------------------------------------------------

  ! Shut Down MPI
  call MPI_Finalize(ierr )

end program ring
