!! simple point-to-point communication with ..
!! .. MPI_Send and MPI_Recv
program ex2f

  implicit none

  include 'mpif.h'

  integer   ::         myrank, nprocs, n, i, islave, master, count
  integer   ::         status(MPI_STATUS_SIZE)
  integer   ::         ierr, resultlen, tag
  character (LEN=MPI_MAX_PROCESSOR_NAME) :: hostname
  real*8    ::         t0, t1, t2
  
  ! Initialize MPI
  Call MPI_Init(Ierr)

  ! Get my rank id and number of MPI processes
  call MPI_Comm_Rank(MPI_COMM_WORLD, myrank, ierr)
  call MPI_Comm_Size(MPI_COMM_WORLD, nprocs, ierr)
  
  ! Get hostname
  call MPI_Get_processor_name(hostname, resultlen, ierr)
  
  ! Synchonization point (not really necessary here)
  t0 = MPI_Wtime()
  call MPI_Barrier (MPI_COMM_WORLD, ierr)
  t1 = MPI_Wtime()

  !-----------------------------------------------------------
  ! Do work here ...

  ! The master reads an integer number form standard input and sends it to all slaves
  if ( myrank .eq. 0 ) then ! Master part
    write(*,*) "input n:"
    read (*,*) n
    ! Send n to all workers
    tag = 10 ! any message tag is fine
    do islave = 1, nprocs -1
      call MPI_Send ( n, 1, MPI_INTEGER, islave, tag, MPI_COMM_WORLD, ierr )
	   if ( ierr .eq. 0 ) then
             write (*,*) "master successfully sent message to slave ", islave
	   else
             write (*,*) "master sending to slave ", islave, "causes error ", ierr
             call MPI_Abort ( MPI_COMM_WORLD, 99 )
	   end if
    end do 
  
  else ! Slave part
    master = 0
    tag = 10
    call MPI_Recv ( n, 1, MPI_INTEGER, master, tag, MPI_COMM_WORLD, MPI_STATUS_IGNORE, ierr )  ! MPI2
	 if ( ierr .eq. 0 ) then
	   write (*,*) "slave ", myrank, " successfully received a message from master"
	 else
	   write (*,*) "master sending to slave ", islave, "causes error ", ierr
	   call MPI_Abort ( MPI_COMM_WORLD, 99 )
	 end if    
  end if
  
  ! The slave add their rank id to this number and returns the result to the master
  
  if ( myrank .ne. 0 ) then ! Slave part
    n = n + myrank
    master = 0
    tag = 10 + myrank
    call MPI_Send ( n, 1, MPI_INTEGER, master, tag, MPI_COMM_WORLD, ierr )
	 if ( ierr .eq. 0 ) then
	   write (*,*) "slave ", myrank, " successfully sent a message to master"
	 else
	   write (*,*) "slave ", islave, " sending to master causes error ", ierr
	   call MPI_Abort ( MPI_COMM_WORLD, 99 )
	 end if    
  else  ! Master part
    do i = 1, nprocs-1
      call MPI_Recv ( n, 1, MPI_INTEGER, MPI_ANY_SOURCE, MPI_ANY_TAG, MPI_COMM_WORLD, status, ierr ) 
	   if ( ierr .eq. 0 ) then
	     write (*,*) "master successfully received a message from any slave: ", n
	     call MPI_Get_count(status, MPI_INTEGER, count, ierr)
	     write (*,*) "source=", status(MPI_SOURCE), " tag=", status(MPI_TAG), " error=", status(MPI_ERROR), " count=", count
	   else
	     write (*,*) "master receiving a message from any slave causes error ", ierr
	     call MPI_Abort ( MPI_COMM_WORLD, 99 )
	   end if    
    end do
  end if

  !-----------------------------------------------------------

  ! Synchronization point (not really necessary here)
  call MPI_Barrier (MPI_COMM_WORLD, ierr)
  t2 = MPI_Wtime()
  write (*,*) "process ", myrank, ": ", t2-t1, " seconds"

  ! Shut Down MPI
  call MPI_Finalize(ierr )

end program ex2f
